---
id: components-bfcli-technical
---
# BFCLI Technical Documentation

This section provides technical details about the BFCLI architecture,
implementation, and design decisions.

## Architecture

BFCLI follows a modular architecture with these key components:

```
┌───────────────────────────────────────────────────────────┐
│                         BFCLI                             │
├───────────────┬───────────────┬───────────────┬───────────┤
│  Command      │  Config       │  API Client   │           │
│  Registry     │  Manager      │  Manager      │  UI       │
├───────────────┴───────────────┴───────────────┼───────────┤
│                                               │           │
│           Command Implementations             │  Plugins  │
│                                               │           │
└───────────────────────────────────┬───────────┴───────────┘
                                    │
                  ┌─────────────────▼─────────────────┐
                  │           BFAPI                   │
                  └───────────────────────────────────┘
```

### Core Components

- **Command Registry**: Manages the registration and execution of commands
- **Config Manager**: Handles configuration loading, saving, and validation
- **API Client Manager**: Manages API connections to BFAPI and other services
- **UI Components**: Handles user interaction, input, and output formatting
- **Command Implementations**: The actual command logic organized by domains
- **Plugin System**: Enables extensibility through external plugins

## Implementation Details

### Command System

Commands in BFCLI follow a consistent pattern:

```javascript
// Command definition
{
  name: 'command-name',
  description: 'Command description',
  arguments: [
    { name: 'arg1', description: 'Argument 1', required: true },
    { name: 'arg2', description: 'Argument 2', required: false }
  ],
  options: [
    { name: '--option1', alias: '-o', description: 'Option 1' }
  ],
  action: async (args, options) => {
    // Command implementation
    return result;
  }
}
```

Each command is registered with the command registry during initialization.

### Configuration System

Configuration is stored in a JSON file located at:

- `~/.bfcli/config.json` (Linux/macOS)
- `%USERPROFILE%\.bfcli\config.json` (Windows)

The structure is:

```json
{
  "apiUrl": "https://api.bluefly.io",
  "apiKey": "user-api-key",
  "defaultOutput": "table",
  "plugins": {
    "plugin1": { "enabled": true, "options": {} }
  }
}
```

### API Client

BFCLI communicates with BFAPI using axios:

```javascript
const apiClient = axios.create({
  baseURL: config.get('apiUrl'),
  headers: {
    Authorization: `Bearer ${config.get('apiKey')}`,
    'Content-Type': 'application/json',
  },
});
```

### Plugin System

Plugins are loaded from:

- `~/.bfcli/plugins/` (global plugins)
- `./node_modules/` (project-specific plugins)

Each plugin must export:

```javascript
export default {
  name: 'plugin-name',
  version: '1.0.0',
  commands: [],
  hooks: {},
  activate: () => {},
  deactivate: () => {},
};
```

## Build Process

BFCLI is built using:

- TypeScript for type checking
- ES Modules for module system
- Commander.js for command line parsing
- Chalk for terminal styling
- Inquirer for interactive prompts

The build process:

1. Transpiles TypeScript to JavaScript
2. Bundles using esbuild
3. Creates executable script

## Testing Strategy

BFCLI uses Jest for testing:

- **Unit tests**: Test individual components in isolation
- **Integration tests**: Test command execution with mock API responses
- **E2E tests**: Test actual CLI execution against test servers

## Debugging

To enable debug logs:

```bash
# Set environment variable
BFCLI_DEBUG=true bfcli command

# Or use the debug option
bfcli --debug command
```

Debug logs are written to `~/.bfcli/logs/debug.log`.
