---
id: components-bfllm-api-API
---
# BFLLM API Reference

This document provides details about the BFLLM API endpoints and usage.

## Authentication

API requests require authentication using API keys:

```javascript
// Environment variables
CLAUDE_API_KEY = your_claude_api_key;
BLUEFLY_API_KEY = your_bluefly_api_key;
```

## Service Interfaces

### Model Service

The Model Service provides operations for working with AI models.

#### Get Models

```javascript
const models = await modelService.getModels();
```

Returns a list of available models with metadata.

#### Get Model

```javascript
const model = await modelService.getModel(modelId);
```

Returns detailed information about a specific model.

#### Generate

```javascript
const response = await modelService.generate({
  modelId: 'claude-3-sonnet-20240229',
  prompt: 'Write a summary of quantum computing',
  temperature: 0.7,
  maxTokens: 1000,
});
```

Generates a text completion using the specified model.

#### Create Embeddings

```javascript
const embeddings = await modelService.createEmbeddings({
  modelId: 'claude-embedding-3',
  texts: ['First text to embed', 'Second text to embed'],
});
```

Creates vector embeddings for the provided texts.

#### Classify

```javascript
const classification = await modelService.classify({
  modelId: 'claude-3-haiku-20240307',
  text: 'This product is amazing and exceeded my expectations!',
  categories: ['positive', 'neutral', 'negative'],
});
```

Classifies text into provided categories.

### Learning Service

The Learning Service handles model training and feedback.

#### Create Dataset

```javascript
const dataset = await learningService.createDataset({
  name: 'customer-queries',
  description: 'Customer service queries and responses',
  type: 'text-generation',
});
```

Creates a new dataset for training.

#### Add Samples

```javascript
await learningService.addSamples(datasetId, [
  {
    input: 'How do I reset my password?',
    output:
      'You can reset your password by clicking on the "Forgot Password" link on the login page.',
  },
  // More samples...
]);
```

Adds training samples to a dataset.

#### Submit Feedback

```javascript
await learningService.submitFeedback({
  modelId: 'claude-3-opus-20240229',
  input: 'What is machine learning?',
  output: 'Machine learning is a branch of artificial intelligence...',
  rating: 4,
  comments: 'Good explanation but could be more concise',
});
```

Submits user feedback for model improvement.

#### Start Training

```javascript
const jobId = await learningService.startTraining({
  modelId: 'base-model-id',
  datasetIds: ['dataset-123', 'dataset-456'],
  hyperparameters: {
    learningRate: 5e-5,
    epochs: 3,
  },
});
```

Starts a model training job.

#### Get Training Status

```javascript
const status = await learningService.getTrainingStatus(jobId);
```

Gets the status of a training job.

## REST API Endpoints

BFLLM also exposes a RESTful API:

### Model Operations

```
GET /api/models
GET /api/models/{modelId}
POST /api/models/generate
POST /api/models/embeddings
POST /api/models/classify
```

### Learning Operations

```
GET /api/datasets
POST /api/datasets
POST /api/datasets/{datasetId}/samples
POST /api/feedback
POST /api/training
GET /api/training/{jobId}
```

## Error Handling

All API calls may return the following error codes:

- `400`: Bad Request - Invalid parameters
- `401`: Unauthorized - Authentication failed
- `403`: Forbidden - Insufficient permissions
- `404`: Not Found - Resource not found
- `429`: Too Many Requests - Rate limit exceeded
- `500`: Internal Server Error - Server-side error

Error responses include:

```json
{
  "error": {
    "code": "invalid_model_id",
    "message": "The provided model ID is not valid or not available",
    "status": 400
  }
}
```

## Rate Limits

- Default rate limit: 100 requests per minute
- Embedding requests: 50 requests per minute
- Training jobs: 5 jobs per hour

Rate limits can be increased for enterprise customers.
