---
id: demo_consolidated-FIELD_ECA_BRIDGE_TECHNICAL_REFERENCE
---
# Field ECA Bridge + LLM Integration: Technical Reference

## Technical Architecture Deep Dive

### System Components Hierarchy

```
/web/modules/custom/
├── llm/                                 # Core LLM Platform
│   ├── src/
│   │   ├── Plugin/
│   │   │   ├── LlmProcessor/           # AI Processor Plugins
│   │   │   │   ├── OllamaProcessor.php
│   │   │   │   ├── OpenAiProcessor.php
│   │   │   │   └── AnthropicProcessor.php
│   │   │   ├── LlmProcessorBase.php    # Base class for processors
│   │   │   ├── LlmProcessorInterface.php
│   │   │   └── LlmProcessorPluginManager.php
│   │   ├── Service/
│   │   │   ├── LlmPlatformService.php  # Main platform service
│   │   │   ├── LlmTokenTracker.php     # Token usage tracking
│   │   │   └── LlmModelRegistry.php    # Model management
│   │   └── Annotation/
│   │       └── LlmProcessor.php        # Plugin annotation
│   ├── llm.services.yml
│   └── llm.info.yml
│
├── field_eca_bridge/                    # Field-ECA Integration
│   ├── src/
│   │   ├── Plugin/
│   │   │   └── ECA/
│   │   │       └── Action/
│   │   │           └── LlmFieldContentGenerationAction.php
│   │   └── Service/
│   │       ├── FieldTokenManager.php
│   │       └── LlmIntegrationService.php
│   ├── field_eca_bridge.services.yml
│   └── field_eca_bridge.info.yml
│
└── api_normalization/                   # API Integration
    ├── src/
    │   ├── Plugin/
    │   │   ├── Field/
    │   │   └── Transform/
    └── api_normalization.info.yml
```

### Service Dependency Graph

```mermaid
graph TB
    subgraph "Core Services"
        ConfigFactory[config.factory]
        LoggerFactory[logger.factory]
        Cache[cache.default]
        HttpClient[http_client]
    end
    
    subgraph "LLM Services"
        PluginManager[plugin.manager.llm_processor]
        LlmPlatform[llm.platform]
        TokenTracker[llm.token_tracker]
        ModelRegistry[llm.model_registry]
    end
    
    subgraph "Field ECA Services"
        TokenManager[field_eca_bridge.token_manager]
        LlmIntegration[field_eca_bridge.llm_integration]
    end
    
    subgraph "Processors"
        OllamaProc[OllamaProcessor]
        OpenAiProc[OpenAiProcessor]
        AnthropicProc[AnthropicProcessor]
    end
    
    ConfigFactory --> LlmPlatform
    LoggerFactory --> LlmPlatform
    Cache --> LlmPlatform
    PluginManager --> LlmPlatform
    
    HttpClient --> OllamaProc
    HttpClient --> OpenAiProc
    HttpClient --> AnthropicProc
    
    LlmPlatform --> LlmIntegration
    TokenManager --> LlmIntegration
    
    PluginManager --> OllamaProc
    PluginManager --> OpenAiProc
    PluginManager --> AnthropicProc
```

### Database Schema

```sql
-- Token tracking table (created by llm module)
CREATE TABLE `llm_token_usage` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `uid` int(11) NOT NULL DEFAULT '0',
  `processor` varchar(32) NOT NULL,
  `model` varchar(64) NOT NULL,
  `prompt_tokens` int(11) NOT NULL DEFAULT '0',
  `completion_tokens` int(11) NOT NULL DEFAULT '0',
  `total_tokens` int(11) NOT NULL DEFAULT '0',
  `cost` decimal(10,6) DEFAULT NULL,
  `created` int(11) NOT NULL,
  PRIMARY KEY (`id`),
  KEY `uid` (`uid`),
  KEY `processor` (`processor`),
  KEY `created` (`created`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- Cache tables used
-- cache_discovery (for plugin definitions)
-- cache_default (for LLM responses)
-- cache_llm_generated (custom cache bin)
```

### Configuration Schema

```yaml
# config/schema/llm.schema.yml
llm.settings:
  type: config_object
  label: 'LLM settings'
  mapping:
    default_processor:
      type: string
      label: 'Default processor'
    api_keys:
      type: mapping
      label: 'API Keys'
      mapping:
        ollama_base_url:
          type: string
          label: 'Ollama base URL'
        openai_api_key:
          type: string
          label: 'OpenAI API key'
        anthropic_api_key:
          type: string
          label: 'Anthropic API key'
    resource_limits:
      type: mapping
      mapping:
        max_concurrent_requests:
          type: integer
        max_tokens_per_request:
          type: integer
        timeout_seconds:
          type: integer

# config/schema/field_eca_bridge.schema.yml
field_eca_bridge.settings:
  type: config_object
  mapping:
    enable_debug_logging:
      type: boolean
    token_storage_method:
      type: string
    cache_generated_content:
      type: boolean
```

## Code Architecture Patterns

### Plugin Discovery System

```php
namespace Drupal\llm\Plugin;

use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;

/**
 * LLM Processor plugin manager.
 */
class LlmProcessorPluginManager extends DefaultPluginManager {

  public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    parent::__construct(
      'Plugin/LlmProcessor',
      $namespaces,
      $module_handler,
      'Drupal\llm\Plugin\LlmProcessorInterface',
      'Drupal\llm\Annotation\LlmProcessor'
    );

    $this->alterInfo('llm_processor_info');
    $this->setCacheBackend($cache_backend, 'llm_processor_plugins');
  }

  /**
   * Gets enabled processors sorted by weight.
   */
  public function getEnabledProcessors(): array {
    $definitions = $this->getDefinitions();
    $enabled = [];

    foreach ($definitions as $plugin_id => $definition) {
      if ($definition['enabled'] ?? TRUE) {
        $enabled[$plugin_id] = $definition;
      }
    }

    // Sort by weight (highest first)
    uasort($enabled, function ($a, $b) {
      $weight_a = $a['weight'] ?? 0;
      $weight_b = $b['weight'] ?? 0;
      return $weight_b <=> $weight_a;
    });

    return $enabled;
  }
}
```

### Service Implementation Pattern

```php
namespace Drupal\llm\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\llm\Plugin\LlmProcessorPluginManager;

/**
 * Main LLM platform service.
 */
class LlmPlatformService implements LlmPlatformInterface {
  
  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;
  
  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;
  
  /**
   * The cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;
  
  /**
   * The processor plugin manager.
   *
   * @var \Drupal\llm\Plugin\LlmProcessorPluginManager
   */
  protected $processorManager;
  
  /**
   * Cached processor instances.
   *
   * @var array
   */
  protected $processors = [];

  public function __construct(
    ConfigFactoryInterface $config_factory,
    LoggerChannelFactoryInterface $logger_factory,
    CacheBackendInterface $cache,
    LlmProcessorPluginManager $processor_manager
  ) {
    $this->configFactory = $config_factory;
    $this->loggerFactory = $logger_factory;
    $this->cache = $cache;
    $this->processorManager = $processor_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function chat(string $prompt, array $options = []): array {
    // Get processor
    $processor_id = $options['processor'] ?? $this->getDefaultProcessor();
    $processor = $this->getProcessor($processor_id);
    
    if (!$processor) {
      return [
        'status' => 'error',
        'error_message' => sprintf('Processor "%s" not found', $processor_id),
      ];
    }
    
    // Check cache
    $cache_key = $this->getCacheKey($prompt, $options);
    if ($cached = $this->cache->get($cache_key)) {
      return $cached->data;
    }
    
    // Process prompt
    try {
      $result = $processor->processPrompt($prompt, $options);
      
      // Cache successful results
      if ($result['status'] === 'success') {
        $this->cache->set($cache_key, $result, time() + 3600);
      }
      
      return $result;
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('llm')->error('Chat error: @message', [
        '@message' => $e->getMessage(),
      ]);
      
      return [
        'status' => 'error',
        'error_message' => $e->getMessage(),
      ];
    }
  }
  
  /**
   * Gets a processor instance.
   */
  protected function getProcessor(string $processor_id) {
    if (!isset($this->processors[$processor_id])) {
      try {
        $this->processors[$processor_id] = $this->processorManager->createInstance($processor_id);
      }
      catch (\Exception $e) {
        return NULL;
      }
    }
    
    return $this->processors[$processor_id];
  }
}
```

### ECA Action Implementation

```php
namespace Drupal\field_eca_bridge\Plugin\ECA\Action;

use Drupal\Core\Entity\EntityInterface;
use Drupal\eca\Plugin\ECA\Action\ActionBase;
use Drupal\field_eca_bridge\Plugin\ECA\TokenFieldAccessTrait;

/**
 * @EcaAction(
 *   id = "llm_field_content_generation",
 *   label = @Translation("LLM field content generation"),
 *   description = @Translation("Generates content for a field using LLM.")
 * )
 */
class LlmFieldContentGenerationAction extends ActionBase {
  use TokenFieldAccessTrait;

  /**
   * {@inheritdoc}
   */
  public function execute(): bool {
    // Get entity from token context
    $entity = $this->getEntity();
    if (!$entity) {
      return FALSE;
    }

    // Get target field
    $target_field = $this->tokenServices->replaceClear($this->configuration['target_field']);
    
    // Check field exists
    if (!$entity->hasField($target_field)) {
      return FALSE;
    }

    // Build requirements
    $requirements = $this->buildRequirements($entity);
    
    // Generate content
    $result = $this->llmIntegration->generateFieldContent(
      $entity->get($target_field)->getFieldDefinition()->getType(),
      $requirements
    );

    // Set field value if successful
    if ($result['success']) {
      $this->setFieldValue($entity, $target_field, $result['content']);
      
      // Save entity if configured
      if ($this->configuration['save_entity']) {
        $entity->save();
      }
      
      // Store in token if configured
      if ($this->configuration['store_content']) {
        $this->tokenServices->addTokenData(
          $this->configuration['content_token'],
          $result['content']
        );
      }
      
      return TRUE;
    }

    return FALSE;
  }
}
```

## Integration Points

### 1. Drupal Form Integration

```php
/**
 * Implements hook_form_FORM_ID_alter() for node forms.
 */
function llm_form_node_form_alter(&$form, FormStateInterface $form_state, $form_id) {
  // Add AI generation button to title field
  if (isset($form['title'])) {
    $form['title']['widget'][0]['#suffix'] = '<button type="button" class="llm-generate-title">Generate with AI</button>';
    $form['#attached']['library'][] = 'llm/field-generation';
  }
}

// JavaScript library (llm.libraries.yml)
field-generation:
  js:
    js/field-generation.js: {}
  dependencies:
    - core/jquery
    - core/drupal.ajax
```

### 2. Views Integration

```php
/**
 * Implements hook_views_data_alter().
 */
function llm_views_data_alter(array &$data) {
  // Add AI-generated content relationship
  $data['node']['llm_generated_summary'] = [
    'title' => t('AI Generated Summary'),
    'field' => [
      'id' => 'llm_generated_field',
      'click sortable' => FALSE,
    ],
  ];
}

// Views field plugin
namespace Drupal\llm\Plugin\views\field;

/**
 * @ViewsField("llm_generated_field")
 */
class LlmGeneratedField extends FieldPluginBase {
  
  public function render(ResultRow $values) {
    $entity = $this->getEntity($values);
    
    // Generate summary on-the-fly
    $llm = \Drupal::service('llm.platform');
    $result = $llm->chat('Summarize: ' . $entity->label(), [
      'processor' => 'ollama',
      'model' => 'llama2:7b',
      'max_tokens' => 100,
    ]);
    
    return $result['status'] === 'success' ? $result['content'] : '';
  }
}
```

### 3. REST API Integration

```php
namespace Drupal\llm\Plugin\rest\resource;

use Drupal\rest\Plugin\ResourceBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * @RestResource(
 *   id = "llm_generate",
 *   label = @Translation("LLM Generation"),
 *   uri_paths = {
 *     "create" = "/api/llm/generate"
 *   }
 * )
 */
class LlmGenerateResource extends ResourceBase {
  
  public function post(array $data) {
    // Validate request
    if (empty($data['prompt'])) {
      return new JsonResponse(['error' => 'Prompt required'], 400);
    }
    
    // Check permissions
    if (!$this->currentUser->hasPermission('use llm generation')) {
      return new JsonResponse(['error' => 'Access denied'], 403);
    }
    
    // Generate content
    $result = $this->llmPlatform->chat($data['prompt'], $data['options'] ?? []);
    
    // Return response
    return new JsonResponse($result);
  }
}
```

### 4. Drush Commands

```php
namespace Drupal\llm\Commands;

use Drush\Commands\DrushCommands;

/**
 * LLM Drush commands.
 */
class LlmCommands extends DrushCommands {
  
  /**
   * Generate content using LLM.
   *
   * @command llm:generate
   * @param string $prompt The prompt to send
   * @option processor The processor to use
   * @option model The model to use
   * @usage llm:generate "Write a haiku about Drupal"
   */
  public function generate($prompt, $options = ['processor' => 'ollama', 'model' => 'llama2:7b']) {
    $result = \Drupal::service('llm.platform')->chat($prompt, $options);
    
    if ($result['status'] === 'success') {
      $this->output()->writeln($result['content']);
      $this->output()->writeln('');
      $this->output()->writeln("Tokens used: {$result['usage']['total_tokens']}");
    } else {
      $this->output()->writeln("<error>Error: {$result['error_message']}</error>");
    }
  }
}
```

## Performance Tuning

### Connection Pooling

```php
// llm.services.yml
services:
  llm.shared_http_client:
    class: GuzzleHttp\Client
    arguments:
      - 
        timeout: 60
        connect_timeout: 10
        pool_size: 10
        keep_alive: true
        headers:
          User-Agent: 'Drupal LLM Platform/1.0'
```

### Async Processing

```php
use GuzzleHttp\Promise;

class AsyncLlmProcessor {
  
  public function processMultiplePrompts(array $prompts) {
    $promises = [];
    
    foreach ($prompts as $key => $prompt) {
      $promises[$key] = $this->client->postAsync('/api/generate', [
        'json' => [
          'model' => 'llama2:7b',
          'prompt' => $prompt,
          'stream' => false,
        ],
      ]);
    }
    
    // Wait for all promises to complete
    $responses = Promise\settle($promises)->wait();
    
    $results = [];
    foreach ($responses as $key => $response) {
      if ($response['state'] === 'fulfilled') {
        $results[$key] = json_decode($response['value']->getBody(), TRUE);
      } else {
        $results[$key] = ['error' => $response['reason']->getMessage()];
      }
    }
    
    return $results;
  }
}
```

### Cache Warming

```php
/**
 * Implements hook_cron().
 */
function llm_cron() {
  // Pre-generate common content
  $common_prompts = [
    'default_page_title' => 'Generate a default page title',
    'default_meta_description' => 'Generate a default meta description',
  ];
  
  $llm = \Drupal::service('llm.platform');
  $cache = \Drupal::cache('llm_generated');
  
  foreach ($common_prompts as $key => $prompt) {
    $cache_key = 'llm:preset:' . $key;
    
    if (!$cache->get($cache_key)) {
      $result = $llm->chat($prompt, ['processor' => 'ollama']);
      
      if ($result['status'] === 'success') {
        $cache->set($cache_key, $result, CacheBackendInterface::CACHE_PERMANENT);
      }
    }
  }
}
```

## Monitoring and Debugging

### Debug Mode

```php
// Enable debug mode in settings.php
$config['llm.settings']['debug_mode'] = TRUE;
$config['llm.settings']['log_all_requests'] = TRUE;
$config['llm.settings']['log_all_responses'] = TRUE;
```

### Performance Monitoring

```php
class LlmPerformanceMonitor {
  
  protected $metrics = [];
  
  public function startTimer($operation) {
    $this->metrics[$operation] = [
      'start' => microtime(TRUE),
      'memory_start' => memory_get_usage(),
    ];
  }
  
  public function endTimer($operation) {
    if (!isset($this->metrics[$operation])) {
      return;
    }
    
    $metrics = &$this->metrics[$operation];
    $metrics['duration'] = microtime(TRUE) - $metrics['start'];
    $metrics['memory_used'] = memory_get_usage() - $metrics['memory_start'];
    
    // Log to watchdog
    \Drupal::logger('llm_performance')->info('@operation completed', [
      '@operation' => $operation,
      '@duration' => $metrics['duration'],
      '@memory' => format_size($metrics['memory_used']),
    ]);
    
    // Send to monitoring service
    if (\Drupal::moduleHandler()->moduleExists('monitoring')) {
      monitoring_sensor_run('llm_operation_time', $metrics['duration']);
    }
  }
}
```

### Error Tracking

```php
class LlmErrorTracker {
  
  public function trackError($processor, $error, $context = []) {
    $data = [
      'processor' => $processor,
      'error' => $error,
      'context' => $context,
      'timestamp' => time(),
      'user' => \Drupal::currentUser()->id(),
    ];
    
    // Log to database
    \Drupal::database()->insert('llm_errors')
      ->fields($data)
      ->execute();
    
    // Send alert if error rate is high
    $error_count = \Drupal::database()->select('llm_errors', 'e')
      ->condition('timestamp', time() - 300, '>')
      ->condition('processor', $processor)
      ->countQuery()
      ->execute()
      ->fetchField();
    
    if ($error_count > 10) {
      // Send alert
      \Drupal::logger('llm')->critical('High error rate for @processor', [
        '@processor' => $processor,
      ]);
    }
  }
}
```

## Testing

### Unit Tests

```php
namespace Drupal\Tests\llm\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\llm\Plugin\LlmProcessor\OllamaProcessor;

/**
 * @coversDefaultClass \Drupal\llm\Plugin\LlmProcessor\OllamaProcessor
 * @group llm
 */
class OllamaProcessorTest extends UnitTestCase {
  
  /**
   * @covers ::processPrompt
   */
  public function testProcessPrompt() {
    // Mock dependencies
    $http_client = $this->prophesize(ClientInterface::class);
    $response = $this->prophesize(ResponseInterface::class);
    
    $response->getBody()->willReturn(json_encode([
      'response' => 'Test response',
      'model' => 'llama2:7b',
      'eval_count' => 10,
    ]));
    
    $http_client->post(Argument::any(), Argument::any())
      ->willReturn($response->reveal());
    
    // Create processor
    $processor = new OllamaProcessor(
      [],
      'ollama',
      ['models' => ['llama2:7b' => []]],
      $this->prophesize(CacheBackendInterface::class)->reveal(),
      $this->prophesize(ConfigFactoryInterface::class)->reveal(),
      $this->prophesize(LoggerChannelFactoryInterface::class)->reveal(),
      $http_client->reveal()
    );
    
    // Test processing
    $result = $processor->processPrompt('Test prompt');
    
    $this->assertEquals('success', $result['status']);
    $this->assertEquals('Test response', $result['content']);
    $this->assertEquals(10, $result['usage']['total_tokens']);
  }
}
```

### Functional Tests

```php
namespace Drupal\Tests\field_eca_bridge\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\node\Entity\Node;

/**
 * Tests Field ECA Bridge integration.
 *
 * @group field_eca_bridge
 */
class FieldEcaBridgeTest extends BrowserTestBase {
  
  protected static $modules = [
    'llm',
    'field_eca_bridge',
    'eca',
    'node',
  ];
  
  /**
   * Tests AI field generation.
   */
  public function testFieldGeneration() {
    // Create test node
    $node = Node::create([
      'type' => 'article',
      'title' => '',
      'body' => [
        'value' => 'Test article content about Drupal and AI integration.',
      ],
    ]);
    
    // Trigger ECA workflow
    $node->save();
    
    // Reload node
    $node = Node::load($node->id());
    
    // Assert title was generated
    $this->assertNotEmpty($node->getTitle());
    $this->assertStringContainsString('AI', $node->getTitle());
  }
}
```

### Integration Tests

```php
namespace Drupal\Tests\llm\Integration;

use Drupal\KernelTests\KernelTestBase;

/**
 * Tests LLM platform integration.
 *
 * @group llm
 */
class LlmPlatformIntegrationTest extends KernelTestBase {
  
  protected static $modules = ['llm'];
  
  /**
   * Tests plugin discovery.
   */
  public function testPluginDiscovery() {
    $plugin_manager = \Drupal::service('plugin.manager.llm_processor');
    $definitions = $plugin_manager->getDefinitions();
    
    // Should have at least Ollama processor
    $this->assertArrayHasKey('ollama', $definitions);
    $this->assertEquals('Ollama Local Models', $definitions['ollama']['label']);
  }
  
  /**
   * Tests service integration.
   */
  public function testServiceIntegration() {
    $llm_platform = \Drupal::service('llm.platform');
    
    // Test available processors
    $processors = $llm_platform->getAvailableProcessors();
    $this->assertNotEmpty($processors);
    
    // Test chat method
    $result = $llm_platform->chat('Hello', [
      'processor' => 'ollama',
      'model' => 'llama2:7b',
    ]);
    
    $this->assertArrayHasKey('status', $result);
  }
}
```

## Deployment

### Production Configuration

```yaml
# config/prod/llm.settings.yml
default_processor: 'ollama'
api_keys:
  ollama_base_url: 'http://ollama-prod.internal:11434'
resource_limits:
  max_concurrent_requests: 10
  max_tokens_per_request: 4096
  timeout_seconds: 60
  retry_attempts: 3
performance:
  enable_caching: true
  cache_ttl: 3600
  batch_size: 20
  async_processing: true
monitoring:
  track_usage: true
  alert_on_errors: true
  error_threshold: 10
  alert_email: 'ops@example.com'
```

### Docker Deployment

```dockerfile
# Dockerfile for Ollama service
FROM ollama/ollama:latest

# Pre-download models
RUN ollama pull llama2:7b && \
    ollama pull llama2:13b && \
    ollama pull codellama:7b && \
    ollama pull mistral:7b

# Health check
HEALTHCHECK --interval=30s --timeout=3s --start-period=5s --retries=3 \
  CMD curl -f http://localhost:11434/api/tags || exit 1

EXPOSE 11434

CMD ["serve"]
```

### Kubernetes Deployment

```yaml
# ollama-deployment.yaml
apiVersion: apps/v1
kind: Deployment
metadata:
  name: ollama
spec:
  replicas: 3
  selector:
    matchLabels:
      app: ollama
  template:
    metadata:
      labels:
        app: ollama
    spec:
      containers:
      - name: ollama
        image: ollama/ollama:latest
        ports:
        - containerPort: 11434
        resources:
          requests:
            memory: "8Gi"
            cpu: "4"
          limits:
            memory: "16Gi"
            cpu: "8"
        livenessProbe:
          httpGet:
            path: /api/tags
            port: 11434
          initialDelaySeconds: 30
          periodSeconds: 10
---
apiVersion: v1
kind: Service
metadata:
  name: ollama-service
spec:
  selector:
    app: ollama
  ports:
    - protocol: TCP
      port: 11434
      targetPort: 11434
  type: LoadBalancer
```

---

**Technical Reference Version**: 1.0  
**Last Updated**: January 10, 2025  
**Architecture Status**: Production Ready
