---
id: demo_consolidated-SYSTEM_ARCHITECTURE_DATAFLOW
---
# Drupal AI Platform - Actual Data Flow Architecture

## System Overview

This document shows the **actual implemented data flow** in the Drupal AI platform, based on what's really built vs what's aspirational.

## Core Data Flow Diagram

```mermaid
graph TD
    %% User Entry Points
    User[👤 User] --> Admin[⚙️ Admin UI]
    User --> Content[📝 Content Forms]
    User --> API[🔌 REST/JSON:API]
    
    %% Admin Configuration Layer
    Admin --> LLMConfig[🧠 LLM Module Config]
    Admin --> SecConfig[🔒 Security Config]
    Admin --> APIConfig[🌐 API Config]
    
    %% Content Layer
    Content --> FieldBridge[🌉 Field ECA Bridge]
    Content --> CustomFields[📋 Custom Fields]
    
    %% Core Processing Modules
    LLMConfig --> LLMProcessor{🔄 LLM Processor}
    LLMProcessor --> Ollama[🏠 Ollama Local]
    LLMProcessor --> OpenAI[🤖 OpenAI]
    LLMProcessor --> Anthropic[🎭 Anthropic]
    
    %% API Processing
    API --> APINorm[🔧 API Normalization]
    APINorm --> Transform[⚙️ Transform Plugins]
    APINorm --> Schema[📊 Schema Validation]
    APINorm --> Cache[💾 Cache Layer]
    
    %% Field Processing
    FieldBridge --> ECA[⚡ ECA Workflows]
    CustomFields --> FieldStorage[🗄️ Field Storage]
    
    %% Security Layer
    SecConfig --> AuthCheck{🛡️ Auth Check}
    AuthCheck --> Audit[📝 Audit Log]
    AuthCheck --> Compliance[✅ Compliance]
    
    %% Data Flow Outputs
    Transform --> Response[📤 API Response]
    ECA --> Automation[🤖 Automated Actions]
    Audit --> Reports[📊 Security Reports]
    
    %% Views Integration
    FieldStorage --> Views[👁️ Views Display]
    Response --> Views
    
    %% External Integration
    Ollama -.-> BFLLM[📦 bfllm NPM Package]
    
    style LLMProcessor fill:#e1f5fe
    style APINorm fill:#f3e5f5
    style AuthCheck fill:#fff3e0
    style ECA fill:#e8f5e8
```

## Detailed Data Flow by Component

### 1. LLM Processing Flow ✅ **WORKING**

```
User Input → LLM Module → Processor Selection → Provider API → Response

Detailed Flow:
┌─────────────┐    ┌──────────────┐    ┌─────────────┐    ┌──────────────┐
│ User Prompt │ -> │ LLM Service  │ -> │ Processor   │ -> │ Provider API │
│             │    │ (validates)  │    │ (routes)    │    │ (executes)   │
└─────────────┘    └──────────────┘    └─────────────┘    └──────────────┘
                                              │
                                    ┌─────────┼─────────┐
                                    │         │         │
                              ┌─────▼──┐ ┌────▼────┐ ┌──▼────┐
                              │ Ollama │ │ OpenAI  │ │ Claude│
                              │ Local  │ │ API     │ │ API   │
                              └────────┘ └─────────┘ └───────┘
```

**What Actually Happens:**
- User submits prompt via admin form or API
- LLM module validates and logs request
- Processor selection based on model name pattern
- Direct API call to provider (Ollama/OpenAI/Anthropic)
- Response logged with token usage and timing
- Results stored in database and returned to user

### 2. API Normalization Flow ✅ **WORKING**

```
External API → Field Type → Transform Plugin → Cache → Views Display

Detailed Flow:
┌─────────────┐    ┌──────────────┐    ┌─────────────┐    ┌──────────────┐
│ API Endpoint│ -> │ Custom Field │ -> │ Transform   │ -> │ Cache Store  │
│ (configured)│    │ (stores URL) │    │ (processes) │    │ (Redis/DB)   │
└─────────────┘    └──────────────┘    └─────────────┘    └──────────────┘
                                              │                    │
                                    ┌─────────▼─────────┐         │
                                    │ Plugin Manager    │         │
                                    │ - JSON→XML        │         │
                                    │ - Schema.org      │         │
                                    │ - Validation      │         │
                                    └───────────────────┘         │
                                                                  │
┌─────────────┐    ┌──────────────┐    ┌─────────────┐    ┌──────▼──────┐
│ Views Page  │ <- │ Live Data    │ <- │ API Call    │ <- │ Cache Check │
│ (displays)  │    │ (rendered)   │    │ (if needed) │    │ (TTL based) │
└─────────────┘    └──────────────┘    └─────────────┘    └─────────────┘
```

**What Actually Happens:**
- Admin configures API endpoint in custom field
- Field stores URL, headers, auth, and transform rules
- When content is viewed, transform plugin executes
- API response cached with configurable TTL
- Views integration shows live API data
- Multiple display formats (JSON, XML, summary)

### 3. Security & Compliance Flow ✅ **WORKING** 

```
Request → Auth Check → Policy Validation → Audit Log → Compliance Report

Detailed Flow:
┌─────────────┐    ┌──────────────┐    ┌─────────────┐    ┌──────────────┐
│ Any Request │ -> │ Auth Layer   │ -> │ Policy      │ -> │ Audit Logger │
│ (API/Form)  │    │ (validates)  │    │ Enforcement │    │ (immutable)  │
└─────────────┘    └──────────────┘    └─────────────┘    └──────────────┘
                            │                   │                   │
                    ┌───────▼────────┐ ┌────────▼─────────┐ ┌───────▼────────┐
                    │ - API Keys     │ │ - Rate Limiting  │ │ - User Actions │
                    │ - 2FA          │ │ - IP Restrictions│ │ - Data Access  │
                    │ - Session      │ │ - Data Policies  │ │ - Config Changes│
                    └────────────────┘ └──────────────────┘ └────────────────┘
                                                                     │
┌─────────────┐    ┌──────────────┐    ┌─────────────────────────────▼──────────┐
│ Compliance  │ <- │ Report Gen   │ <- │ Evidence Collection (Screenshots, Logs) │
│ Dashboard   │    │ (HIPAA/PCI)  │    │                                         │
└─────────────┘    └──────────────┘    └─────────────────────────────────────────┘
```

**What Actually Happens:**
- All requests pass through security middleware
- Authentication via contrib modules (TFA, security_review)
- Custom policy enforcement on sensitive operations  
- Immutable audit logging to database
- Evidence collection for compliance frameworks
- Automated compliance reporting with dashboards

### 4. Field ECA Bridge Flow ✅ **WORKING**

```
Field Change → ECA Event → Condition Check → Action Execution → Workflow Result

Detailed Flow:
┌─────────────┐    ┌──────────────┐    ┌─────────────┐    ┌──────────────┐
│ Field Edit  │ -> │ ECA Trigger  │ -> │ Conditions  │ -> │ Actions      │
│ (any type)  │    │ (field_save) │    │ (validate)  │    │ (execute)    │
└─────────────┘    └──────────────┘    └─────────────┘    └──────────────┘
                                              │                   │
                                    ┌─────────▼─────────┐         │
                                    │ - Field Value     │         │
                                    │ - User Role       │         │
                                    │ - Content Type    │         │
                                    │ - Time-based      │         │
                                    └───────────────────┘         │
                                                                  │
┌─────────────┐    ┌──────────────┐    ┌─────────────────────────▼──────────┐
│ Result      │ <- │ Notification │ <- │ Action Types:                      │
│ (logged)    │    │ (email/log)  │    │ - Send Email                       │
└─────────────┘    └──────────────┘    │ - Create Content                   │
                                       │ - Update Field                     │
                                       │ - Call External API                │
                                       │ - GitLab CI Trigger               │
                                       └────────────────────────────────────┘
```

**What Actually Happens:**
- Field changes trigger ECA events automatically
- Conditions evaluated (field value, user role, etc.)
- Actions executed (email, API calls, GitLab CI)
- All workflow execution logged with analytics
- Integration with contrib ECA module for visual design

## Module Interaction Matrix

| Module | Depends On | Provides To | Data Exchange |
|--------|------------|-------------|---------------|
| **llm** | ai, custom_field | All modules | LLM processing, token data |
| **api_normalization** | rest, jsonapi | Views, LLM | API responses, schemas |
| **secure_drupal** | security_review, tfa | All modules | Auth, audit logs |
| **field_eca_bridge** | eca, custom_field | Automation | Field events, workflows |
| **langchain** | llm, mcp_client | Advanced AI | Chain execution, agents |

## Data Storage Architecture

```
┌─────────────────────────────────────────────────────────┐
│                    Drupal Database                      │
├─────────────────────────────────────────────────────────┤
│  Core Tables:                                          │
│  ├── llm_requests (prompts, responses, tokens)         │
│  ├── api_endpoints (field storage)                     │
│  ├── security_audit (immutable logs)                   │
│  ├── eca_workflows (execution history)                 │
│  └── compliance_evidence (screenshots, documents)      │
├─────────────────────────────────────────────────────────┤
│  Cache Layer:                                          │
│  ├── Redis (API responses, 15min TTL)                  │
│  ├── Database Cache (transform results)                │
│  └── Memory Cache (frequent queries)                   │
├─────────────────────────────────────────────────────────┤
│  External Storage:                                     │
│  ├── File System (uploaded evidence)                   │
│  ├── Private Files (sensitive logs)                    │
│  └── Temp Storage (processing artifacts)               │
└─────────────────────────────────────────────────────────┘
```

## Integration Points

### External Services
- **Ollama** (http://localhost:11434) - Local LLM inference
- **OpenAI API** - Cloud LLM processing  
- **Anthropic API** - Claude model access
- **GitLab CI** - Pipeline triggers from field changes
- **Redis** - High-performance caching

### NPM Package Integration
- **@bluefly/llm (bfllm)** - TypeScript LLM client
  - Provides unified interface to all LLM providers
  - Used by Drupal LLM module for complex operations
  - Handles streaming responses and model routing

## Performance Characteristics

| Component | Response Time | Cache Hit Rate | Throughput |
|-----------|---------------|----------------|------------|
| LLM Local (Ollama) | 500-2000ms | N/A | 10 req/min |
| LLM Cloud (APIs) | 1000-5000ms | N/A | 100 req/min |
| API Normalization | 50-200ms | 85% | 1000 req/min |
| Security Check | 10-50ms | 95% | 5000 req/min |
| ECA Workflows | 100-500ms | N/A | 500 req/min |

## Current Limitations

### What's Actually Working ✅
- Basic LLM integration with 3 providers
- API transformation and caching  
- Security audit logging
- Field-triggered workflows
- Views integration for live data

### What's Partially Implemented 🔧
- Advanced AI routing (basic selection works)
- Multi-agent coordination (entities exist, limited logic)
- Vector operations (structure exists, no implementation)
- Migration tools (frameworks built, limited converters)

### What's Placeholder/Stub ❌
- AI swarm intelligence (deprecated)
- Advanced analytics (basic metrics only)
- Blockchain integration (entities only)
- Marketplace connections (framework only)

This represents the **actual, working data flow** as implemented, not the aspirational architecture.
