---
id: drupal_native_ai_nativeaijs-LOGS_API
---
# Native AI.js Logs API Documentation

This document outlines the API endpoints for accessing and managing logs in the Native AI.js module for Drupal.

## Overview

The Logs API provides access to AI model interaction logs, allowing developers and administrators to:
- Retrieve logs of AI model interactions
- Access detailed information about specific log entries
- Obtain statistical information about model usage and performance

All endpoints return data in JSON format and require proper authentication.

## Authentication

All API endpoints require authentication using JWT (JSON Web Token). Include the token in the Authorization header:

```
Authorization: Bearer {your_jwt_token}
```

To obtain a token, authenticate via the Drupal REST API or use the `/api/native-aijs/auth/token` endpoint.

## Rate Limiting

API requests are limited to 100 requests per minute per authenticated user. Exceeding this limit will result in HTTP 429 (Too Many Requests) responses.

## Caching

- GET requests are cached for 5 minutes by default
- Cache can be bypassed by including `?_no_cache=1` in the request URL
- Statistics endpoint responses are cached for 15 minutes

## Endpoints

### List Logs

Retrieves a paginated list of AI model interaction logs.

**URL**: `/api/native-aijs/logs`

**Method**: `GET`

**URL Parameters**:

| Parameter    | Type    | Description                             | Default | Example         |
|--------------|---------|----------------------------------------|---------|-----------------|
| `page`       | integer | Page number for pagination              | 1       | `?page=2`       |
| `limit`      | integer | Number of records per page (max 100)    | 25      | `?limit=50`     |
| `model_id`   | string  | Filter by specific model ID             | -       | `?model_id=gpt4` |
| `status`     | string  | Filter by status (success, error, etc.) | -       | `?status=error` |
| `start_date` | string  | Filter logs after this date (ISO 8601)  | -       | `?start_date=2025-05-01T00:00:00Z` |
| `end_date`   | string  | Filter logs before this date (ISO 8601) | -       | `?end_date=2025-05-31T23:59:59Z` |
| `user_id`    | integer | Filter logs by user ID                  | -       | `?user_id=42`   |
| `sort`       | string  | Field to sort by                        | created | `?sort=tokens`  |
| `direction`  | string  | Sort direction (asc or desc)            | desc    | `?direction=asc` |

**Response**:

```json
{
  "data": [
    {
      "id": "123",
      "model_id": "claude-3-opus",
      "user_id": 42,
      "request": {
        "prompt": "Summarize the key points of the article",
        "options": {
          "temperature": 0.7,
          "max_tokens": 500
        }
      },
      "response": {
        "content": "The article discusses several key points including...",
        "finish_reason": "stop"
      },
      "tokens": {
        "prompt": 32,
        "completion": 128,
        "total": 160
      },
      "duration_ms": 1250,
      "status": "success",
      "created": "2025-05-15T14:22:36Z",
      "metadata": {
        "source": "content_editor",
        "session_id": "abcd1234"
      }
    },
    // Additional log entries...
  ],
  "pagination": {
    "total": 1358,
    "page": 1,
    "limit": 25,
    "pages": 55
  }
}
```

**Status Codes**:

- `200 OK`: Successfully retrieved logs
- `400 Bad Request`: Invalid parameters
- `401 Unauthorized`: Missing or invalid authentication
- `403 Forbidden`: Insufficient permissions
- `429 Too Many Requests`: Rate limit exceeded

### Get Log Details

Retrieves detailed information about a specific log entry.

**URL**: `/api/native-aijs/logs/{id}`

**Method**: `GET`

**URL Parameters**: None

**Path Parameters**:

| Parameter | Type   | Description           | Example |
|-----------|--------|-----------------------|---------|
| `id`      | string | Unique ID of the log  | `123`   |

**Response**:

```json
{
  "data": {
    "id": "123",
    "model_id": "claude-3-opus",
    "user_id": 42,
    "request": {
      "prompt": "Summarize the key points of the article",
      "options": {
        "temperature": 0.7,
        "max_tokens": 500
      },
      "context": {
        "article_id": "1542",
        "article_title": "Latest Research on AI Development"
      }
    },
    "response": {
      "content": "The article discusses several key points including...",
      "finish_reason": "stop",
      "raw_response": {
        // Full API response
      }
    },
    "tokens": {
      "prompt": 32,
      "completion": 128,
      "total": 160
    },
    "duration_ms": 1250,
    "status": "success",
    "created": "2025-05-15T14:22:36Z",
    "metadata": {
      "source": "content_editor",
      "session_id": "abcd1234",
      "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7)...",
      "ip_address": "192.168.1.1"
    },
    "error": null
  }
}
```

**Status Codes**:

- `200 OK`: Successfully retrieved log details
- `401 Unauthorized`: Missing or invalid authentication
- `403 Forbidden`: Insufficient permissions
- `404 Not Found`: Log entry not found
- `429 Too Many Requests`: Rate limit exceeded

### Get Log Statistics

Retrieves aggregated statistics about AI model usage and performance.

**URL**: `/api/native-aijs/logs/statistics`

**Method**: `GET`

**URL Parameters**:

| Parameter    | Type    | Description                             | Default | Example         |
|--------------|---------|----------------------------------------|---------|-----------------|
| `start_date` | string  | Start date for statistics (ISO 8601)    | -30 days | `?start_date=2025-05-01T00:00:00Z` |
| `end_date`   | string  | End date for statistics (ISO 8601)      | now     | `?end_date=2025-05-31T23:59:59Z` |
| `model_id`   | string  | Filter by specific model ID             | -       | `?model_id=gpt4` |
| `user_id`    | integer | Filter statistics by user ID            | -       | `?user_id=42`   |
| `group_by`   | string  | Group statistics by field (day, model, user) | day | `?group_by=model` |

**Response**:

```json
{
  "data": {
    "total_requests": 12583,
    "total_tokens": 2456890,
    "average_duration_ms": 980,
    "success_rate": 98.7,
    "error_rate": 1.3,
    "total_users": 53,
    "breakdown_by_model": [
      {
        "model_id": "claude-3-opus",
        "requests": 5621,
        "tokens": 1235690,
        "average_duration_ms": 1250
      },
      {
        "model_id": "claude-3-sonnet",
        "requests": 4892,
        "tokens": 856432,
        "average_duration_ms": 850
      },
      {
        "model_id": "gpt-4",
        "requests": 2070,
        "tokens": 364768,
        "average_duration_ms": 920
      }
    ],
    "timeline": [
      {
        "date": "2025-05-01",
        "requests": 423,
        "tokens": 84600,
        "average_duration_ms": 950
      },
      // Additional days...
    ],
    "most_active_users": [
      {
        "user_id": 12,
        "requests": 1542,
        "tokens": 308400
      },
      // Additional users...
    ]
  }
}
```

**Status Codes**:

- `200 OK`: Successfully retrieved statistics
- `400 Bad Request`: Invalid parameters
- `401 Unauthorized`: Missing or invalid authentication
- `403 Forbidden`: Insufficient permissions
- `429 Too Many Requests`: Rate limit exceeded

## Examples

### cURL Examples

#### List Logs

```bash
curl -X GET "https://example.com/api/native-aijs/logs?limit=10&status=success" \
  -H "Authorization: Bearer YOUR_JWT_TOKEN" \
  -H "Accept: application/json"
```

#### Get Log Details

```bash
curl -X GET "https://example.com/api/native-aijs/logs/123" \
  -H "Authorization: Bearer YOUR_JWT_TOKEN" \
  -H "Accept: application/json"
```

#### Get Log Statistics

```bash
curl -X GET "https://example.com/api/native-aijs/logs/statistics?group_by=model" \
  -H "Authorization: Bearer YOUR_JWT_TOKEN" \
  -H "Accept: application/json"
```

### JavaScript Examples

#### List Logs

```javascript
// Using fetch API
async function getLogs() {
  try {
    const response = await fetch('https://example.com/api/native-aijs/logs?limit=10&status=success', {
      method: 'GET',
      headers: {
        'Authorization': `Bearer ${YOUR_JWT_TOKEN}`,
        'Accept': 'application/json'
      }
    });
    
    if (!response.ok) {
      throw new Error(`HTTP error! Status: ${response.status}`);
    }
    
    const data = await response.json();
    console.log('Logs:', data);
    return data;
  } catch (error) {
    console.error('Error fetching logs:', error);
  }
}
```

#### Get Log Details

```javascript
// Using axios
import axios from 'axios';

async function getLogDetails(logId) {
  try {
    const response = await axios.get(`https://example.com/api/native-aijs/logs/${logId}`, {
      headers: {
        'Authorization': `Bearer ${YOUR_JWT_TOKEN}`,
        'Accept': 'application/json'
      }
    });
    
    console.log('Log details:', response.data);
    return response.data;
  } catch (error) {
    console.error('Error fetching log details:', error.response || error);
  }
}
```

#### Get Log Statistics

```javascript
// Using fetch API with async/await
async function getLogStatistics() {
  try {
    const response = await fetch('https://example.com/api/native-aijs/logs/statistics?group_by=model', {
      method: 'GET',
      headers: {
        'Authorization': `Bearer ${YOUR_JWT_TOKEN}`,
        'Accept': 'application/json'
      }
    });
    
    if (!response.ok) {
      throw new Error(`HTTP error! Status: ${response.status}`);
    }
    
    const data = await response.json();
    console.log('Log statistics:', data);
    
    // Example: Calculate average tokens per request for each model
    if (data.data && data.data.breakdown_by_model) {
      const averageTokens = data.data.breakdown_by_model.map(model => ({
        model_id: model.model_id,
        avg_tokens_per_request: Math.round(model.tokens / model.requests)
      }));
      
      console.log('Average tokens per request by model:', averageTokens);
    }
    
    return data;
  } catch (error) {
    console.error('Error fetching log statistics:', error);
  }
}
```

## Error Responses

All API endpoints return consistent error formats:

```json
{
  "error": {
    "code": "UNAUTHORIZED",
    "message": "Authentication required to access this resource",
    "details": "The request is missing a valid JWT token"
  }
}
```

Common error codes:

| Code            | HTTP Status | Description                                     |
|-----------------|-------------|-------------------------------------------------|
| `UNAUTHORIZED`  | 401         | Missing or invalid authentication               |
| `FORBIDDEN`     | 403         | Insufficient permissions                        |
| `NOT_FOUND`     | 404         | Requested resource not found                    |
| `BAD_REQUEST`   | 400         | Invalid request parameters                      |
| `RATE_LIMITED`  | 429         | Rate limit exceeded                             |
| `SERVER_ERROR`  | 500         | Internal server error                           |

## Security Considerations

- All requests must be made over HTTPS
- JWT tokens should be treated as sensitive information
- Tokens expire after 24 hours by default
- User permissions are enforced at the API level
- IP-based rate limiting provides additional security

## Additional Resources

- [Native AI.js Module Documentation](README)
- [Authentication API Documentation](AUTH_API)
- [Models API Documentation](MODELS_API)
- [Drupal Services Integration](SERVICES)

## Changelog

| Version | Date       | Changes                                             |
|---------|------------|-----------------------------------------------------|
| 1.0.0   | 2025-05-01 | Initial release of the Logs API                     |
| 1.1.0   | 2025-05-15 | Added filtering by user_id and improved statistics  |
| 1.2.0   | 2025-05-30 | Added detailed error information in log entries     |
