<?php

namespace Drupal\drupal_native_ai_nativeaijs\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResult;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Datetime\DateFormatterInterface;

/**
 * Provides a block displaying Native AI.js logs summary.
 *
 * @Block(
 *   id = "native_aijs_logs_block",
 *   admin_label = @Translation("Native AI.js Logs Dashboard"),
 *   category = @Translation("AI Platform"),
 * )
 */
class NativeAiJsLogsBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * Number of log entries to display.
   *
   * @var int
   */
  protected $logEntryLimit;

  /**
   * Constructs a new NativeAiJsLogsBlock instance.
   *
   * @param array $configuration
   *   The plugin configuration.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    Connection $database,
    DateFormatterInterface $date_formatter,
    AccountInterface $current_user
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->database = $database;
    $this->dateFormatter = $date_formatter;
    $this->currentUser = $current_user;
    $this->logEntryLimit = $configuration['log_entry_limit'] ?? 5;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('database'),
      $container->get('date.formatter'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'log_entry_limit' => 5,
      'cache_time' => 300, // 5 minutes
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    $form['log_entry_limit'] = [
      '#type' => 'number',
      '#title' => $this->t('Number of log entries to display'),
      '#default_value' => $this->configuration['log_entry_limit'],
      '#min' => 1,
      '#max' => 20,
      '#description' => $this->t('The maximum number of recent logs to display in the dashboard widget.'),
    ];

    $form['cache_time'] = [
      '#type' => 'number',
      '#title' => $this->t('Cache lifetime'),
      '#default_value' => $this->configuration['cache_time'],
      '#min' => 0,
      '#max' => 3600,
      '#description' => $this->t('Time in seconds to cache the log data. Set to 0 to disable caching.'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);
    $this->configuration['log_entry_limit'] = $form_state->getValue('log_entry_limit');
    $this->configuration['cache_time'] = $form_state->getValue('cache_time');
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    // Get log statistics
    $stats = $this->getLogStatistics();
    
    // Get recent logs
    $logs = $this->getRecentLogs($this->configuration['log_entry_limit']);
    
    // Build the render array
    $build = [
      '#theme' => 'native_aijs_logs_summary',
      '#stats' => $stats,
      '#logs' => $logs,
      '#view_all_link' => Link::fromTextAndUrl(
        $this->t('View all logs'),
        Url::fromRoute('drupal_native_ai_nativeaijs.logs')
      )->toString(),
      '#attached' => [
        'library' => ['drupal_native_ai_nativeaijs/logs_dashboard'],
      ],
      '#cache' => [
        'contexts' => ['user.permissions'],
        'tags' => ['drupal_native_ai_nativeaijs:logs'],
        'max-age' => $this->configuration['cache_time'],
      ],
    ];

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    return Cache::mergeContexts(parent::getCacheContexts(), ['user.permissions']);
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    return Cache::mergeTags(parent::getCacheTags(), ['drupal_native_ai_nativeaijs:logs']);
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    return $this->configuration['cache_time'];
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account) {
    return AccessResult::allowedIfHasPermission($account, 'view native ai logs');
  }

  /**
   * Retrieves statistics from the Native AI.js logs.
   *
   * @return array
   *   An array of statistics including total calls, average response time, and error rate.
   */
  protected function getLogStatistics() {
    // Get total number of logs for the last 24 hours
    $total_query = $this->database->select('drupal_native_ai_nativeaijs_logs', 'l')
      ->condition('timestamp', REQUEST_TIME - 86400, '>')
      ->countQuery();
    $total_calls = $total_query->execute()->fetchField();
    
    // Get count of errors for error rate calculation
    $error_query = $this->database->select('drupal_native_ai_nativeaijs_logs', 'l')
      ->condition('timestamp', REQUEST_TIME - 86400, '>')
      ->condition('status', 'error')
      ->countQuery();
    $error_count = $error_query->execute()->fetchField();
    
    // Calculate error rate
    $error_rate = $total_calls > 0 ? round(($error_count / $total_calls) * 100, 2) : 0;
    
    // Get average response time
    $avg_time_query = $this->database->select('drupal_native_ai_nativeaijs_logs', 'l')
      ->condition('timestamp', REQUEST_TIME - 86400, '>')
      ->fields('l', ['performance_metrics'])
      ->condition('performance_metrics', '', '!=');
      
    $avg_time_result = $avg_time_query->execute();
    
    $total_time = 0;
    $time_count = 0;
    
    while ($record = $avg_time_result->fetchAssoc()) {
      $metrics = unserialize($record['performance_metrics']);
      if (isset($metrics['response_time'])) {
        $total_time += $metrics['response_time'];
        $time_count++;
      }
    }
    
    $avg_response_time = $time_count > 0 ? round($total_time / $time_count, 2) : 0;
    
    // Get most used model
    $model_query = $this->database->select('drupal_native_ai_nativeaijs_logs', 'l')
      ->fields('l', ['model'])
      ->condition('timestamp', REQUEST_TIME - 86400, '>')
      ->groupBy('model');
    $model_query->addExpression('COUNT(id)', 'count');
    $model_query->orderBy('count', 'DESC');
    $model_query->range(0, 1);
    
    $most_used_model_result = $model_query->execute()->fetchAssoc();
    $most_used_model = $most_used_model_result ? $most_used_model_result['model'] : 'N/A';
    
    // Get usage by operation type
    $operation_query = $this->database->select('drupal_native_ai_nativeaijs_logs', 'l')
      ->fields('l', ['operation_type'])
      ->condition('timestamp', REQUEST_TIME - 86400, '>')
      ->groupBy('operation_type');
    $operation_query->addExpression('COUNT(id)', 'count');
    $operation_query->orderBy('count', 'DESC');
    
    $operations = [];
    $operation_results = $operation_query->execute();
    
    while ($record = $operation_results->fetchAssoc()) {
      $operations[$record['operation_type']] = $record['count'];
    }
    
    return [
      'total_calls' => $total_calls,
      'avg_response_time' => $avg_response_time, // in ms
      'error_rate' => $error_rate, // percentage
      'most_used_model' => $most_used_model,
      'operations' => $operations,
    ];
  }

  /**
   * Retrieves recent logs from the database.
   *
   * @param int $limit
   *   The maximum number of logs to retrieve.
   *
   * @return array
   *   An array of log entries.
   */
  protected function getRecentLogs($limit) {
    $query = $this->database->select('drupal_native_ai_nativeaijs_logs', 'l')
      ->fields('l', ['id', 'timestamp', 'operation_type', 'message', 'status', 'model', 'request_id', 'performance_metrics'])
      ->orderBy('timestamp', 'DESC')
      ->range(0, $limit);
      
    $result = $query->execute();
    $logs = [];
    
    while ($record = $result->fetchAssoc()) {
      // Format the timestamp
      $record['formatted_time'] = $this->dateFormatter->format($record['timestamp'], 'short');
      
      // Unserialize performance metrics
      if (!empty($record['performance_metrics'])) {
        $record['performance_metrics'] = unserialize($record['performance_metrics']);
      }
      
      // Create a link to the detailed log view
      $record['log_link'] = Link::createFromRoute(
        $this->t('Details'),
        'drupal_native_ai_nativeaijs.log_detail',
        ['log_id' => $record['id']]
      )->toString();
      
      $logs[] = $record;
    }
    
    return $logs;
  }

}