---
id: drupal_native_ai_nativeaijs-README
---
# Drupal Native AI NativeAIJS Module

**Status**: Development in progress  
**Focus**: Drupal integration with Native AI.js framework

A Drupal module that integrates the Native AI.js TypeScript framework with Drupal, providing AI capabilities for Drupal sites.

## Current Status

### What Works
- Basic Drupal module structure
- Node.js server integration
- Simple text generation API
- Basic logging system
- Admin configuration interface

### What's In Development
- Advanced model management
- Comprehensive logging dashboard
- Entity-aware processing
- Extensible plugin system
- Advanced authentication

### Known Limitations
- Limited model provider support
- Basic error handling
- No streaming support yet
- Limited entity processing
- Basic admin interface

## Overview

The `drupal_native_ai_nativeaijs` module connects Drupal with the `@bluefly/native-ai` framework, enabling AI capabilities within your Drupal website. This module provides infrastructure for integrating local and cloud AI models.

## Requirements

- Drupal 9.4+ or Drupal 10
- PHP 8.1+
- Node.js 18+
- Native AI.js server (included)

## Installation

1. **Install the module using Composer:**

```bash
composer require bluefly/drupal_native_ai_nativeaijs
```

2. **Enable the module:**

```bash
drush en drupal_native_ai_nativeaijs
```

3. **Configure the Native AI.js server:**

```bash
cd modules/contrib/drupal_native_ai_nativeaijs/native_aijs
npm install
npm run build
```

4. **Start the server:**

```bash
npm run start
```

For production environments, use a process manager like PM2:

```bash
npm install -g pm2
pm2 start dist/server.js --name native-aijs
```

5. **Configure the module:**

Visit `/admin/config/services/native-ai` and configure:
- Server URL (default: http://localhost:3005)
- Authentication settings
- Default models
- Logging preferences

## Configuration

### Module Settings

Navigate to **Administration » Configuration » Services » Native AI** to configure:

- **Server Connection**: Configure the connection to the Native AI.js server
- **Default Models**: Set default models for text generation
- **Authentication**: Configure authentication methods
- **Performance Options**: Tune caching settings
- **Logging Level**: Configure logging preferences

### Permissions

The module provides the following permissions:

- **Administer Native AI**: Configure the module settings
- **View Native AI logs**: Access the logs dashboard
- **Use Native AI text generation**: Generate text with AI models
- **Use Native AI embeddings**: Generate embeddings with AI models

Configure these permissions at **Administration » People » Permissions**.

## Basic Usage

### PHP API

```php
// Generate text with an AI model
$ai_service = \Drupal::service('drupal_native_ai_nativeaijs.ai_service');

$result = $ai_service->generateText(
  'Write a product description for a coffee maker.',
  [
    'model' => 'llama-2-7b',
    'temperature' => 0.7,
    'max_tokens' => 500,
  ]
);

if ($result['success']) {
  $generated_text = $result['output'];
  $usage = $result['usage'];
} else {
  $error = $result['error'];
}

// Generate embeddings
$embedding_result = $ai_service->generateEmbeddings(
  'Coffee makers and brewing equipment',
  [
    'model' => 'text-embedding-ada-002',
  ]
);

if ($embedding_result['success']) {
  $embedding = $embedding_result['embedding'];
}
```

### JavaScript API

```javascript
// Initialize when the page loads
Drupal.behaviors.nativeAI = {
  attach: function (context, settings) {
    if (Drupal.NativeAI && context === document) {
      // Generate text
      Drupal.NativeAI.generate('Write a tagline for a coffee shop', {
        model: 'llama-2-7b',
        parameters: {
          temperature: 0.7,
          max_tokens: 50,
        }
      }).then(result => {
        if (result.success) {
          console.log(result.output);
        } else {
          console.error(result.error);
        }
      });
    }
  }
};
```

## Events System

The module provides events that other modules can subscribe to:

### PHP Events

```php
/**
 * Respond to AI generation events.
 */
function mymodule_event_subscriber_service() {
  $dispatcher = \Drupal::service('event_dispatcher');
  
  $dispatcher->addListener(
    'drupal_native_ai_nativeaijs.text_generation',
    function ($event) {
      $prompt = $event->getPrompt();
      $result = $event->getResult();
      
      \Drupal::logger('mymodule')->notice(
        'AI generated text with @tokens tokens',
        ['@tokens' => $result['usage']['total_tokens']]
      );
    }
  );
}
```

### Available Events

| Event Name | Description | Data |
|------------|-------------|------|
| `drupal_native_ai_nativeaijs.text_generation` | Fired when text is generated | prompt, options, result |
| `drupal_native_ai_nativeaijs.embedding_generation` | Fired when embeddings are generated | text, options, result |
| `drupal_native_ai_nativeaijs.error` | Fired when an error occurs | error, context |

## API Endpoints

The module provides REST endpoints for programmatic access:

- `GET /api/native-ai/models` - List available models
- `POST /api/native-ai/generate` - Generate text with AI
- `POST /api/native-ai/embed` - Generate embeddings for text
- `GET /api/native-ai/logs` - Query and filter logs

### Authentication

API endpoints require authentication using:
- Drupal session cookie for browser-based access
- API key authentication for programmatic access

## Troubleshooting

### Common Issues

1. **Connection to Native AI.js server fails**
   - Ensure the server is running and accessible
   - Check firewall settings
   - Verify the server URL in the module configuration

2. **Model not found errors**
   - Verify the model ID in your request
   - Check if the model is properly registered and enabled
   - Look for model initialization errors in the logs

3. **Performance issues**
   - Increase the cache lifetime in the module settings
   - Consider upgrading hardware for local model inference
   - Optimize the model parameters for your use case

### Logs

Check the following logs for troubleshooting:

- Native AI logs at **Administration » Reports » Native AI Logs**
- Drupal logs at **Administration » Reports » Recent log messages**
- Native AI.js server logs in your server's log directory

## Performance Optimization

The module includes several features for performance optimization:

- **Response Caching**: Configurable caching for AI responses
- **Batch Processing**: Process large content volumes efficiently
- **Load Balancing**: Distribute requests across multiple model instances

## Security Considerations

- All API requests are authenticated and authorized
- Token management follows Drupal security best practices
- Sensitive information is stored securely using Drupal's State API
- Role-based access control for all AI operations
- Audit logging for security monitoring

## Contributing

Contributions are welcome! Please follow these steps:

1. Review the [contribution guidelines](CONTRIBUTING)
2. Write tests first (following our TDD approach)
3. Submit a merge request with your changes

## License

GPL-2.0-or-later

---

**Last Updated**: June 2025
