---
id: migration-API_DOCUMENTATION
---
# LLM Platform Drupal Modules - Complete API Documentation

## Overview

This comprehensive API documentation covers all public interfaces, services, and integration patterns for the LLM Platform Drupal modules. The documentation is designed for developers integrating AI capabilities into their Drupal applications.

## Table of Contents

1. [Core API Interfaces](#core-api-interfaces)
2. [Service APIs](#service-apis)
3. [Event System](#event-system)
4. [Plugin System](#plugin-system)
5. [Integration Patterns](#integration-patterns)
6. [Configuration APIs](#configuration-apis)
7. [Streaming APIs](#streaming-apis)
8. [Field Processing APIs](#field-processing-apis)
9. [Workflow Automation APIs](#workflow-automation-apis)
10. [Performance and Analytics APIs](#performance-and-analytics-apis)

## Core API Interfaces

### AiProviderInterface

The foundational interface for all AI providers in the platform.

```php
namespace Drupal\drupal_native_ai\Provider;

/**
 * Interface for AI providers.
 */
interface AiProviderInterface {

  /**
   * Process input data with AI capabilities.
   *
   * @param mixed $input
   *   The input data to process.
   * @param array $options
   *   Processing options including:
   *   - 'operation': string - The type of operation to perform
   *   - 'model': string - Specific model to use
   *   - 'timeout': int - Request timeout in seconds
   *   - 'stream': bool - Enable streaming response
   *   - 'context': array - Additional context data
   *
   * @return \Drupal\drupal_native_ai\Result\AiResultInterface
   *   The processing result.
   *
   * @throws \Drupal\drupal_native_ai\Exception\AiProviderException
   *   If processing fails.
   */
  public function process($input, array $options = []): AiResultInterface;

  /**
   * Get provider capabilities.
   *
   * @return array
   *   Array of capability identifiers:
   *   - 'text_generation': Text generation capabilities
   *   - 'text_analysis': Text analysis and classification
   *   - 'image_analysis': Image processing capabilities
   *   - 'embedding_generation': Vector embedding generation
   *   - 'streaming': Real-time streaming support
   */
  public function getCapabilities(): array;

  /**
   * Validate provider configuration.
   *
   * @return bool
   *   TRUE if configuration is valid, FALSE otherwise.
   */
  public function validateConfiguration(): bool;

  /**
   * Get provider status information.
   *
   * @return array
   *   Status information including:
   *   - 'available': bool - Provider availability
   *   - 'health': string - Health status (healthy, degraded, unavailable)
   *   - 'response_time': float - Average response time in seconds
   *   - 'error_rate': float - Error rate percentage
   *   - 'last_check': int - Unix timestamp of last health check
   */
  public function getStatus(): array;

  /**
   * Get supported models for this provider.
   *
   * @return array
   *   Array of model definitions keyed by model ID:
   *   - 'name': string - Human-readable model name
   *   - 'description': string - Model description
   *   - 'capabilities': array - Model-specific capabilities
   *   - 'cost': array - Cost information (tokens, requests)
   *   - 'limits': array - Rate limits and constraints
   */
  public function getSupportedModels(): array;
}
```

### AiResultInterface

Interface for AI processing results.

```php
namespace Drupal\drupal_native_ai\Result;

/**
 * Interface for AI processing results.
 */
interface AiResultInterface {

  /**
   * Check if the operation was successful.
   *
   * @return bool
   *   TRUE if successful, FALSE otherwise.
   */
  public function isSuccessful(): bool;

  /**
   * Get the processed data.
   *
   * @return mixed
   *   The processed result data.
   */
  public function getData();

  /**
   * Get result metadata.
   *
   * @return array
   *   Metadata including:
   *   - 'provider': string - Provider used
   *   - 'model': string - Model used
   *   - 'processing_time': float - Time taken in seconds
   *   - 'tokens_used': int - Number of tokens consumed
   *   - 'confidence': float - Confidence score (0-1)
   *   - 'cached': bool - Whether result was cached
   */
  public function getMetadata(): array;

  /**
   * Get any errors that occurred.
   *
   * @return array
   *   Array of error messages.
   */
  public function getErrors(): array;

  /**
   * Convert result to array format.
   *
   * @return array
   *   Complete result data as array.
   */
  public function toArray(): array;

  /**
   * Get confidence score for the result.
   *
   * @return float
   *   Confidence score between 0 and 1.
   */
  public function getConfidence(): float;
}
```

## Service APIs

### AI Provider Manager

Central service for managing AI providers.

```php
// Get the provider manager
$providerManager = \Drupal::service('drupal_native_ai.provider_manager');

// Process content with auto-selected provider
$result = $providerManager->process('Analyze this text for sentiment', [
  'operation' => 'sentiment_analysis',
  'auto_select_provider' => TRUE,
  'fallback_enabled' => TRUE
]);

// Process with specific provider
$result = $providerManager->processWithProvider('openai', 'Generate a summary', [
  'operation' => 'text_summarization',
  'model' => 'gpt-4',
  'max_tokens' => 500
]);

// Get available providers
$providers = $providerManager->getAvailableProviders();

// Get provider by capability
$textProviders = $providerManager->getProvidersByCapability('text_generation');

// Benchmark providers
$benchmark = $providerManager->benchmarkProviders([
  'test_data' => 'Sample text for benchmarking',
  'operations' => ['sentiment_analysis', 'text_classification'],
  'metrics' => ['response_time', 'accuracy', 'cost']
]);
```

### Pattern Analyzer Service

Service for analyzing content patterns and extracting insights.

```php
// Get the pattern analyzer
$patternAnalyzer = \Drupal::service('drupal_native_ai.pattern_analyzer');

// Analyze entity patterns
$entity = \Drupal\node\Entity\Node::load(123);
$patterns = $patternAnalyzer->analyzeEntity($entity, [
  'detect_relationships' => TRUE,
  'analyze_content_structure' => TRUE,
  'generate_insights' => TRUE,
  'confidence_threshold' => 0.7
]);

// Analyze content patterns
$contentPatterns = $patternAnalyzer->analyzeContent($text, [
  'pattern_types' => ['email', 'phone', 'url', 'date'],
  'extract_entities' => TRUE,
  'classify_content' => TRUE
]);

// Analyze field patterns
$fieldPatterns = $patternAnalyzer->analyzeField($field_value, $field_definition, [
  'suggest_optimizations' => TRUE,
  'detect_anomalies' => TRUE,
  'generate_validation_rules' => TRUE
]);

// Get pattern insights
$insights = $patternAnalyzer->getInsights($patterns, [
  'include_recommendations' => TRUE,
  'generate_action_items' => TRUE,
  'priority_threshold' => 'medium'
]);
```

### Model Comparison Service

Service for comparing and benchmarking AI models.

```php
// Get the model comparison service
$modelComparison = \Drupal::service('drupal_native_ai.model_comparison');

// Compare models on specific task
$comparison = $modelComparison->compareModels([
  'models' => ['gpt-4', 'claude-3-sonnet', 'native_ai'],
  'test_data' => $testDataSet,
  'task' => 'text_classification',
  'metrics' => ['accuracy', 'response_time', 'cost_efficiency']
]);

// Run comprehensive benchmark
$benchmark = $modelComparison->runBenchmark([
  'test_suite' => 'standard',
  'include_providers' => ['openai', 'anthropic', 'native'],
  'tasks' => ['completion', 'analysis', 'classification'],
  'iterations' => 100
]);

// Get model recommendations
$recommendations = $modelComparison->getRecommendations($requirements, [
  'task_type' => 'content_analysis',
  'performance_priority' => 'balanced',
  'cost_constraints' => ['max_monthly' => 500],
  'latency_requirements' => ['max_response_time' => 2.0]
]);

// Track model performance over time
$performance = $modelComparison->trackPerformance('gpt-4', [
  'timeframe' => '30 days',
  'metrics' => ['accuracy', 'response_time', 'error_rate'],
  'aggregation' => 'daily_average'
]);
```

## Event System

### AI Processing Events

Events fired during AI processing operations.

```php
namespace Drupal\drupal_native_ai\Event;

/**
 * Event fired before AI processing begins.
 */
class AiPreProcessEvent extends Event {
  
  const EVENT_NAME = 'drupal_native_ai.pre_process';
  
  protected $input;
  protected $options;
  protected $provider;
  
  public function __construct($input, array $options, AiProviderInterface $provider) {
    $this->input = $input;
    $this->options = $options;
    $this->provider = $provider;
  }
  
  // Getters and setters for input, options, provider
}

/**
 * Event fired after AI processing completes.
 */
class AiPostProcessEvent extends Event {
  
  const EVENT_NAME = 'drupal_native_ai.post_process';
  
  protected $result;
  protected $originalInput;
  protected $options;
  
  // Event implementation
}

/**
 * Event fired when AI processing fails.
 */
class AiProcessingErrorEvent extends Event {
  
  const EVENT_NAME = 'drupal_native_ai.processing_error';
  
  protected $error;
  protected $input;
  protected $provider;
  
  // Event implementation
}
```

### Event Subscriber Example

```php
namespace Drupal\mymodule\EventSubscriber;

use Drupal\drupal_native_ai\Event\AiPreProcessEvent;
use Drupal\drupal_native_ai\Event\AiPostProcessEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Example AI event subscriber.
 */
class MyAiEventSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      AiPreProcessEvent::EVENT_NAME => ['onPreProcess', 10],
      AiPostProcessEvent::EVENT_NAME => ['onPostProcess', 10],
    ];
  }

  /**
   * Handles pre-processing events.
   */
  public function onPreProcess(AiPreProcessEvent $event) {
    // Log AI processing start
    \Drupal::logger('mymodule')->info('AI processing started for provider: @provider', [
      '@provider' => $event->getProvider()->getId()
    ]);
    
    // Modify input if needed
    $input = $event->getInput();
    if (is_string($input)) {
      $event->setInput(trim($input));
    }
    
    // Add custom options
    $options = $event->getOptions();
    $options['custom_tracking_id'] = uniqid();
    $event->setOptions($options);
  }

  /**
   * Handles post-processing events.
   */
  public function onPostProcess(AiPostProcessEvent $event) {
    $result = $event->getResult();
    
    // Log successful processing
    if ($result->isSuccessful()) {
      \Drupal::logger('mymodule')->info('AI processing completed successfully');
      
      // Store result for analytics
      $this->storeAnalytics($result);
    }
  }
}
```

## Plugin System

### AI Provider Plugin

Create custom AI providers using the plugin system.

```php
namespace Drupal\mymodule\Plugin\AIProvider;

use Drupal\drupal_native_ai\Plugin\AIProvider\AIProviderPluginBase;
use Drupal\drupal_native_ai\Annotation\AIProvider;

/**
 * Custom AI provider plugin.
 *
 * @AIProvider(
 *   id = "custom_provider",
 *   label = @Translation("Custom AI Provider"),
 *   description = @Translation("Custom AI processing provider"),
 *   capabilities = {
 *     "text_generation",
 *     "text_analysis",
 *     "custom_processing"
 *   }
 * )
 */
class CustomAiProvider extends AIProviderPluginBase {

  /**
   * {@inheritdoc}
   */
  public function process($input, array $options = []) {
    // Custom processing logic
    $operation = $options['operation'] ?? 'default';
    
    switch ($operation) {
      case 'text_analysis':
        return $this->analyzeText($input, $options);
      case 'text_generation':
        return $this->generateText($input, $options);
      default:
        return $this->defaultProcessing($input, $options);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getCapabilities() {
    return [
      'text_generation',
      'text_analysis',
      'custom_processing'
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfiguration() {
    $config = $this->getConfiguration();
    return !empty($config['api_endpoint']) && !empty($config['api_key']);
  }

  /**
   * Custom text analysis implementation.
   */
  protected function analyzeText($text, array $options) {
    // Implementation
  }

  /**
   * Custom text generation implementation.
   */
  protected function generateText($prompt, array $options) {
    // Implementation
  }
}
```

### Pattern Analyzer Plugin

Create custom pattern analyzers.

```php
namespace Drupal\mymodule\Plugin\PatternAnalyzer;

use Drupal\drupal_native_ai\Plugin\PatternAnalyzer\PatternAnalyzerBase;
use Drupal\drupal_native_ai\Annotation\PatternAnalyzer;

/**
 * Email pattern analyzer.
 *
 * @PatternAnalyzer(
 *   id = "email_analyzer",
 *   label = @Translation("Email Pattern Analyzer"),
 *   description = @Translation("Analyzes email patterns in content"),
 *   pattern_types = {"email", "communication"}
 * )
 */
class EmailPatternAnalyzer extends PatternAnalyzerBase {

  /**
   * {@inheritdoc}
   */
  public function analyzePattern($data, array $options = []) {
    $patterns = [];
    
    // Email regex pattern
    $emailPattern = '/\b[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\.[A-Z|a-z]{2,}\b/';
    
    if (preg_match_all($emailPattern, $data, $matches)) {
      foreach ($matches[0] as $email) {
        $patterns[] = [
          'type' => 'email',
          'value' => $email,
          'confidence' => $this->calculateEmailConfidence($email),
          'metadata' => [
            'domain' => substr(strrchr($email, '@'), 1),
            'local_part' => strstr($email, '@', true)
          ]
        ];
      }
    }
    
    return $patterns;
  }

  /**
   * {@inheritdoc}
   */
  public function getPatternTypes() {
    return ['email'];
  }

  /**
   * Calculate confidence score for email pattern.
   */
  protected function calculateEmailConfidence($email) {
    $score = 0.5; // Base score
    
    // Common domains get higher confidence
    $commonDomains = ['gmail.com', 'yahoo.com', 'outlook.com', 'hotmail.com'];
    $domain = substr(strrchr($email, '@'), 1);
    
    if (in_array($domain, $commonDomains)) {
      $score += 0.3;
    }
    
    // Valid TLD structure
    if (preg_match('/\.[a-z]{2,4}$/i', $domain)) {
      $score += 0.2;
    }
    
    return min($score, 1.0);
  }
}
```

## Integration Patterns

### Service Discovery Pattern

Use service discovery to integrate with optional modules.

```php
namespace Drupal\mymodule\Service;

use Drupal\drupal_native_ai\ServiceDiscovery\ModuleDiscovery;

class MyIntegrationService {

  protected $discovery;

  public function __construct(ModuleDiscovery $discovery) {
    $this->discovery = $discovery;
  }

  /**
   * Process content with best available AI service.
   */
  public function processContent($content, array $options = []) {
    // Try advanced LLM platform first
    if ($this->discovery->isModuleAvailable('llm')) {
      $platform = $this->discovery->getServiceOrFallback(
        'llm.platform',
        function() { return $this->createFallbackLlm(); }
      );
      return $platform->process($content, $options);
    }
    
    // Try native AI as secondary option
    if ($this->discovery->isModuleAvailable('drupal_native_ai')) {
      $provider = $this->discovery->getServiceOrFallback(
        'drupal_native_ai.provider',
        function() { return $this->createFallbackProvider(); }
      );
      return $provider->process($content, $options);
    }
    
    // Use basic processing as last resort
    return $this->basicProcessing($content, $options);
  }

  /**
   * Get analytics if available.
   */
  public function getAnalytics($timeframe = '30 days') {
    return $this->discovery->getModuleServiceOrFallback(
      'field_eca_bridge',
      'field_eca_bridge.workflow_analytics',
      function() {
        return new BasicAnalytics();
      }
    )->generateAnalytics($timeframe);
  }
}
```

### Provider Factory Pattern

Create providers dynamically based on configuration.

```php
namespace Drupal\mymodule\Factory;

use Drupal\drupal_native_ai\Provider\AiProviderInterface;

class AiProviderFactory {

  /**
   * Create provider based on configuration.
   */
  public static function createProvider(array $config): AiProviderInterface {
    $type = $config['type'] ?? 'native';
    
    switch ($type) {
      case 'openai':
        return new OpenAiProvider($config);
      case 'anthropic':
        return new AnthropicProvider($config);
      case 'custom':
        return new CustomProvider($config);
      default:
        return new NativeAiProvider($config);
    }
  }

  /**
   * Create provider with fallback chain.
   */
  public static function createWithFallback(array $primaryConfig, array $fallbackConfigs = []): AiProviderInterface {
    try {
      return static::createProvider($primaryConfig);
    } catch (\Exception $e) {
      foreach ($fallbackConfigs as $fallbackConfig) {
        try {
          return static::createProvider($fallbackConfig);
        } catch (\Exception $fallbackException) {
          continue;
        }
      }
      
      // Final fallback to basic provider
      return new BasicAiProvider([]);
    }
  }
}
```

## Configuration APIs

### Provider Configuration

Programmatically configure AI providers.

```php
// Configure OpenAI provider
$config = \Drupal::configFactory()->getEditable('drupal_native_ai.providers.openai');
$config->setData([
  'enabled' => TRUE,
  'api_key' => $apiKey,
  'model' => 'gpt-4',
  'max_tokens' => 2000,
  'temperature' => 0.7,
  'timeout' => 30,
  'rate_limit' => [
    'requests_per_minute' => 60,
    'tokens_per_minute' => 100000
  ],
  'features' => [
    'streaming' => TRUE,
    'function_calling' => TRUE,
    'vision' => FALSE
  ]
]);
$config->save();

// Configure fallback behavior
$fallbackConfig = \Drupal::configFactory()->getEditable('drupal_native_ai.fallback');
$fallbackConfig->setData([
  'enabled' => TRUE,
  'fallback_chain' => ['anthropic', 'native_ai'],
  'retry_attempts' => 3,
  'retry_delay' => 1000, // milliseconds
  'circuit_breaker' => [
    'failure_threshold' => 5,
    'recovery_timeout' => 60 // seconds
  ]
]);
$fallbackConfig->save();
```

### Field ECA Bridge Configuration

Configure field automation and ECA integration.

```php
// Configure field automation
$fieldConfig = \Drupal::configFactory()->getEditable('field_eca_bridge.automation');
$fieldConfig->setData([
  'auto_validation' => TRUE,
  'ai_recommendations' => [
    'enabled' => TRUE,
    'confidence_threshold' => 0.8,
    'provider' => 'auto_select'
  ],
  'workflow_triggers' => [
    'field_change' => TRUE,
    'entity_save' => TRUE,
    'validation_failure' => TRUE
  ],
  'performance_monitoring' => [
    'enabled' => TRUE,
    'metrics' => ['response_time', 'accuracy', 'error_rate'],
    'alert_thresholds' => [
      'max_response_time' => 5.0,
      'max_error_rate' => 0.05
    ]
  ]
]);
$fieldConfig->save();
```

## Streaming APIs

### Real-time AI Streaming

Implement real-time AI streaming capabilities.

```php
// Get streaming service
$streamingService = \Drupal::service('drupal_native_ai.streaming_service');

// Start streaming operation
$stream = $streamingService->startStream('content_generation', [
  'prompt' => 'Write an article about AI in healthcare',
  'provider' => 'openai',
  'model' => 'gpt-4',
  'stream_options' => [
    'chunk_size' => 1024,
    'timeout' => 300,
    'heartbeat_interval' => 30
  ]
]);

// Handle streaming data
$stream->onData(function($chunk) {
  // Process incoming data chunk
  echo "Received: " . $chunk['content'];
  
  // Update progress
  $progress = $chunk['metadata']['progress'] ?? 0;
  echo "Progress: {$progress}%\n";
});

$stream->onComplete(function($finalResult) {
  echo "Streaming completed\n";
  echo "Final result: " . $finalResult['content'];
});

$stream->onError(function($error) {
  echo "Stream error: " . $error['message'];
});

// Start the stream
$stream->start();

// WebSocket streaming for real-time updates
$websocketService = \Drupal::service('drupal_native_ai_nativeaijs.websocket_streaming');

$websocketStream = $websocketService->createStream([
  'endpoint' => '/ai/stream',
  'authentication' => ['type' => 'jwt', 'token' => $jwtToken],
  'options' => [
    'auto_reconnect' => TRUE,
    'max_reconnect_attempts' => 5,
    'reconnect_delay' => 2000
  ]
]);

// Client-side JavaScript integration
?>
<script>
const aiStream = new DrupalAiStream({
  endpoint: '/ai/websocket',
  token: '<?php echo $jwtToken; ?>',
  onMessage: function(data) {
    document.getElementById('ai-output').innerHTML += data.content;
  },
  onComplete: function(result) {
    console.log('AI streaming completed', result);
  }
});

aiStream.send({
  operation: 'text_generation',
  prompt: 'Generate content...',
  options: { stream: true }
});
</script>
```

## Field Processing APIs

### AI-Enhanced Field Processing

Process fields with AI capabilities.

```php
// Get field mapping service
$fieldMapper = \Drupal::service('field_eca_bridge.field_mapping_service');

// Process field with AI enhancement
$enhancedValue = $fieldMapper->processField($fieldValue, [
  'operation' => 'content_enhancement',
  'ai_provider' => 'auto_select',
  'enhancement_types' => [
    'grammar_correction',
    'style_improvement',
    'fact_checking',
    'seo_optimization'
  ],
  'preserve_intent' => TRUE,
  'confidence_threshold' => 0.8
]);

// Get AI recommendations for field configuration
$recommendations = $fieldMapper->getFieldRecommendations($fieldDefinition, [
  'analyze_existing_content' => TRUE,
  'content_sample_size' => 100,
  'include_performance_metrics' => TRUE,
  'suggest_widgets' => TRUE,
  'generate_validation_rules' => TRUE
]);

// Validate field with AI
$validationService = \Drupal::service('field_eca_bridge.validation_service');
$validationResult = $validationService->validateWithAI($fieldValue, $fieldDefinition, [
  'validation_types' => [
    'content_quality',
    'format_compliance',
    'data_consistency',
    'accessibility_compliance'
  ],
  'severity_threshold' => 'warning',
  'auto_correct' => FALSE
]);

if (!$validationResult->isValid()) {
  foreach ($validationResult->getViolations() as $violation) {
    \Drupal::messenger()->addError($violation->getMessage());
  }
}
```

### Field Analytics and Insights

Analyze field performance and usage patterns.

```php
// Get workflow analytics service
$analytics = \Drupal::service('field_eca_bridge.workflow_analytics');

// Generate field usage analytics
$fieldAnalytics = $analytics->analyzeFieldUsage('field_body', [
  'timeframe' => '90 days',
  'include_performance_metrics' => TRUE,
  'analyze_content_patterns' => TRUE,
  'generate_optimization_suggestions' => TRUE,
  'track_user_behavior' => TRUE
]);

// Get real-time field metrics
$realtimeMetrics = $analytics->getRealTimeMetrics('field_body', [
  'metrics' => [
    'processing_time',
    'error_rate',
    'cache_hit_ratio',
    'ai_enhancement_usage',
    'validation_failure_rate'
  ],
  'aggregation_window' => '5 minutes',
  'alert_on_anomalies' => TRUE
]);

// Generate field optimization report
$optimizationReport = $analytics->generateOptimizationReport([
  'fields' => ['field_body', 'field_summary', 'field_tags'],
  'analysis_depth' => 'comprehensive',
  'include_ai_recommendations' => TRUE,
  'benchmark_against' => 'industry_standards',
  'performance_targets' => [
    'max_processing_time' => 2.0,
    'min_cache_hit_ratio' => 0.8,
    'max_error_rate' => 0.02
  ]
]);
```

## Workflow Automation APIs

### ECA Workflow Integration

Integrate AI capabilities with ECA workflows.

```php
// Get ECA integration service
$ecaIntegration = \Drupal::service('field_eca_bridge.eca_integration');

// Create AI-powered ECA condition
$condition = $ecaIntegration->createAICondition([
  'condition_type' => 'ai_content_analysis',
  'analysis_operation' => 'sentiment_analysis',
  'confidence_threshold' => 0.8,
  'expected_sentiment' => 'positive',
  'fallback_behavior' => 'skip_condition'
]);

// Create AI-powered ECA action
$action = $ecaIntegration->createAIAction([
  'action_type' => 'ai_content_enhancement',
  'enhancement_operation' => 'grammar_correction',
  'provider' => 'auto_select',
  'preserve_meaning' => TRUE,
  'apply_automatically' => FALSE
]);

// Register custom ECA plugins with AI capabilities
$ecaIntegration->registerAIPlugin([
  'plugin_id' => 'ai_field_validator',
  'plugin_type' => 'condition',
  'ai_operation' => 'field_validation',
  'configuration_schema' => [
    'validation_types' => ['required', 'array'],
    'confidence_threshold' => ['optional', 'float'],
    'provider_preference' => ['optional', 'string']
  ]
]);

// Trigger AI-enhanced workflow
$workflowResult = $ecaIntegration->triggerWorkflow('ai_content_processing', [
  'entity' => $node,
  'field_name' => 'field_body',
  'ai_operations' => [
    'analyze_content',
    'enhance_quality',
    'validate_accuracy'
  ],
  'workflow_options' => [
    'async_processing' => TRUE,
    'notify_on_completion' => TRUE,
    'store_intermediate_results' => TRUE
  ]
]);
```

### Predictive Workflow Optimization

Use AI to optimize workflow performance.

```php
// Get workflow predictor service
$predictor = \Drupal::service('field_eca_bridge.workflow_predictor');

// Predict optimal workflow configuration
$predictions = $predictor->predictOptimalConfiguration($workflowDefinition, [
  'historical_data_timeframe' => '6 months',
  'performance_targets' => [
    'max_execution_time' => 30,
    'min_success_rate' => 0.95,
    'max_resource_usage' => 80
  ],
  'optimization_goals' => [
    'minimize_execution_time',
    'maximize_success_rate',
    'optimize_resource_usage'
  ],
  'constraints' => [
    'preserve_functionality' => TRUE,
    'maintain_data_integrity' => TRUE
  ]
]);

// Apply predicted optimizations
if ($predictions['confidence'] > 0.8) {
  $optimizedWorkflow = $predictor->applyOptimizations(
    $workflowDefinition, 
    $predictions['optimizations']
  );
  
  // Test optimized workflow
  $testResults = $predictor->testWorkflow($optimizedWorkflow, [
    'test_data_size' => 1000,
    'validation_metrics' => ['accuracy', 'performance', 'reliability'],
    'rollback_on_failure' => TRUE
  ]);
  
  if ($testResults['success']) {
    $predictor->deployOptimizations($optimizedWorkflow);
  }
}
```

## Performance and Analytics APIs

### Performance Monitoring

Monitor and optimize AI operation performance.

```php
// Get performance monitoring service
$monitor = \Drupal::service('llm.performance_monitor');

// Start performance tracking
$trackingId = $monitor->startTracking('ai_operation', [
  'operation_type' => 'text_generation',
  'provider' => 'openai',
  'model' => 'gpt-4',
  'expected_duration' => 5.0
]);

// Perform AI operation
$result = $aiProvider->process($input, $options);

// End tracking and get metrics
$metrics = $monitor->endTracking($trackingId, [
  'success' => $result->isSuccessful(),
  'token_usage' => $result->getMetadata()['tokens_used'] ?? 0,
  'response_quality' => $result->getConfidence()
]);

// Get comprehensive performance analytics
$analytics = $monitor->getAnalytics([
  'timeframe' => '7 days',
  'group_by' => ['provider', 'operation_type'],
  'metrics' => [
    'average_response_time',
    'success_rate',
    'error_distribution',
    'cost_analysis',
    'performance_trends'
  ],
  'filters' => [
    'min_confidence' => 0.7,
    'exclude_cached_results' => FALSE
  ]
]);

// Set up real-time alerts
$monitor->configureAlerts([
  'response_time_threshold' => 10.0,
  'error_rate_threshold' => 0.05,
  'cost_limit_daily' => 100.00,
  'notification_channels' => ['email', 'slack', 'drupal_message']
]);
```

### Predictive Analytics

Use AI to predict system behavior and optimize performance.

```php
// Get predictive analytics service
$predictor = \Drupal::service('llm.predictive_analytics_engine');

// Predict resource usage
$resourcePrediction = $predictor->predictResourceUsage([
  'prediction_horizon' => '24 hours',
  'confidence_interval' => 0.95,
  'include_factors' => [
    'historical_usage',
    'scheduled_tasks',
    'user_activity_patterns',
    'seasonal_trends'
  ]
]);

// Predict optimal scaling
$scalingRecommendations = $predictor->predictOptimalScaling([
  'current_load' => $currentMetrics,
  'growth_projections' => $growthData,
  'performance_targets' => [
    'max_response_time' => 2.0,
    'min_availability' => 0.999,
    'cost_efficiency' => 'high'
  ],
  'scaling_constraints' => [
    'max_instances' => 10,
    'budget_limit' => 1000.00
  ]
]);

// Predict potential issues
$issuesPrediction = $predictor->predictPotentialIssues([
  'analysis_window' => '48 hours',
  'issue_types' => [
    'performance_degradation',
    'resource_exhaustion',
    'error_rate_spike',
    'cost_overrun'
  ],
  'prevention_suggestions' => TRUE,
  'automated_mitigation' => FALSE
]);

// Generate optimization recommendations
$optimizations = $predictor->generateOptimizationRecommendations([
  'optimization_goals' => ['performance', 'cost', 'reliability'],
  'priority_weighting' => [
    'performance' => 0.4,
    'cost' => 0.3,
    'reliability' => 0.3
  ],
  'implementation_complexity' => 'medium',
  'expected_impact_threshold' => 0.1 // 10% improvement minimum
]);
```

This comprehensive API documentation provides developers with all the tools needed to integrate AI capabilities into their Drupal applications, from basic AI processing to advanced workflow automation and performance optimization.
