---
id: migration-ARCHIVE
---
# CommonNPM Projects - Archive Documentation

## Overview
This document provides archival documentation for standalone test files, demos, and deprecated code across the CommonNPM projects that should be moved to proper test directories or archived.

## Project: bfllm

### Standalone Test Files
These files should be moved to `__tests__/` directory or archived:

1. **test-fallback.js** - Fallback system testing
   - Purpose: Tests fallback model implementations
   - Value: Contains fallback patterns for resilient AI systems
   - Action: Convert to proper Jest test in `__tests__/fallback.test.ts`

2. **test-language-learning.js** - Language learning feature tests
   - Purpose: Tests language learning API functionality
   - Value: Integration test patterns for ML features
   - Action: Move to `__tests__/features/language-learning.test.ts`

3. **test-language-learning-standalone.js** - Standalone language tests
   - Purpose: Independent language learning tests
   - Value: Can run without full system setup
   - Action: Archive as example in `docs/examples/`

## Project: bfmcp

### Standalone Test Files

1. **simple-test.js** - Basic functionality test
   - Purpose: Quick smoke test
   - Value: Minimal test setup example
   - Action: Archive to `__DELETE_LATER/bfmcp_test_files_2025-01-07/`

2. **simple-test-streaming-vector.js** - Streaming vector tests
   - Purpose: Tests vector streaming capabilities
   - Value: Streaming implementation patterns
   - Action: Convert to `__tests__/streaming/vector.test.ts`

3. **test-implementations.js** - Implementation tests
   - Purpose: Tests various MCP implementations
   - Value: Comprehensive implementation coverage
   - Action: Split into multiple focused tests in `__tests__/`

4. **test-imports.js** - Import verification
   - Purpose: Validates module imports work correctly
   - Value: Package integrity testing
   - Action: Move to `__tests__/integration/imports.test.ts`

## Project: bfapi

### Test Files Already Organized
The bfapi project has properly organized tests in `__tests__/` directory. However, these validation scripts should be archived:

1. **validate-monitoring-tests.js**
2. **validate-tdd-implementation.js**
3. **run-monitoring-tdd-workflow.js**
4. **run-monitoring-tdd-workflow-wrapper.js**

Action: Move to `scripts/archived/` with documentation

## Project: bfrfp

### Standalone Test Files

1. **simple-free-model-test.cjs** - Free model testing
   - Purpose: Tests free AI model integration
   - Value: Cost-effective model testing patterns
   - Action: Convert to Jest test

2. **test-ai-integration.js** - AI integration tests
   - Purpose: Tests AI service integration
   - Value: Integration patterns
   - Action: Move to `__tests__/integration/`

3. **test-ai-integration-wrapper.js** - Wrapper for AI tests
   - Purpose: Test execution wrapper
   - Value: Limited, mostly redundant
   - Action: Archive to `__DELETE_LATER/`

## Project: bfui

### Standalone Test Files

1. **test-accessibility-system.js** - Accessibility testing
   - Purpose: Tests accessibility compliance
   - Value: A11y testing patterns
   - Action: Keep but move to `__tests__/accessibility/`

2. **test-accessible-modal.js** - Modal accessibility
   - Purpose: Tests modal component accessibility
   - Value: Component-specific a11y tests
   - Action: Move to `__tests__/components/modal.test.ts`

3. **test-build.js** - Build process testing
   - Purpose: Validates build output
   - Value: Build verification
   - Action: Move to `scripts/test/`

4. **test-components.js** - Component testing
   - Purpose: General component tests
   - Value: Should be split into specific tests
   - Action: Split and move to `__tests__/components/`

5. **test-examples.js** - Example code testing
   - Purpose: Validates documentation examples
   - Value: Ensures examples work
   - Action: Move to `scripts/validate-examples.js`

## Project: bfcli

### Test Organization
Most tests are properly organized in `__tests__/` directory. These scripts should be reviewed:

1. **test-shared-migration.js** - Shared utilities migration test
2. **test-native-ai-integration.js** - Native AI integration test
3. **test-tokens-cli.js** - Token management CLI test
4. **test-tokens-validate.js** - Token validation test
5. **test-validate-command.js** - Command validation test

Action: These appear to be one-time migration/validation scripts. Archive to `__DELETE_LATER/bfcli_migration_scripts_2025-01-07/`

## Project: bfcicomponents

### Standalone Test Files

1. **test.cjs** - Generic test file
   - Purpose: Unknown/generic testing
   - Action: Archive to `__DELETE_LATER/`

2. **test-yaml-fix.cjs** - YAML fixing test
   - Purpose: Tests YAML processing
   - Action: If valuable, move to `__tests__/utils/yaml.test.ts`

## Valuable Patterns to Preserve

### 1. Fallback Testing (bfllm)
- Resilient system design
- Graceful degradation patterns
- Should be documented in architecture guides

### 2. Streaming Tests (bfmcp)
- Real-time data processing
- Vector streaming implementation
- Valuable for performance documentation

### 3. Accessibility Testing (bfui)
- Comprehensive a11y validation
- Component-specific accessibility
- Should become standard practice

### 4. Integration Testing (bfapi)
- Cross-service communication
- API contract validation
- OpenAPI compliance testing

## Migration Strategy

### Phase 1: Categorization (Immediate)
1. Identify test purpose and value
2. Determine migration path
3. Document valuable patterns

### Phase 2: Migration (1 Week)
1. Convert valuable tests to Jest format
2. Move to appropriate `__tests__/` directories
3. Update import paths

### Phase 3: Archival (2 Weeks)
1. Create `__DELETE_LATER` directories
2. Move deprecated/redundant files
3. Update CI/CD configurations

### Phase 4: Documentation (Ongoing)
1. Extract valuable patterns to docs
2. Create testing best practices guide
3. Update contributing guidelines

## Archive Structure

```
__DELETE_LATER/
├── common_npm_test_files_2025-01-07/
│   ├── bfllm/
│   │   └── test-language-learning-standalone.js
│   ├── bfmcp/
│   │   └── simple-test.js
│   ├── bfrfp/
│   │   └── test-ai-integration-wrapper.js
│   ├── bfui/
│   │   └── test-examples.js
│   ├── bfcli/
│   │   ├── test-shared-migration.js
│   │   ├── test-tokens-cli.js
│   │   └── test-validate-command.js
│   └── bfcicomponents/
│       └── test.cjs
└── validation_scripts_2025-01-07/
    ├── validate-monitoring-tests.js
    ├── validate-tdd-implementation.js
    └── run-monitoring-tdd-workflow.js
```

## Notes

- Most CommonNPM projects already have good test organization
- Standalone test files are mostly legacy or one-time scripts
- Valuable testing patterns should be preserved in documentation
- Focus on converting useful tests to proper Jest format

---

**Created**: 2025-01-07
**Status**: Ready for migration
**Next Action**: Begin categorizing and migrating valuable tests
