---
id: modules-development-native_ai-AGENT_INTEGRATION
---
# Agent Management Integration

This document describes how the Drupal Native AI module integrates with the llm module's agent management system.

## Overview

The Drupal Native AI module provides a bridge between its AI providers and the agent management system in the llm module. This integration allows agents created in the llm module to use the AI providers registered in the Drupal Native AI module.

## Architecture

The integration follows the adapter pattern, where the `AgentFactoryAdapter` class implements the necessary methods to fulfill the `AgentFactoryInterface` in the llm module.

```
+-------------------------+      +-------------------------+
|                         |      |                         |
|  llm Module             |      | drupal_native_ai Module|
|                         |      |                         |
|  +------------------+   |      |  +------------------+   |
|  |                  |   |      |  |                  |   |
|  | AgentInterface   |   |      |  | AiProviderManager|   |
|  | AgentFactory     |<---------|--+ AgentFactoryAdapter   |
|  | AgentRegistry    |   |      |  |                  |   |
|  |                  |   |      |  |                  |   |
|  +------------------+   |      |  +------------------+   |
|                         |      |                         |
+-------------------------+      +-------------------------+
```

## Implementation

The `AgentFactoryAdapter` provides the following functionality:

1. Agent CRUD operations (create, read, update, delete)
2. Agent execution (both synchronous and streaming)
3. Agent configuration validation

These operations are delegated to the best available AI provider that can handle the requested operation with the required capabilities.

## Service Registration

The `AgentFactoryAdapter` is registered as a service in the Drupal service container and tagged with:

- `llm.agent_factory` - Allows it to be discovered by the llm module as an agent factory implementation
- `llm.agent_registry_provider` - Allows it to provide agents to the llm module's agent registry

## Fallback Handling

The adapter uses the service discovery pattern to detect if the llm module is available. If not, it logs a notice and provides limited functionality. This ensures graceful degradation when the llm module is not installed.

## Usage

### Creating an Agent

```php
// Get the agent factory adapter
$agent_factory = \Drupal::service('drupal_native_ai.adapter.agent_factory');

// Create an agent
$agent = $agent_factory->createAgent([
  'name' => 'Content Reviewer',
  'description' => 'Reviews content for quality and consistency',
  'model_id' => 'gpt-4',
  'capabilities' => ['content_review', 'text_analysis'],
  'system_instructions' => 'You are a content reviewer who checks for quality issues.',
]);
```

### Executing an Agent

```php
// Get the agent factory adapter
$agent_factory = \Drupal::service('drupal_native_ai.adapter.agent_factory');

// Execute an agent
$result = $agent_factory->executeAgent('content-reviewer-1', [
  'content' => 'This is the content to review.',
  'guidelines' => 'Check for spelling, grammar, and clarity.',
]);
```

## Integration with AI Providers

AI providers in the Drupal Native AI module can implement agent-related methods to be used by the AgentFactoryAdapter:

- `createAgent($config)`
- `updateAgent($agent_id, $config)`
- `deleteAgent($agent_id)`
- `getAgent($agent_id)`
- `getAgents($filters)`
- `executeAgent($agent_id, $input, $options)`
- `validateAgentConfig($config)`

These methods are called dynamically by the adapter based on the requested operation.
