---
id: modules-development-native_ai-MODULE_INTEGRATION_GUIDE
---
# Module Integration Guide

This guide provides detailed instructions for integrating Drupal Native AI with different modules in the Drupal AI ecosystem.

## Contents

1. [LLM Module Integration](#llm-module-integration)
2. [AI Swarm Intelligence Integration](#ai-swarm-intelligence-integration)
3. [Langchain Integration](#langchain-integration)
4. [Field ECA Bridge Integration](#field-eca-bridge-integration)
5. [API Normalization Integration](#api-normalization-integration)
6. [Adding Integration for a New Module](#adding-integration-for-a-new-module)

## LLM Module Integration

### Overview

The LLM module provides a framework for integrating large language models into Drupal. Drupal Native AI integrates with the LLM module by implementing the `LlmProviderInterface` and registering as a provider.

### Implementation

1. **Adapter Class**: `Drupal\drupal_native_ai\Adapter\LlmModuleAdapter`
2. **Interface Implemented**: `Drupal\llm\Plugin\Ai\Provider\LlmProviderInterface`
3. **Operations Supported**:
   - `generate()`: Text generation
   - `chat()`: Chat completion
   - `generateImage()`: Image generation (via fallback)
   - `textToSpeech()`: Text to speech conversion (via fallback)
   - `speechToText()`: Speech to text conversion (via fallback)
   - `generateEmbeddings()`: Embedding generation
   - `moderateContent()`: Content moderation
   - `translateText()`: Text translation (via fallback)
   - `classifyImage()`: Image classification (via fallback)

### Configuration

In the Drupal Native AI settings (`/admin/config/ai/native/settings`), you can configure:

1. Whether to use the native provider for each LLM operation
2. Which fallback provider to use for each operation
3. Whether to enable integration with the LLM module

### Example Usage

```php
// Get the LLM platform service from the container
$llm_platform = \Drupal::service('llm.platform');

// Generate text using the configured provider (will use Drupal Native AI if configured)
$text = $llm_platform->generate('Tell me about Drupal 11');

// Explicitly use the Drupal Native AI provider
$text = $llm_platform->generate('Tell me about Drupal 11', [
  'provider' => 'drupal_native_ai',
]);
```

## AI Swarm Intelligence Integration

### Overview

The AI Swarm Intelligence module provides advanced pattern detection and emergent behavior analysis. Drupal Native AI integrates with this module to provide basic pattern analysis capabilities without requiring external AI services.

### Implementation

1. **Adapter Class**: `Drupal\drupal_native_ai\Adapter\SwarmIntelligenceAdapter`
2. **Operations Supported**:
   - `analyzePatternEmergence()`: Analyzes emerging patterns in data
   - `analyzeSwarmBehavior()`: Analyzes swarm behavior patterns
   - `optimizeSwarmConfiguration()`: Optimizes swarm configuration
   - `identifyEmergentPatterns()`: Identifies emergent patterns

### Configuration

In the Drupal Native AI settings (`/admin/config/ai/native/settings`), you can configure:

1. Whether to use the native provider for each swarm operation
2. Which fallback provider to use for each operation
3. Whether to enable integration with the AI Swarm Intelligence module

### Example Usage

```php
// Get the swarm intelligence service from the container
$swarm_service = \Drupal::service('ai_swarm_intelligence.service');

// Analyze patterns using the configured provider (will use Drupal Native AI if configured)
$patterns = $swarm_service->analyzePatterns($data);

// Identify emergent patterns with a specific threshold
$emergent_patterns = $swarm_service->identifyEmergentPatterns($data, 70);
```

## Langchain Integration

### Overview

The Langchain module provides orchestration of AI components into workflows and chains. Drupal Native AI integrates with Langchain to provide basic capabilities without requiring external AI services.

### Implementation

1. **Adapter Class**: `Drupal\drupal_native_ai\Adapter\LangchainAdapter`
2. **Operations Supported**:
   - `executePrompt()`: Executes a prompt
   - `orchestrateChain()`: Orchestrates a chain (via fallback)
   - `createEmbeddings()`: Creates embeddings
   - `executeAgent()`: Executes an agent (via fallback)

### Configuration

In the Drupal Native AI settings (`/admin/config/ai/native/settings`), you can configure:

1. Whether to use the native provider for each Langchain operation
2. Which fallback provider to use for each operation
3. Whether to enable integration with the Langchain module

### Example Usage

```php
// Get the Langchain service from the container
$langchain_service = \Drupal::service('langchain.service');

// Execute a prompt using the configured provider (will use Drupal Native AI if configured)
$result = $langchain_service->executePrompt('Tell me about Drupal 11');

// Create embeddings for a text
$embeddings = $langchain_service->createEmbeddings('This is a sample text');
```

## Field ECA Bridge Integration

### Overview

The Field ECA Bridge module connects custom fields with the Event-Condition-Action system, enabling powerful, high-performance data storage with automated workflows and business rule automation. Drupal Native AI integrates with this module to provide AI-powered field processing, validation, transformation, and intelligence for automated workflows.

### Implementation

1. **Adapter Class**: `Drupal\drupal_native_ai\Adapter\FieldEcaBridgeAdapter`
2. **Interface Implemented**: `Drupal\field_eca_bridge\FieldAiProcessorInterface`
3. **Operations Supported**:
   - `processFieldValue()`: Processes field values with AI intelligence
   - `validateFieldValue()`: Validates field values with AI-powered validation
   - `transformFieldValue()`: Transforms field values based on AI analysis
   - `selectOptimalModel()`: Selects the optimal AI model based on requirements
   - `configureAgent()`: Configures an AI agent based on model capabilities
   - `analyzeEntityFields()`: Analyzes multiple fields on an entity
   - `calculateFieldValue()`: Performs AI-driven calculations on field values

### Configuration

In the Drupal Native AI settings (`/admin/config/ai/native/settings`), you can configure:

1. Whether to use the native provider for each field ECA bridge operation
2. Which fallback provider to use for each operation
3. Whether to enable integration with the Field ECA Bridge module

### Example Usage

```php
// Get the Field ECA Bridge AI processor service from the container
$field_eca_ai = \Drupal::service('field_eca_bridge.ai_processor');

// Process a field value using the configured provider
$processed_value = $field_eca_ai->processFieldValue('field value data', [
  'field_name' => 'field_content',
  'entity_type' => 'node',
  'bundle' => 'article',
]);

// Validate a field value against defined constraints
$is_valid = $field_eca_ai->validateFieldValue('field value data', [
  'field_name' => 'field_sensitive_content',
  'constraints' => ['no_profanity', 'appropriate_content'],
]);

// Select the optimal model for a particular task
$model = $field_eca_ai->selectOptimalModel([
  'vision_support' => TRUE,
  'context_window' => 16000,
  'embeddings_support' => TRUE,
], ['task' => 'document_analysis']);

// Configure an agent based on model capabilities
$agent_config = $field_eca_ai->configureAgent([
  'vision_support' => TRUE,
  'context_window' => 32000,
  'tool_support' => TRUE,
], ['task' => 'content_moderation']);

// Analyze multiple fields on an entity
$node = Node::load(123);
$analysis = $field_eca_ai->analyzeEntityFields($node, [
  'field_body', 
  'field_image',
  'field_tags',
]);

// Calculate a field value based on multiple inputs
$calculated_value = $field_eca_ai->calculateFieldValue([
  'value1' => 10,
  'value2' => 20,
  'value3' => 5,
], 'weighted_average', ['weights' => [0.5, 0.3, 0.2]]);
```

### Use Cases

1. **Intelligent Content Validation**: Validate that content meets guidelines before saving
2. **Smart Field Processing**: Transform user input into structured data
3. **Model Selection Workflows**: Select optimal AI models based on content needs
4. **Agent Configuration**: Automatically configure agents based on model capabilities
5. **Entity Analysis**: Analyze multiple fields on an entity for patterns and insights
6. **Field Calculations**: Perform AI-driven calculations on field values

## API Normalization Integration

### Overview

The API Normalization module provides standardized API integration with intelligent schema discovery, field mapping, and transformation capabilities. Drupal Native AI integrates with this module to provide AI-powered schema analysis, semantic field matching, relationship detection, and mapping suggestions without requiring external commercial AI services.

### Implementation

1. **Adapter Class**: `Drupal\drupal_native_ai\Adapter\ApiNormalizationAdapter`
2. **Operations Supported**:
   - `analyzeSchema()`: Analyzes a schema to detect field types, relationships, and patterns
   - `generateMappings()`: Generates semantic field mappings based on source and target schemas
   - `analyzeSemantics()`: Analyzes semantics of text content
   - `detectRelationships()`: Detects entity relationships in a schema
   - `enhanceMapping()`: Enhances field mappings with AI-powered suggestions

### Configuration

In the Drupal Native AI settings (`/admin/config/ai/native/settings`), you can configure:

1. Whether to use the native provider for each API normalization operation
2. Which fallback provider to use for each operation
3. Whether to enable integration with the API Normalization module

### Example Usage

```php
// Get the API Normalization services from the container
$api_normalization = \Drupal::service('api_normalization.ai_provider');
$ai_entity_deriver = \Drupal::service('ai_entity_deriver.provider');

// Analyze OpenAPI schema to detect field types and relationships
$schema = [
  'components' => [
    'schemas' => [
      'User' => [
        'properties' => [
          'email' => ['type' => 'string', 'format' => 'email'],
          'name' => ['type' => 'string'],
        ],
      ],
    ],
  ],
];

$analysis = $api_normalization->analyzeSchema($schema, ['source' => 'openapi']);

// Generate mappings between source and target schemas
$source_schema = [
  'User' => [
    'fields' => [
      'email' => ['type' => 'string', 'format' => 'email'],
      'full_name' => ['type' => 'string'],
    ],
  ],
];

$target_schema = [
  'user' => [
    'fields' => [
      'mail' => ['type' => 'email'],
      'name' => ['type' => 'string'],
    ],
  ],
];

$mappings = $api_normalization->generateMappings($source_schema, $target_schema);

// Analyze text semantics for entity mapping
$semantics = $api_normalization->analyzeSemantics('This field contains user email addresses for notifications');

// Detect relationships between entities
$relationships = $api_normalization->detectRelationships([
  'User' => ['fields' => ['id' => ['type' => 'integer']]],
  'Post' => ['fields' => ['user_id' => ['type' => 'integer']]],
]);

// Enhance mapping with AI-powered suggestions
$enhanced_mapping = $api_normalization->enhanceMapping($initial_mapping);
```

### Integration with AI Entity Deriver

The adapter also integrates with the AI Entity Deriver submodule to provide enhanced entity type discovery and field mapping capabilities:

```php
// Use the AI Entity Deriver to discover entities in an OpenAPI schema
$attributes = $ai_entity_deriver->detectAttributes($schema);

// Generate semantic mappings for attributes
$semantic_mappings = $ai_entity_deriver->generateSemanticMappings($attributes);

// Analyze complex patterns in schema structures
$complex_patterns = $ai_entity_deriver->analyzeComplexPatterns($schema);
```

### Use Cases

1. **API Schema Analysis**: Intelligently analyze and understand API schemas
2. **Field Mapping Generation**: Generate semantic mappings between source and target fields
3. **Relationship Detection**: Identify relationships between entities in a schema
4. **Mapping Enhancement**: Enhance field mappings with AI-powered suggestions
5. **Semantic Analysis**: Analyze text content for semantic patterns and insights
6. **Schema Discovery**: Automatically discover and analyze API schemas
7. **Entity Deriving**: Generate Drupal entity definitions from external APIs

## Adding Integration for a New Module

To add Drupal Native AI integration for a new module:

### Step 1: Create a New Adapter Class

Create a new adapter class that extends `NativeAiAdapterBase` and implements the module's provider interface:

```php
<?php

namespace Drupal\drupal_native_ai\Adapter;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\State\StateInterface;
use Drupal\drupal_native_ai\Service\DrupalNativeAIProvider;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Adapter for the Example module.
 */
class ExampleModuleAdapter extends NativeAiAdapterBase implements ExampleProviderInterface, ContainerInjectionInterface {

  // Implement methods from ExampleProviderInterface...
  
  /**
   * {@inheritdoc}
   */
  protected function loadFallbackProviderService(string $provider_id): ?object {
    // Load the fallback provider for this module...
  }
  
  // Add module-specific methods here...
}
```

### Step 2: Register the Adapter as a Service

Add a service definition in `drupal_native_ai.services.yml`:

```yaml
  drupal_native_ai.example_adapter:
    class: Drupal\drupal_native_ai\Adapter\ExampleModuleAdapter
    arguments:
      - '@drupal_native_ai.provider'
      - '@config.factory'
      - '@logger.factory'
      - '@state'
    tags:
      - { name: 'example_module.provider' }
```

### Step 3: Add Module Integration Configuration

Update the default configuration in `config/install/drupal_native_ai.settings.yml`:

```yaml
module_integrations:
  example_module:
    enabled: false
    adapter: 'Drupal\drupal_native_ai\Adapter\ExampleModuleAdapter'
```

### Step 4: Update the Settings Form

Update the settings form in `src/Form/SettingsForm.php` to include the new module:

```php
$modules = [
  // Existing modules...
  'example_module' => $this->t('Example Module'),
];

foreach ($modules as $module_id => $module_label) {
  // Add form elements for the new module...
}
```

### Step 5: Implement Module-Specific Operations

Implement the operations required by the module's provider interface, using the native AI provider when appropriate and falling back to other providers when necessary.

### Step 6: Add Documentation

Update the documentation to include information about the new module integration:

1. Add a section to this guide
2. Update the integration pattern documentation
3. Add examples and usage instructions

### Step 7: Create Tests

Create tests for the new adapter to ensure it works correctly:

1. Unit tests for the adapter class
2. Integration tests with the module
3. Functional tests for the configuration UI
