---
id: modules-development-native_ai-PROVIDER_AUTO_DETECTION
---
# Provider Auto-Detection System

The Provider Auto-Detection System is a powerful feature of Drupal Native AI that automatically detects and configures AI providers in your Drupal site.

## Overview

The system scans your Drupal installation for available AI providers, determines their status, and helps you configure them optimally while maintaining Drupal Native AI as the default provider. This ensures that your site uses native AI capabilities whenever possible, with commercial providers acting as optional fallbacks.

## Provider Detection

The system detects the following types of providers:

1. **Native Providers**
   - Drupal Native AI (always available)

2. **Commercial Providers**
   - OpenAI (via LLM module)
   - Anthropic (via LLM module)
   - Hugging Face

3. **Local Providers**
   - Ollama

4. **Framework Providers**
   - LangChain

For each provider, the system determines:
- Availability status (configured, available, missing dependency)
- Module providing the provider
- Configuration requirements
- Capabilities and supported operations

## Provider Configuration

The system offers three setup types:

1. **Recommended Setup**
   - Drupal Native AI as the primary provider
   - Commercial providers as fallbacks for specialized operations
   - Native provider forced for operations it can handle well

2. **Hybrid Setup**
   - Mix of native and commercial providers based on capabilities
   - Best provider selected for each operation
   - Native provider not forced

3. **Minimal Setup**
   - Only Drupal Native AI without external providers
   - Native provider forced for all operations

## Operation Mapping

The system automatically maps operations to appropriate providers based on:

1. **Operation Requirements**
   - Required capabilities for each operation
   - Provider capabilities
   - Operation preferences

2. **Provider Capabilities**
   - Text generation
   - Image generation
   - Speech operations
   - Embeddings
   - Translation
   - Content moderation
   - Pattern analysis
   - Entity learning

3. **Provider Preferences**
   - Native provider preferences (operations it handles well)
   - Commercial provider specialties

## Usage

### Web Interface

The Provider Auto-Detection System is integrated into the Setup Wizard at `/admin/config/ai/native/setup`.

### Drush Commands

```bash
# Detect available providers
drush drupal_native_ai:detect-providers

# Configure providers with recommended setup
drush drupal_native_ai:configure-providers recommended

# Configure providers with hybrid setup (no forcing native)
drush drupal_native_ai:configure-providers hybrid --no-force-native

# Configure providers with specific providers
drush drupal_native_ai:configure-providers recommended --providers=drupal_native_ai,llm.openai,ollama
```

### Shell Script

The module includes a shell script for easier command-line configuration:

```bash
# Basic usage
./scripts/configure-providers.sh

# With options
./scripts/configure-providers.sh --type hybrid --no-force --providers drupal_native_ai,llm.openai,ollama
```

### API Usage

The Provider Detection Service can be used programmatically:

```php
// Get the service
$provider_detection = \Drupal::service('drupal_native_ai.provider_detection');

// Detect providers
$providers = $provider_detection->detectProviders();

// Get recommended providers for a setup type
$recommended = $provider_detection->getRecommendedProviders('recommended');

// Configure providers
$result = $provider_detection->configureProviders(
  'recommended',     // Setup type
  $recommended,      // Providers to enable
  TRUE,              // Force native provider
  TRUE               // Auto-configure fallbacks
);
```

## Benefits

1. **Simplified Setup**
   - Automatic provider detection reduces manual configuration
   - Smart defaults based on available providers
   - Guided setup process

2. **Optimal Configuration**
   - Native provider used whenever possible
   - Commercial providers used only when needed
   - Operation-specific provider selection

3. **Flexibility**
   - Multiple setup types for different needs
   - Command-line and API access
   - Fine-grained control over provider selection

4. **Integration**
   - Works seamlessly with other Drupal modules
   - Integrates with LLM module
   - Supports various provider types

## Technical Details

The Provider Auto-Detection System is implemented in the following components:

1. **ProviderDetectionService**
   - Core service for provider detection and configuration
   - Scans for available providers
   - Configures providers based on setup type
   - Maps operations to appropriate providers

2. **Drush Commands**
   - Command-line interface for provider detection and configuration
   - Formatted output for provider information
   - Options for different setup types and provider selection

3. **Setup Wizard**
   - Web interface for guided provider configuration
   - Provider selection with status indicators
   - Setup type selection and configuration

4. **Provider Status Block**
   - Visual representation of configured providers
   - Status indicators for provider availability
   - Quick access to provider configuration

## Extending the System

To add support for new providers:

1. Implement the `AiProviderInterface`
2. Register your provider with the `ai.provider` service tag
3. Add provider information to the `ProviderDetectionService::detectProviders()` method
4. Define provider capabilities and preferred operations
5. Update the operation capabilities mapping if needed
