---
id: modules-development-native_ai-README
---
# Drupal Native AI

[![Status](https://img.shields.io/badge/status-production-green)](https://gitlab.bluefly.io/llm/drupal/drupal_native_ai/-/commits/main)
[![Version](https://img.shields.io/badge/version-1.0.0-blue)](CHANGELOG.md)
[![Test Coverage](https://img.shields.io/badge/coverage-95%25-green)](coverage/)
[![License](https://img.shields.io/badge/license-GPL--2.0-orange)](LICENSE)
[![OpenAPI](https://img.shields.io/badge/OpenAPI-3.1-blue)](openapi.yaml)
[![TDD](https://img.shields.io/badge/TDD-compliant-success)](../../TDD-ENFORCEMENT-POLICY.md)
[![Drupal](https://img.shields.io/badge/Drupal-11.x-blue)](composer.json)
[![PHP](https://img.shields.io/badge/PHP-8.2+-purple)](composer.json)

> **TODO: Drupal.org Compatibility Roadmap**  
> We are working to make this module compatible with drupal.org standards by:  
> 
> **Action Items:**
> 
> 1. **Resolve circular dependency with the `llm` module**
>    - Move `llm` from required to optional dependencies in the .info.yml file
>    - Update service definitions to check for llm module availability
>    - Create fallback implementations for when llm is not available
>
> 2. **Implement shared interfaces within the module**
>    - Create `src/Interface` directory for common interfaces
>    - Define or import core interfaces like `ProviderInterface`, `ModelInterface`
>    - Update services to type-hint against interfaces instead of concrete classes
>
> 3. **Implement service discovery patterns**
>    - Create a `ServiceDiscovery` class to check for module and service availability
>    - Add graceful degradation when optional services aren't available
>    - Use dependency injection and factory patterns for flexible service creation
>
> 4. **Create module-specific recipe**
>    - Create `drupal_native_ai.recipe.yml` for standalone installation
>    - Define configuration actions for default settings
>    - Document recipe usage in installation instructions
>
> See the `/todos` directory for detailed implementation plans and examples.

## Overview

Enterprise Drupal modules implementing AI functionality with OpenAPI 3.1 compliance. Provides unified AI model integration, distributed processing, and field processing capabilities through a modular, recipe-based architecture.

## Features

### Core Capabilities
- **AI Integration**: Unified model provider interface
- **Swarm Intelligence**: Distributed AI processing
- **Field Processing**: ECA-based field automation
- **Security**: Enterprise-grade security framework
- **Testing**: 95%+ test coverage with TDD

### Advanced Features
- **Content Generation**: Experience Builder integration
- **Migration Tools**: AI-powered content migration
- **API Standardization**: OpenAPI 3.1 compliance
- **Multi-tenancy**: Secure multi-tenant support
- **Blockchain**: Secure transaction logging

## Architecture

### Core Components
- **AI Service**: Model provider integration
- **Swarm System**: Distributed processing
- **Field Bridge**: ECA field automation
- **API Layer**: OpenAPI 3.1 endpoints
- **Security**: Access control and audit

### Platform Integration
```php
// Integration stack
Drupal → AI Service → Model Providers
Drupal → Swarm System → Distributed Processing
Drupal → Field Bridge → Content Automation
Drupal → OpenAPI → API Gateway
```

### Infrastructure
- **Containerization**: DDEV support
- **Database**: PostgreSQL with extensions
- **Testing**: PHPUnit with TDD
- **CI/CD**: GitLab CI templates
- **Documentation**: OpenAPI integration

## Module Structure

### Core AI Integration
- [drupal_native_ai](drupal_native_ai/): AI model integration
  - Submodules:
    - **drupal_native_ai_ui_patterns**: UI Patterns integration for consistent interfaces
    - **drupal_native_ai_opensource**: Open Source AI model integration
    - **drupal_native_ai_openai**: OpenAI provider integration
    - **drupal_native_ai_anthropic**: Anthropic provider integration
    - **drupal_native_ai_huggingface**: HuggingFace provider integration
    - **drupal_native_ai_ollama**: Ollama provider integration
    - **drupal_native_ai_langchain**: LangChain integration
    - **drupal_native_ai_nativeaijs**: Native AI.js integration
- [ai_swarm_intelligence](ai_swarm_intelligence/): Distributed processing
- [field_eca_bridge](field_eca_bridge/): Field processing

### Experience and Content
- [experience_builder_converter](experience_builder_converter/): Content generation
- [drupal_migration_plus](drupal_migration_plus/): Migration tools
- [api_normalization](api_normalization/): API standardization

### Infrastructure and Security
- [secure_drupal](secure_drupal/): Security features
- [multitenancy](multitenancy/): Multi-tenant support
- [blockchain_manager](blockchain_manager/): Blockchain integration

### Integration and Services
- [marketplace_integration](marketplace_integration/): Marketplace connectivity
- [mcp_client_extras](mcp_client_extras/): MCP extensions
- [alternative_services](alternative_services/): Service alternatives

### Recipes and Templates
- [recipe_onboarding](recipe_onboarding/): Installation recipes
- Marketplace templates

## Prerequisites

### Core Requirements
- PHP 8.2 or later
- Drupal 11.x
- Composer 2.x
- DDEV (for local development)
- PostgreSQL 15+ (for vector support)

### Optional Requirements
- Redis (for caching)
- Elasticsearch (for search)
- WebSocket server
- Vector database providers

## Module Reorganization

### Submodule Integration (June 2025)

As part of our effort to standardize the module structure and improve maintainability, we've integrated the following standalone modules as submodules of the main drupal_native_ai module:

1. **drupal_native_ai_ui_patterns**
   - Provides UI Patterns integration with Bluefly UI React components
   - Enables consistent pattern library, documentation, and benchmarking
   - Supports lazy loading and responsive design patterns
   - Previously existed as a standalone module in `_DrupalModules/drupal_native_ai_ui_patterns`

2. **drupal_native_ai_opensource**
   - Provides integration with open-source AI models (Ollama, LocalAI, etc.)
   - Enables AI functionality without commercial API keys
   - Supports text generation, chat completions, and streaming
   - Previously existed as a standalone module in `_DrupalModules/drupal_native_ai_opensource`

This integration simplifies the codebase, reduces duplication, and provides a clearer structure for development. For backward compatibility, the previous directory structure is still supported temporarily but is deprecated and will be removed in a future update.

## Installation

### Quick Start

```bash
# Install via Composer
composer require bluefly/drupal_native_ai

# Enable module
drush en drupal_native_ai

# Configure AI providers
drush config:set drupal_native_ai.settings providers.openai_api_key "your-key"

# Enable real-time features
drush config:set drupal_native_ai.settings realtime.enabled true
```

### Production Setup

```bash
# Install with all dependencies
composer require bluefly/drupal_native_ai --with-all-dependencies

# Apply recipe
drush recipe:apply drupal_native_ai

# Setup production
drush drupal_native_ai:setup-wizard --environment=production

# Configure OpenAPI
drush drupal_native_ai:openapi:generate
drush drupal_native_ai:openapi:validate
```

## Usage

### Module Integration

```php
// Example: Using the AI service in a custom module
use Drupal\drupal_native_ai\Service\AIService;

$ai_service = \Drupal::service('drupal_native_ai.service');
$result = $ai_service->process($input, [
  'openapi' => [
    'spec' => 'openapi.yaml',
    'validate' => true
  ]
]);
```

### Recipe Integration

```yaml
# Example: Including in a Drupal recipe
name: AI-Enabled Site
type: recipe
install:
  - drupal_native_ai
  - ai_swarm_intelligence
config:
  import:
    - drupal_native_ai.settings
    - drupal_native_ai.openapi
```

### Template Integration

```yaml
# Example: Including in a marketplace template
template:
  name: AI-Powered Site
  type: template
  modules:
    - drupal_native_ai
    - experience_builder_converter
  config:
    - drupal_native_ai.settings
    - drupal_native_ai.openapi
```

## Development

### Setup

```bash
# Install development dependencies
composer install --dev

# Setup development environment
./setup-ddev.sh

# Start development server
ddev start
```

### Testing

```bash
# Run all tests
ddev run-tests

# Run specific test suites
ddev run-tests Unit
ddev run-tests Kernel
ddev run-tests Functional

# Run with PHPUnit
./vendor/bin/phpunit -c phpunit.ddev.xml tests/src/Unit/

# Run TDD workflow
drush recipe:tdd:gen    # Generate test skeletons
drush recipe:tdd:verify # Verify tests fail as expected
drush recipe:tdd:check  # Check implementation meets tests
drush recipe:tdd:install # Install and validate in environment
```

### Building

```bash
# Build for production
drush recipe:build

# Build documentation
drush recipe:build-docs

# Build OpenAPI spec
drush recipe:build-api
```

## Contributing

See [CONTRIBUTING.md](CONTRIBUTING) for development guidelines.

## Security

- Report security issues to security@bluefly.io
- Follow our [Security Policy](SECURITY)
- Review [Security Guidelines](docs/security/guidelines)

## License

GPL-2.0 License - see [LICENSE](LICENSE) for details

## Support

- [Documentation](https://docs.bluefly.io/drupal-native-ai)
- [Issue Queue](https://gitlab.bluefly.io/llm/drupal/drupal_native_ai/-/issues)
- [Community Support](https://community.bluefly.io)
- [API Reference](https://api.bluefly.io/drupal-native-ai/docs)

## Credits

- [Bluefly.io](https://bluefly.io)
- [Contributors](https://gitlab.bluefly.io/llm/drupal/drupal_native_ai/-/graphs/main)
- [Drupal](https://www.drupal.org)
- [OpenAPI](https://www.openapis.org)

---

The Native AI modules provide AI integration for Drupal 11 with OpenAPI compliance and field automation capabilities.

## Streaming Service

This module implements a streaming service for AI operations in Drupal, providing:
- Response streaming
- Metrics collection
- State management
- Error handling
- Integration with Drupal's state and config systems
- Configurable metrics and thresholds

### Features

- Response streaming implementation
- Metrics collection (tokens, latency, cost)
- State management for streaming operations
- Error handling and logging
- Integration with Drupal's state and config systems
- Configurable metrics and thresholds

## Migration Guide

### For Module Developers

To integrate AI streaming functionality into your module:

1. **Add Service Dependency**

   In your module's service definition (`your_module.services.yml`):

   ```yaml
   services:
     your_module.ai_service:
       class: Drupal\your_module\Service\YourAiService
       arguments: ['@ai.streaming', '@logger.factory']
   ```

2. **Use the Streaming Trait**

   In your service class:

   ```php
   use Drupal\drupal_native_ai\Service\Trait\AiStreamingTrait;

   class YourAiService {
     use AiStreamingTrait;

     public function __construct(
       AiStreamingServiceInterface $streaming_service,
       LoggerChannelFactoryInterface $logger_factory
     ) {
       $this->setStreamingService($streaming_service);
       $this->logger = $logger_factory->get('your_module');
     }

     public function streamChat(array $messages, array $options = []): StreamedResponse {
       return $this->createStreamingResponse(function ($chunkHandler) use ($messages, $options) {
         // Your streaming logic here
         foreach ($messages as $message) {
           // Process message
           $chunkHandler($content, [
             'tokensUsed' => $tokens,
             'cost' => $cost,
           ]);
         }
       }, $options);
     }
   }
   ```

3. **Handle State and Metrics**

   The streaming service provides state and metrics objects:

   ```php
   $state = $this->createStreamingState();
   $metrics = $this->createStreamingMetrics();

   // Update state
   $state->setStatus('streaming');
   $state->appendContent($chunk);

   // Update metrics
   $metrics->setTokensUsed($tokens);
   $metrics->setLatency($latency);
   $metrics->setCost($cost);

   // Add custom metrics
   $metrics->setCustomMetric('custom_key', $value);
   ```

4. **Error Handling**

   The streaming service handles errors automatically, but you can also handle them in your code:

   ```php
   try {
     // Your streaming logic
   } catch (\Exception $e) {
     $this->logStreamingError($e->getMessage(), [
       'context' => $context,
     ]);
     throw $e;
   }
   ```

### Configuration

The streaming service supports configuration through Drupal's config system:

```php
// Get config
$config = \Drupal::config('drupal_native_ai.settings');

// Set thresholds
$config->set('metrics_thresholds', [
  'max_tokens' => 1000000,
  'max_cost' => 1000.0,
])->save();
```

### Metrics and Monitoring

The service automatically tracks and logs metrics:

- Tokens used
- Operation latency
- Cost in credits
- Custom metrics

Metrics are stored in Drupal's state system and can be accessed for monitoring:

```php
$metrics = \Drupal::state()->get('drupal_native_ai.streaming_metrics', []);
```

### Best Practices

1. **Always Use the Trait**

   The `AiStreamingTrait` provides a consistent interface and handles common functionality.

2. **Handle State Properly**

   Use the state object to track operation progress and status.

3. **Track Metrics**

   Always provide metrics for tokens, latency, and cost when streaming.

4. **Error Handling**

   Use the provided error handling and logging methods.

5. **Custom Headers**

   Use the options array to set custom headers when needed:

   ```php
   $options = [
     'headers' => [
       'X-Custom-Header' => 'value',
     ],
   ];
   ```

## API Reference

### StreamingServiceInterface

- `createStreamingChatResponse(callable $streamCallback, array $options = []): StreamedResponse`
- `createStreamingState(): AiOperationStateInterface`
- `createStreamingMetrics(): StreamingMetricsInterface`

### AiOperationStateInterface

- `getStatus(): string`
- `setStatus(string $status): void`
- `getError(): ?string`
- `setError(?string $error): void`
- `getAccumulatedContent(): string`
- `setAccumulatedContent(string $content): void`
- `isComplete(): bool`
- `setComplete(bool $complete): void`
- `getMessages(): array`
- `setMessages(array $messages): void`
- `getMetrics(): ?array`
- `setMetrics(?array $metrics): void`

### StreamingMetricsInterface

- `getTokensUsed(): int`
- `setTokensUsed(int $tokens): void`
- `getLatency(): float`
- `setLatency(float $latency): void`
- `getCost(): float`
- `setCost(float $cost): void`
- `toArray(): array` 
