---
id: modules-development-native_ai-TESTING_STANDARDS
---
# Testing Standards for Drupal Native AI

This document outlines the testing standards and methodologies for the Drupal Native AI module.

## Testing Structure

The module follows Drupal's standard testing structure:

```
drupal_native_ai/
├── tests/
│   ├── src/
│   │   ├── Unit/               # Unit tests
│   │   ├── Kernel/             # Kernel tests
│   │   ├── Functional/         # Functional tests
│   │   └── FunctionalJavascript/ # JavaScript-enabled tests
│   └── fixtures/               # Test fixtures
```

## Test Types

### Unit Tests

Unit tests focus on testing individual components in isolation:

- Service methods
- Plugin implementations
- Utility functions
- Helper classes

Example unit test:

```php
namespace Drupal\Tests\drupal_native_ai\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\drupal_native_ai\Utility\TokenCounter;

/**
 * @group drupal_native_ai
 * @coversDefaultClass \Drupal\drupal_native_ai\Utility\TokenCounter
 */
class TokenCounterTest extends UnitTestCase {

  /**
   * Tests the counting of tokens in a text string.
   */
  public function testTokenCount() {
    $text = "This is a sample text with 10 tokens.";
    $counter = new TokenCounter();
    $count = $counter->countTokens($text);
    
    $this->assertEquals(10, $count);
  }
}
```

### Kernel Tests

Kernel tests validate interaction with Drupal's API:

- Config schemas
- Database operations
- Service interactions
- Plugin discovery and execution

Example kernel test:

```php
namespace Drupal\Tests\drupal_native_ai\Kernel;

use Drupal\KernelTests\KernelTestBase;

/**
 * @group drupal_native_ai
 */
class AiProviderManagerTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'field',
    'drupal_native_ai',
  ];

  /**
   * The AI provider manager.
   *
   * @var \Drupal\drupal_native_ai\AiProviderManagerInterface
   */
  protected $providerManager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installConfig(['drupal_native_ai']);
    $this->providerManager = $this->container->get('plugin.manager.drupal_native_ai_provider');
  }

  /**
   * Tests provider discovery.
   */
  public function testProviderDiscovery() {
    $definitions = $this->providerManager->getDefinitions();
    $this->assertNotEmpty($definitions);
    $this->assertArrayHasKey('default', $definitions);
  }
}
```

### Functional Tests

Functional tests validate end-to-end functionality:

- Admin UI functionality
- Form submissions
- Integration with other modules
- End-to-end workflows

Example functional test:

```php
namespace Drupal\Tests\drupal_native_ai\Functional;

use Drupal\Tests\BrowserTestBase;

/**
 * @group drupal_native_ai
 */
class AiSettingsFormTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'drupal_native_ai',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Tests the AI settings form.
   */
  public function testSettingsForm() {
    $admin_user = $this->drupalCreateUser(['administer drupal_native_ai']);
    $this->drupalLogin($admin_user);
    
    $this->drupalGet('admin/config/ai/drupal_native_ai');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Drupal Native AI Settings');
    
    // Test form submission
    $edit = [
      'default_provider' => 'default',
      'enable_streaming' => TRUE,
    ];
    $this->submitForm($edit, 'Save configuration');
    $this->assertSession()->pageTextContains('The configuration options have been saved.');
  }
}
```

### Functional JavaScript Tests

JavaScript-enabled tests validate UI interactions:

- AJAX functionality
- Interactive UI elements
- Real-time updates
- Client-side validation

Example functional JavaScript test:

```php
namespace Drupal\Tests\drupal_native_ai\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * @group drupal_native_ai
 */
class AiStreamingTest extends WebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'drupal_native_ai',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Tests the streaming functionality in the UI.
   */
  public function testStreamingUI() {
    $admin_user = $this->drupalCreateUser(['administer drupal_native_ai']);
    $this->drupalLogin($admin_user);
    
    $this->drupalGet('admin/config/ai/drupal_native_ai/test-streaming');
    $this->assertSession()->pageTextContains('Streaming Test');
    
    // Click the start streaming button
    $this->click('input[name="start_streaming"]');
    
    // Wait for streaming to start
    $this->assertWaitOnAjaxRequest();
    
    // Verify streaming content appears
    $this->assertSession()->elementExists('css', '.streaming-content');
    $this->assertSession()->elementTextContains('css', '.streaming-content', 'Streaming content');
    
    // Wait for streaming to complete
    $this->assertSession()->waitForText('Streaming complete');
  }
}
```

## Running Tests

### Running All Tests

To run all tests for the module:

```bash
./vendor/bin/phpunit -c core/phpunit.xml modules/contrib/drupal_native_ai
```

### Running Specific Test Suites

To run specific test suites:

```bash
# Unit tests only
./vendor/bin/phpunit -c core/phpunit.xml modules/contrib/drupal_native_ai/tests/src/Unit

# Kernel tests only
./vendor/bin/phpunit -c core/phpunit.xml modules/contrib/drupal_native_ai/tests/src/Kernel

# Functional tests only
./vendor/bin/phpunit -c core/phpunit.xml modules/contrib/drupal_native_ai/tests/src/Functional

# Functional JavaScript tests only
./vendor/bin/phpunit -c core/phpunit.xml modules/contrib/drupal_native_ai/tests/src/FunctionalJavascript
```

### Running Individual Tests

To run individual test classes:

```bash
./vendor/bin/phpunit -c core/phpunit.xml modules/contrib/drupal_native_ai/tests/src/Unit/TokenCounterTest.php
```

To run specific test methods:

```bash
./vendor/bin/phpunit -c core/phpunit.xml --filter testTokenCount modules/contrib/drupal_native_ai/tests/src/Unit/TokenCounterTest.php
```

## Test Coverage

The module aims for comprehensive test coverage:

- Core services: 100% coverage
- Plugin implementations: 100% coverage
- UI components: 90%+ coverage
- Integration points: 90%+ coverage

To generate a test coverage report:

```bash
XDEBUG_MODE=coverage ./vendor/bin/phpunit -c core/phpunit.xml --coverage-html ./coverage modules/contrib/drupal_native_ai
```

## AI-Specific Testing Considerations

Testing AI-related functionality presents unique challenges:

### Mocking AI Providers

For unit and kernel tests, AI providers should be mocked to avoid external dependencies:

```php
$mock_provider = $this->createMock(AiProviderInterface::class);
$mock_provider->method('process')
  ->willReturn(new AiResult('Mocked response', ['tokens' => 10]));
```

### Testing Streaming Responses

For streaming tests, use predictable response streams:

```php
$mock_streaming_service = $this->createMock(StreamingServiceInterface::class);
$mock_streaming_service->method('stream')
  ->willReturnCallback(function ($callback) {
    $callback('Chunk 1', 10);
    $callback('Chunk 2', 10);
    $callback('Chunk 3', 10);
    return new StreamingResult('Chunk 1Chunk 2Chunk 3', ['tokens' => 30]);
  });
```

### Testing with Real Providers

For integration testing with real providers, use environment variables to configure API keys:

```php
if (!getenv('DRUPAL_NATIVE_AI_TEST_API_KEY')) {
  $this->markTestSkipped('No API key available for testing.');
}

$config = $this->config('drupal_native_ai.settings');
$config->set('api_key', getenv('DRUPAL_NATIVE_AI_TEST_API_KEY'));
$config->save();
```

## Continuous Integration

The module is set up for continuous integration with:

- PHPUnit for automated testing
- PHPStan for static analysis
- PHP_CodeSniffer for coding standards validation

### CI Configuration

Example GitHub Actions configuration:

```yaml
name: Test

on:
  push:
    branches: [ main ]
  pull_request:
    branches: [ main ]

jobs:
  test:
    runs-on: ubuntu-latest
    
    steps:
    - uses: actions/checkout@v2
    
    - name: Install dependencies
      run: composer install
    
    - name: Run tests
      run: ./vendor/bin/phpunit -c phpunit.xml.dist
    
    - name: Run static analysis
      run: ./vendor/bin/phpstan analyse
    
    - name: Check coding standards
      run: ./vendor/bin/phpcs
```

## Test Documentation

All tests should include comprehensive documentation:

- Class-level documentation explaining the purpose of the test class
- Method-level documentation explaining the purpose of each test method
- Data provider documentation explaining test variations
- Comprehensive assertions with meaningful messages

Example:

```php
/**
 * Tests the token counting functionality.
 *
 * This test class validates that the TokenCounter utility correctly
 * counts tokens in various input texts, including edge cases like
 * empty strings, special characters, and multilingual content.
 *
 * @group drupal_native_ai
 * @coversDefaultClass \Drupal\drupal_native_ai\Utility\TokenCounter
 */
class TokenCounterTest extends UnitTestCase {

  /**
   * Tests token counting with various input types.
   *
   * @param string $text
   *   The input text to count tokens in.
   * @param int $expected_count
   *   The expected token count.
   * @param string $description
   *   Description of the test case.
   *
   * @dataProvider providerTokenCount
   */
  public function testTokenCount($text, $expected_count, $description) {
    $counter = new TokenCounter();
    $count = $counter->countTokens($text);
    
    $this->assertEquals(
      $expected_count,
      $count,
      sprintf('Failed to correctly count tokens for case: %s', $description)
    );
  }

  /**
   * Data provider for testTokenCount.
   *
   * @return array
   *   Test cases.
   */
  public function providerTokenCount() {
    return [
      ['This is a simple sentence.', 6, 'Simple sentence'],
      ['', 0, 'Empty string'],
      ['Special: chars! 123', 5, 'Special characters and numbers'],
      ['Multilingual: English, Español, Français', 7, 'Multilingual content'],
    ];
  }
}
```

## Resources

- [Drupal Testing Documentation](https://www.drupal.org/docs/testing)
- [PHPUnit Documentation](https://phpunit.readthedocs.io/)
- [Drupal JavaScript Testing](https://www.drupal.org/docs/8/testing/javascript-testing)
