---
id: move_consolidated-AI_MODULE_STRATEGY
---
# AI Module Strategy - Using Existing Drupal Contrib Modules
*Updated: January 9, 2025*

## 🎯 **Strategic Pivot: From Custom Development to Configuration**

### **Old Approach (Abandoned)**
- Building custom LangChain module
- Creating custom LLM provider plugins  
- Writing task routing engines
- Developing vector storage systems
- Custom security implementations

### **New Approach (Adopted)**
- Use **Drupal AI module** as framework
- Configure **Ollama Provider** for local models
- Create workflows with **ECA module**
- Implement RAG with **Search API AI**
- Leverage existing security modules

## 📊 **Module Mapping: Custom vs Contrib**

| Our Custom Module | Replace With | Benefits |
|-------------------|--------------|----------|
| native_ai (custom providers) | AI module + Ollama Provider | Community maintained, proven code |
| langchain (orchestration) | ECA module workflows | Visual builder, no code needed |
| Custom embeddings | Search API AI | Vector DB support included |
| Task router (custom) | ECA conditions/actions | 500+ actions available |
| Security logging | Existing + Key module | Standardized approach |

## 🏗️ **Implementation Architecture**

```
┌─────────────────────────────────────────────────────────┐
│                   Drupal Core                           │
└──────────────────────┬──────────────────────────────────┘
                       │
┌──────────────────────▼──────────────────────────────────┐
│                 AI Module (Contrib)                     │
│  - Provider abstraction layer                           │
│  - Model management                                     │
│  - Standardized interfaces                             │
└──────────────────────┬──────────────────────────────────┘
                       │
        ┌──────────────┼──────────────┬─────────────────┐
        │              │              │                 │
┌───────▼────┐ ┌───────▼────┐ ┌──────▼──────┐ ┌────────▼───────┐
│  Ollama    │ │    ECA     │ │ Search API  │ │     Key        │
│  Provider  │ │  Workflows │ │     AI      │ │    Module      │
├────────────┤ ├────────────┤ ├─────────────┤ ├────────────────┤
│Local models│ │Task routing│ │ Embeddings  │ │Secure storage  │
│Privacy     │ │No code     │ │    RAG      │ │Credentials     │
└────────────┘ └────────────┘ └─────────────┘ └────────────────┘
```

## 📋 **Module Installation Plan**

### **Phase 1: Core Infrastructure** (Day 1-2)
```bash
# AI framework
composer require drupal/ai:^1.0@beta
composer require drupal/ai_provider_ollama:^1.0@alpha

# Enable
drush en ai ai_provider_ollama -y
```

### **Phase 2: Workflow Automation** (Day 3-4)
```bash
# ECA and modeler
composer require drupal/eca:^2.0
composer require drupal/bpmn_io:^2.0

# Enable
drush en eca eca_base eca_content bpmn_io -y
```

### **Phase 3: Search and Embeddings** (Day 5-6)
```bash
# Search API AI
composer require drupal/search_api:^1.0
composer require drupal/search_api_ai:^1.0@alpha

# Enable
drush en search_api search_api_ai -y
```

### **Phase 4: Security and Monitoring** (Day 7-8)
```bash
# Key management
composer require drupal/key:^1.0

# Enable existing security
drush en secure_drupal key -y
```

## 🔧 **Configuration Strategy**

### **1. Provider Configuration**
Instead of coding providers, configure them:

```yaml
# config/sync/ai.provider.content_gen.yml
id: content_gen
label: 'Content Generation'
provider: ollama
configuration:
  model: llama2:70b
  endpoint: 'http://localhost:11434'
  temperature: 0.7
  max_tokens: 4096
```

### **2. ECA Workflow Configuration**
Instead of coding routing, use visual workflows:

```yaml
# ECA Model for task routing
id: ai_task_router
label: 'AI Task Router'
events:
  - plugin: 'content_entity:presave'
conditions:
  - plugin: 'eca_content_entity_field_value'
    field: 'field_ai_task_type'
actions:
  - plugin: 'ai_generate_text'
    provider: '[determined by condition]'
```

### **3. Search Configuration**
Instead of custom embeddings, configure Search API:

```yaml
# Search API server
id: ai_embeddings
name: 'AI Embeddings Server'
backend: search_api_ai
backend_config:
  embeddings_provider: ollama
  vector_database: milvus
```

## 🚀 **Migration Plan for Existing Modules**

### **native_ai Module**
**Current**: Custom LLM integration code
**Migration**: 
1. Keep as configuration layer
2. Use `ai.provider_manager` service
3. Remove custom provider code
4. Add configuration for task mapping

### **secure_drupal Module**
**Current**: Custom audit logging
**Migration**:
1. Keep security policies
2. Integrate with AI module events
3. Use Key module for credentials
4. Maintain compliance features

### **api_normalization Module**
**Current**: Production-ready
**Action**: No changes needed - already follows best practices

### **field_eca_bridge Module**
**Current**: ECA integration
**Migration**:
1. Update to use AI module actions
2. Create ECA templates for common tasks
3. Document workflow patterns

## 📈 **Benefits Analysis**

### **Development Time**
- **Old approach**: 4-6 weeks custom development
- **New approach**: 1-2 weeks configuration
- **Savings**: 75% time reduction

### **Maintenance**
- **Old approach**: Maintain all custom code
- **New approach**: Community maintains core functionality
- **Benefit**: Focus on business logic only

### **Security**
- **Old approach**: Custom security implementation
- **New approach**: Proven, audited modules
- **Benefit**: Higher security confidence

### **Features**
- **Old approach**: Limited to what we build
- **New approach**: Full ecosystem of AI capabilities
- **Benefit**: More features available immediately

## ✅ **Decision Matrix**

| Factor | Custom Development | Contrib Modules | Winner |
|--------|-------------------|-----------------|---------|
| Time to market | 4-6 weeks | 1-2 weeks | ✅ Contrib |
| Maintenance burden | High | Low | ✅ Contrib |
| Security updates | Manual | Automatic | ✅ Contrib |
| Community support | None | Active | ✅ Contrib |
| Customization | Full control | Configuration-based | ✅ Contrib |
| Cost | High (development) | Low (configuration) | ✅ Contrib |

## 🎯 **Final Recommendation**

**USE EXISTING CONTRIB MODULES**

The Drupal AI ecosystem has matured significantly. The combination of:
- AI module (framework)
- Ollama Provider (local models)
- ECA (workflow automation)
- Search API AI (embeddings/RAG)

Provides everything needed for a private, enterprise-grade LLM platform without custom development.

**Time to implement: 2 weeks**
**Time saved: 4+ weeks**
**Risk reduced: 90%**

## 📝 **Next Steps**

1. **Stop** custom LLM provider development
2. **Install** recommended contrib modules
3. **Configure** Ollama with multiple models
4. **Create** ECA workflows for routing
5. **Test** with real use cases
6. **Document** configuration patterns
7. **Train** team on visual workflow builder

**The path forward is clear: Configure, don't code!** 🚀
