---
id: platform-archived-PARALLEL_PROMPT_2_LLM_MODULE
---
# Parallel Development Prompt 2: LLM Module Native Integration

## 🎯 OBJECTIVE
Upgrade the `llm` module from 50% to 90%+ innovation score using the proven native Drupal integration patterns from `api_normalization` module.

## 📋 CURRENT STATUS
- **Module Location**: `./DEMO/web/modules/custom/llm/`
- **Current Score**: 50/100 (needs native integration)
- **Target Score**: 90/100 (following api_normalization success)
- **Reference Module**: `api_normalization` (95/100 - completed)

## 🎨 PROVEN PATTERNS TO APPLY

### 1. Field API Integration
**Reference**: `./DEMO/web/modules/custom/api_normalization/src/Plugin/Field/FieldType/ApiEndpointItem.php`

**Task**: Create AI model configuration field types:
- `LlmModelItem.php` - LLM model configuration field
- `LlmPromptItem.php` - Prompt template field
- `LlmResponseItem.php` - AI response storage field
- `TokenUsageItem.php` - Token usage tracking field

**Requirements**:
- 8+ properties per field type (following api_normalization pattern)
- Model-specific validation (tokens, parameters, etc.)
- Helper methods for AI processing
- Database schema for AI data

### 2. Plugin System Implementation
**Reference**: `./DEMO/web/modules/custom/api_normalization/src/Plugin/ApiTransformPluginManager.php`

**Task**: Create AI processing plugin architecture:
- `LlmProcessorPluginManager.php` - AI processor plugin manager
- `LlmProcessorInterface.php` - AI processor interface
- `LlmProcessorBase.php` - Base AI processor class
- `LlmProcessor.php` - Annotation class

**Requirements**:
- AI model discovery and validation
- Configuration methods for different AI providers
- Caching for AI responses
- Exception handling for AI errors

### 3. Views Integration
**Reference**: `./DEMO/web/modules/custom/api_normalization/src/Plugin/views/field/ApiResponse.php`

**Task**: Create AI monitoring Views fields:
- `LlmResponse.php` - Live AI response display
- `TokenUsage.php` - Token usage monitoring field
- `ModelStatus.php` - AI model status field
- `PromptHistory.php` - Prompt history field

**Requirements**:
- Live AI data display with caching
- Multiple display formats (JSON, summary, status)
- Performance optimization for large responses
- Error handling for AI failures

### 4. Widget and Formatter Implementation
**Reference**: `./DEMO/web/modules/custom/api_normalization/src/Plugin/Field/FieldWidget/ApiEndpointWidget.php`

**Task**: Create AI field widgets and formatters:
- `LlmModelWidget.php` - AI model configuration widget
- `LlmPromptWidget.php` - Prompt template widget
- `LlmModelFormatter.php` - AI model display formatter
- `LlmResponseFormatter.php` - AI response formatter

**Requirements**:
- Collapsible sections for complex AI configurations
- AJAX integration for model testing
- Multiple display modes (detailed, summary, status)

### 5. Theme Integration
**Reference**: `./DEMO/web/modules/custom/api_normalization/templates/`

**Task**: Create AI-themed templates:
- AI dashboard templates
- CSS with dark mode for AI interfaces
- JavaScript for AI interactions (real-time responses)
- Responsive design for AI configuration

## 📦 CONTRIB MODULES TO INTEGRATE

```yaml
dependencies:
  - ai:ai                    # AI integration framework
  - ai:ai_logging           # AI audit logging
  - ai:ai_automators        # AI automation
  - ai_agents:ai_agents     # AI agent management
  - openai:openai           # OpenAI integration
  - anthropic:anthropic     # Anthropic integration
  - langchain:langchain     # LangChain integration
  - mcp:mcp                 # Model Context Protocol
  - mcp_client:mcp_client   # MCP client integration
  - field:field             # Native Field API
  - plugin:plugin           # Native Plugin system
  - views:views             # Native Views integration
  - queue:queue             # Queue for async AI processing
```

## 🚀 IMPLEMENTATION STEPS

### Step 1: Field API Implementation (Day 1)
1. Create `LlmModelItem.php` field type with AI-specific properties
2. Create `LlmModelWidget.php` widget with model configuration
3. Create `LlmModelFormatter.php` formatter with multiple display modes
4. Test field functionality with real AI models

### Step 2: Plugin System (Day 2)
1. Create AI processor plugin manager and interface
2. Create base AI processor class with common functionality
3. Create example AI processor plugins (OpenAI, Anthropic, etc.)
4. Test plugin discovery and AI model configuration

### Step 3: Views Integration (Day 3)
1. Create AI response Views field with live data
2. Create token usage monitoring Views field
3. Add caching and performance optimization for AI data
4. Test Views integration with real AI responses

### Step 4: AI Provider Integration (Day 4)
1. Integrate with ai:ai contrib module ecosystem
2. Add OpenAI and Anthropic provider configurations
3. Implement MCP client integration
4. Test AI provider switching and configuration

### Step 5: Theme and Testing (Day 5)
1. Create AI dashboard templates with real-time updates
2. Add CSS styling with AI-specific dark mode
3. Add JavaScript for AI interactions and live responses
4. Comprehensive testing and documentation

## 🎯 SUCCESS CRITERIA

### Technical Requirements
- [ ] Field API integration complete with AI-specific field types
- [ ] Plugin system implemented for AI processors
- [ ] Views integration working with live AI data
- [ ] Theme integration complete with AI dashboard
- [ ] All AI contrib modules properly integrated

### AI-Specific Requirements
- [ ] Multiple AI provider support (OpenAI, Anthropic, etc.)
- [ ] Token usage tracking and monitoring
- [ ] Prompt template management
- [ ] AI response caching and optimization
- [ ] Error handling for AI failures

### Quality Requirements
- [ ] 90/100+ innovation score achieved
- [ ] All existing AI functionality preserved
- [ ] Performance optimized with AI response caching
- [ ] Comprehensive error handling for AI edge cases
- [ ] Drupal.org compliance ready

## 🔧 DEVELOPMENT COMMANDS

```bash
# Navigate to module
cd ./DEMO/web/modules/custom/llm/

# Enable module with AI dependencies
cd ./DEMO/
drush en llm ai ai_logging ai_automators openai anthropic langchain mcp mcp_client

# Clear cache after changes
drush cache:rebuild

# Check AI module status
drush pml | grep -E "(llm|ai|openai|anthropic)"

# Test AI functionality
drush llm:test

# Run module tests
drush test-run llm
```

## 📁 KEY FILES TO CREATE/UPDATE

### Field API Files
- `src/Plugin/Field/FieldType/LlmModelItem.php`
- `src/Plugin/Field/FieldType/LlmPromptItem.php`
- `src/Plugin/Field/FieldType/LlmResponseItem.php`
- `src/Plugin/Field/FieldWidget/LlmModelWidget.php`
- `src/Plugin/Field/FieldFormatter/LlmModelFormatter.php`

### Plugin System Files
- `src/Plugin/LlmProcessorPluginManager.php`
- `src/Plugin/LlmProcessorInterface.php`
- `src/Plugin/LlmProcessorBase.php`
- `src/Annotation/LlmProcessor.php`

### AI Processor Plugins
- `src/Plugin/LlmProcessor/OpenAiProcessor.php`
- `src/Plugin/LlmProcessor/AnthropicProcessor.php`
- `src/Plugin/LlmProcessor/McpProcessor.php`

### Views Integration Files
- `src/Plugin/views/field/LlmResponse.php`
- `src/Plugin/views/field/TokenUsage.php`
- `src/Plugin/views/field/ModelStatus.php`

### Theme Files
- `templates/llm-model-formatter.html.twig`
- `templates/llm-response-formatter.html.twig`
- `templates/llm-dashboard.html.twig`
- `css/llm-module.css`
- `js/llm-module.js`

### Configuration Files
- `llm.info.yml` (update AI dependencies)
- `llm.services.yml` (add AI plugin manager)
- `llm.libraries.yml` (add AI-specific CSS/JS)
- `config/install/llm.settings.yml` (AI model defaults)

## 🤖 AI-SPECIFIC FEATURES TO IMPLEMENT

### Model Configuration
- Multiple AI provider support (OpenAI, Anthropic, etc.)
- Model parameter configuration (temperature, max_tokens, etc.)
- API key management (secure storage)
- Model switching and fallback

### Prompt Management
- Prompt template system
- Variable substitution in prompts
- Prompt versioning and history
- Prompt performance tracking

### Response Handling
- AI response caching
- Response validation and sanitization
- Error handling for AI failures
- Response formatting and display

### Token Usage Tracking
- Token consumption monitoring
- Usage limits and alerts
- Cost calculation and tracking
- Usage analytics and reporting

## 🔒 SECURITY CONSIDERATIONS

### API Key Management
- Secure storage of AI provider API keys
- Key rotation and management
- Access control for AI configurations
- Audit logging for AI usage

### Content Filtering
- Input sanitization for AI prompts
- Output validation for AI responses
- Content moderation for generated text
- Privacy protection for sensitive data

## 🎯 FINAL DELIVERABLE
A fully upgraded `llm` module achieving 90/100+ innovation score with native Drupal integration patterns, comprehensive AI provider support, and enterprise-grade quality matching the `api_normalization` success.
