---
id: platform-archived-QUICK_REFERENCE_GUIDE
---
# Native Integration Quick Reference Guide
*Fast implementation checklist for 95/100 innovation scores*

## 🚀 Quick Start Checklist

### Before You Start
- [ ] Read NATIVE_INTEGRATION_METHODOLOGY.md
- [ ] Study api_normalization module as reference
- [ ] Plan your field properties (6-12 recommended)
- [ ] Define your plugin types and capabilities

### Field API Implementation (Priority 1)
```php
// 1. Create field type class
class YourFieldItem extends FieldItemBase {
  // propertyDefinitions() - 6-12 properties
  // schema() - database structure
  // Computed properties for business logic
}

// 2. Create widget class  
class YourFieldWidget extends WidgetBase {
  // formElement() - collapsible sections
  // AJAX callbacks for dynamic updates
  // Validation and error handling
}

// 3. Create formatter class
class YourFieldFormatter extends FormatterBase {
  // viewElements() - themed output
  // Multiple display modes
  // Performance optimization
}
```

### Plugin System (Priority 2)
```php
// 1. Plugin interface
interface YourPluginInterface {
  public function execute($input, array $options = []);
  public function validate($input): array;
  public function getCapabilities(): array;
}

// 2. Plugin manager
class YourPluginManager extends DefaultPluginManager {
  // Discovery, categorization, health checks
}

// 3. Plugin base class
abstract class YourPluginBase implements YourPluginInterface {
  // Common functionality, caching, logging
}
```

### Views Integration (Priority 3)
```php
// Custom Views field plugin
class YourViewsField extends FieldPluginBase {
  public function render(ResultRow $values) {
    // Cached rendering with multiple formats
  }
}
```

### Theme Integration (Priority 4)
```twig
{# your-formatter.html.twig #}
<div{{ attributes }}>
  {# Responsive, accessible markup #}
</div>
```

## 📋 Innovation Score Targets

| Component | Target Score | Key Requirements |
|-----------|-------------|------------------|
| Field API | 90%+ | Custom type + widget + formatter |
| Plugin System | 85%+ | Manager + interface + base class |
| Views Integration | 80%+ | Custom field plugin + caching |
| Theme Integration | 85%+ | Twig + CSS + JavaScript |
| **Overall** | **90%+** | **All components working together** |

## ⚡ Speed Implementation Tips

### Field API Fast Track
1. **Copy from api_normalization**: Start with working example
2. **Properties first**: Define 6-12 meaningful properties
3. **Widget sections**: Use collapsible details for organization
4. **Multiple formatters**: At least 3 display modes

### Plugin System Fast Track
1. **Interface design**: Define 5-8 core methods
2. **Manager functionality**: Discovery + categorization + health
3. **Base class power**: Caching + logging + validation
4. **Example plugins**: 2-3 working examples

### Views Fast Track
1. **Field plugin**: Custom display of your data
2. **Multiple formats**: JSON, table, summary views
3. **Performance**: Implement caching from day one
4. **Configuration**: User-configurable display options

### Theme Fast Track
1. **Twig templates**: Responsive and accessible
2. **CSS modern**: Variables, dark mode, responsive
3. **JavaScript progressive**: Enhancement, not requirement
4. **Library definition**: Proper Drupal asset management

## 🛠️ Essential File Structure

```
your_module/
├── src/
│   ├── Plugin/
│   │   ├── Field/
│   │   │   ├── FieldType/YourFieldItem.php
│   │   │   ├── FieldWidget/YourFieldWidget.php
│   │   │   └── FieldFormatter/YourFieldFormatter.php
│   │   ├── YourPlugin/
│   │   │   ├── ExamplePlugin.php
│   │   │   └── AdvancedPlugin.php
│   │   ├── views/field/YourViewsField.php
│   │   └── YourPluginManager.php
│   ├── Annotation/YourPlugin.php
│   └── YourPluginInterface.php
├── templates/
│   └── your-field-formatter.html.twig
├── css/
│   └── your-field-formatter.css
├── js/
│   └── your-field-formatter.js
├── your_module.libraries.yml
├── your_module.services.yml
└── your_module.module
```

## 🔧 Code Templates

### Service Definition Template
```yaml
# your_module.services.yml
services:
  your_module.plugin_manager:
    class: Drupal\your_module\Plugin\YourPluginManager
    arguments:
      - 'Plugin/YourPlugin'
      - '@container.namespaces'
      - '@module_handler'
      - '@cache.discovery'
    tags:
      - { name: plugin_manager_cache_clear }
```

### Library Definition Template
```yaml
# your_module.libraries.yml
your_field_formatter:
  css:
    theme:
      css/your-field-formatter.css: {}
  js:
    js/your-field-formatter.js: {}
  dependencies:
    - core/jquery
    - core/drupal
    - core/once
```

### Annotation Template
```php
/**
 * @YourPlugin(
 *   id = "plugin_id",
 *   label = @Translation("Plugin Label"),
 *   description = @Translation("Plugin description."),
 *   category = @Translation("Category"),
 *   weight = 0
 * )
 */
```

## 🚨 Common Mistakes to Avoid

### Field API Mistakes
❌ Too few properties (less than 4)
❌ No computed properties
❌ Basic widget without sections
❌ Single formatter mode only

### Plugin System Mistakes  
❌ No plugin manager
❌ Missing interface definition
❌ No categorization system
❌ Lack of health checking

### Views Mistakes
❌ No custom field plugins
❌ Missing caching implementation
❌ Single display format only
❌ No user configuration

### Theme Mistakes
❌ Hardcoded HTML in PHP
❌ No responsive design
❌ Missing accessibility features
❌ Inline CSS/JavaScript

## ⏱️ Time Estimates

| Task | Time Estimate | Priority |
|------|---------------|----------|
| Field API complete | 2-3 days | High |
| Plugin system | 2-3 days | High |
| Views integration | 1-2 days | Medium |
| Theme integration | 1-2 days | Medium |
| Testing & docs | 2-3 days | High |
| **Total** | **8-13 days** | |

## 📈 Success Validation

### Week 1 Checkpoint
- [ ] Field type with 6+ properties ✓
- [ ] Widget with collapsible sections ✓
- [ ] Formatter with 3+ display modes ✓
- [ ] Basic plugin system working ✓

### Week 2 Checkpoint  
- [ ] Plugin manager with categorization ✓
- [ ] 2-3 example plugins ✓
- [ ] Views field plugin functional ✓
- [ ] Theme integration complete ✓

### Final Validation
- [ ] Innovation score 90%+ ✓
- [ ] All four pillars implemented ✓
- [ ] Tests passing with 95%+ coverage ✓
- [ ] Documentation complete ✓

## 🔗 Reference Modules

### Study These Examples
1. **api_normalization**: Perfect field API implementation
2. **secure_drupal**: Security-focused native integration  
3. **llm**: AI model configuration patterns
4. **Core field modules**: text, number, entity_reference

### Copy These Patterns
- Field API: api_normalization/src/Plugin/Field/
- Plugin system: secure_drupal/src/Plugin/
- Views: llm/src/Plugin/views/
- Theme: api_normalization/templates/

## 🎯 Innovation Score Calculator

```
Field API (25%):
- Custom field type (5%)
- Sophisticated widget (5%)
- Multiple formatters (5%)
- Computed properties (5%)
- Validation/constraints (5%)

Plugin System (25%):
- Plugin interface (5%)
- Plugin manager (5%)
- Plugin base class (5%)
- Example plugins (5%)
- Health checking (5%)

Views Integration (20%):
- Custom field plugin (10%)
- Caching implementation (5%)
- Multiple formats (5%)

Theme Integration (20%):
- Twig templates (5%)
- CSS with modern features (5%)
- JavaScript enhancement (5%)
- Accessibility (5%)

Testing & Docs (10%):
- Test coverage >95% (5%)
- Comprehensive docs (5%)
```

**Target: 90%+ for Drupal.org submission readiness**

---

*This guide is designed for rapid implementation of proven native integration patterns. Follow the methodology document for detailed explanations and architectural decisions.*
