---
id: shared-README
---
# Bluefly Shared Utilities

Welcome to the documentation for the Bluefly Shared Utilities. This section provides comprehensive information about the shared utilities available in the `@bluefly/bfcli/shared` package and the migration of Bluefly projects to use these utilities.

## Overview

The Bluefly Shared Utilities provide standardized implementations of common functionality across all Bluefly projects. By centralizing these utilities, we reduce code duplication, improve maintainability, and ensure consistent behavior across our platform.

## Documentation

- [Shared Components Reference](../../shared/SHARED_COMPONENTS) - Detailed reference of available shared utilities
- [Migration Guide](../../shared/SHARED_UTILITIES_MIGRATION_GUIDE) - Guide to migrating projects to use shared utilities
- [Migration Progress](../../shared/MIGRATION_PROGRESS) - Current status of project migrations
- [Migration Summary](../../shared/MIGRATION_SUMMARY) - Comprehensive summary of the migration initiative

## Project-Specific Migration Documentation

- [BFAPI Migration](../../../bfapi/docs/SHARED_UTILITIES_MIGRATION)
- [BFLLM Migration](../../../bfllm/docs/SHARED_UTILITIES_MIGRATION)
- [BFMCP Migration](../../../bfmcp/docs/SHARED_UTILITIES_MIGRATION)
- [BFRFP Migration](../../../bfrfp/docs/SHARED_UTILITIES_MIGRATION)
- [BFAPICLIENT Migration](../../../bfapiclient/docs/SHARED_UTILITIES_MIGRATION)

## Core Components

### Error Handling

The error handling system provides standardized error classes, error creation utilities, and middleware for handling errors consistently.

```typescript
import { createError, ERROR_CODES } from '@bluefly/bfcli/shared';

const error = createError(
  'Failed to connect to service',
  ERROR_CODES.SERVICE_UNAVAILABLE,
  503,
  { service: 'api', attempt: 3 }
);
```

### Logging

The logging system provides consistent, structured logging with support for different log levels, context tracking, and output formatting.

```typescript
import { BlueflyLogger } from '@bluefly/bfcli/shared';

const logger = new BlueflyLogger({
  prefix: '[my-service]',
  level: 'info',
  timestamp: true
});

logger.info('Operation completed', { 
  userId: '123', 
  correlationId: 'abc-123' 
});
```

### HTTP Client

The HTTP client provides a resilient, feature-rich client for making HTTP requests with built-in retry logic, circuit breaker pattern, and standardized error handling.

```typescript
import { HttpClient } from '@bluefly/bfcli/shared';

const httpClient = new HttpClient({
  baseURL: 'https://api.example.com',
  timeout: 5000,
  retries: 3
});

const response = await httpClient.get('/users/123');
```

### Observability

The observability system provides tools for monitoring application performance, tracking requests, and collecting metrics.

```typescript
import { metrics, tracer } from '@bluefly/bfcli/shared';

// Record metrics
metrics.counter('api_requests_total', 1, {
  method: 'GET',
  path: '/users',
  status: 200
});

// Create a span
const span = tracer.startSpan('process_request');
// ... operation ...
span.end();
```

## Getting Started

To use the shared utilities in your project:

1. Add the dependency to your package.json:
   ```json
   {
     "dependencies": {
       "@bluefly/bfcli": "^1.0.0"
     }
   }
   ```

2. Import the utilities using ES module syntax:
   ```typescript
   import { logger } from '@bluefly/bfcli/shared/logger/index.js';
   import { createError } from '@bluefly/bfcli/shared/errors/index.js';
   import { HttpClient } from '@bluefly/bfcli/shared/http/index.js';
   ```

3. Configure the utilities for your service:
   ```typescript
   const serviceLogger = new BlueflyLogger({
     prefix: '[my-service]',
     level: process.env.LOG_LEVEL || 'info'
   });
   
   const httpClient = new HttpClient({
     baseURL: process.env.API_URL,
     timeout: 5000,
     retries: 3
   });
   ```

For detailed information on each utility, see the [Shared Components Reference](../../shared/SHARED_COMPONENTS).
