---
title: Plugin Development - Unified Compliance Framework
description: >-
  How to extend the Unified Compliance and Security Framework with custom
  plugins
id: unified-compliance-plugin-development
---

# Plugin Development Guide

The Unified Compliance and Security Framework is designed to be highly extensible through plugins. This guide explains how to create custom plugins to extend the framework's functionality.

## Plugin Architecture Overview

The framework uses a plugin-based architecture with several plugin types:

1. **Compliance Plugins**: Implement compliance frameworks and validation rules
2. **Policy Plugins**: Implement security policies and enforcement mechanisms
3. **Audit Plugins**: Extend audit logging capabilities
4. **Storage Plugins**: Customize where and how audit data is stored
5. **Connector Plugins**: Integrate with external systems like GitLab, GitHub, JIRA

## Creating a Compliance Plugin

Compliance plugins implement specific compliance frameworks and validation logic.

### TypeScript Implementation

```typescript
import { CompliancePlugin, ComplianceFrameworkEngine } from 'secure_project';

export class MyCompliancePlugin implements CompliancePlugin {
  id = 'my-compliance-plugin';
  name = 'My Compliance Plugin';
  version = '1.0.0';
  description = 'Custom compliance plugin';
  
  frameworks = [
    {
      id: 'MY_FRAMEWORK',
      name: 'My Custom Framework',
      version: '1.0.0',
      jurisdiction: 'Internal',
      provider: 'Organization',
      description: 'Custom compliance framework',
      controls: [
        {
          id: 'MY-001',
          name: 'My First Control',
          description: 'Description of first control',
          category: 'security',
          severity: 'high',
          automatable: true,
          requirements: [
            'Requirement 1',
            'Requirement 2',
          ],
          validationRules: [
            {
              type: 'condition',
              rule: 'someCondition === true',
            }
          ]
        },
        {
          id: 'MY-002',
          name: 'My Second Control',
          description: 'Description of second control',
          category: 'privacy',
          severity: 'medium',
          automatable: true,
          requirements: [
            'Requirement 1',
            'Requirement 2',
          ],
          validationRules: [
            {
              type: 'threshold',
              metric: 'data_encryption_percentage',
              threshold: 100,
              comparison: 'equals'
            }
          ]
        }
      ]
    }
  ];
  
  async initialize(engine: ComplianceFrameworkEngine): Promise<void> {
    // Store reference to engine for later use
    this.engine = engine;
    
    // Register with engine
    await engine.registerPlugin(this);
    
    // Additional initialization
    this.logger = engine.getLogger();
    this.config = engine.getConfig();
  }
  
  getAPI(): Record<string, Function> {
    return {
      validateMyFramework: this.validateMyFramework.bind(this),
      getComplianceStatus: this.getComplianceStatus.bind(this),
      generateReport: this.generateReport.bind(this),
    };
  }
  
  async validateMyFramework(context: any): Promise<any> {
    this.logger.info('Validating MY_FRAMEWORK compliance');
    
    const results = {
      frameworkId: 'MY_FRAMEWORK',
      controls: {},
      summary: {
        passCount: 0,
        failCount: 0,
        naCount: 0,
        totalCount: this.frameworks[0].controls.length,
      }
    };
    
    // Validate each control
    for (const control of this.frameworks[0].controls) {
      const controlResult = await this.validateControl(control, context);
      results.controls[control.id] = controlResult;
      
      // Update summary
      if (controlResult.status === 'pass') {
        results.summary.passCount++;
      } else if (controlResult.status === 'fail') {
        results.summary.failCount++;
      } else {
        results.summary.naCount++;
      }
    }
    
    return results;
  }
  
  private async validateControl(control, context): Promise<any> {
    // Implementation of control validation logic
    this.logger.debug(`Validating control ${control.id}`);
    
    const result = {
      controlId: control.id,
      status: 'pass',  // 'pass', 'fail', or 'na'
      details: {},
      evidence: [],
      timestamp: new Date().toISOString(),
    };
    
    // Implement validation logic based on control type
    if (control.validationRules) {
      for (const rule of control.validationRules) {
        const ruleResult = await this.validateRule(rule, context);
        if (!ruleResult.pass) {
          result.status = 'fail';
          result.details = {
            ...result.details,
            ...ruleResult.details
          };
          result.evidence.push(ruleResult.evidence);
        }
      }
    }
    
    return result;
  }
  
  private async validateRule(rule, context): Promise<any> {
    // Implement rule validation logic
    let pass = false;
    let details = {};
    let evidence = '';
    
    switch (rule.type) {
      case 'condition':
        // Evaluate condition
        pass = /* evaluation logic */;
        evidence = `Condition evaluated to ${pass}`;
        break;
        
      case 'threshold':
        // Get metric value
        const metricValue = /* get metric value */;
        // Compare against threshold
        pass = this.compareThreshold(metricValue, rule.threshold, rule.comparison);
        evidence = `Metric ${rule.metric} value ${metricValue} ${rule.comparison} ${rule.threshold}`;
        break;
        
      default:
        pass = false;
        evidence = `Unknown rule type: ${rule.type}`;
    }
    
    return { pass, details, evidence };
  }
  
  private compareThreshold(value, threshold, comparison): boolean {
    switch (comparison) {
      case 'equals': return value === threshold;
      case 'not_equals': return value !== threshold;
      case 'greater_than': return value > threshold;
      case 'less_than': return value < threshold;
      case 'greater_than_or_equals': return value >= threshold;
      case 'less_than_or_equals': return value <= threshold;
      default: return false;
    }
  }
  
  async getComplianceStatus(): Promise<any> {
    // Implementation of compliance status reporting
  }
  
  async generateReport(options: any): Promise<any> {
    // Implementation of report generation
  }
}
```

### Drupal Implementation

```php
<?php

namespace Drupal\my_module\Plugin\UnifiedCompliance;

use Drupal\unified_compliance_security\Plugin\CompliancePluginBase;
use Drupal\unified_compliance_security\Plugin\CompliancePluginInterface;

/**
 * @CompliancePlugin(
 *   id = "my_custom_compliance",
 *   label = @Translation("My Custom Compliance"),
 *   description = @Translation("Custom compliance framework"),
 * )
 */
class MyCustomCompliancePlugin extends CompliancePluginBase implements CompliancePluginInterface {

  /**
   * {@inheritdoc}
   */
  public function getFrameworks() {
    return [
      'MY_FRAMEWORK' => [
        'name' => 'My Custom Framework',
        'version' => '1.0.0',
        'controls' => $this->getControls(),
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function validateCompliance($context) {
    $this->logger->info('Validating MY_FRAMEWORK compliance');
    
    $results = [
      'frameworkId' => 'MY_FRAMEWORK',
      'controls' => [],
      'summary' => [
        'passCount' => 0,
        'failCount' => 0,
        'naCount' => 0,
        'totalCount' => count($this->getControls()),
      ],
    ];
    
    // Validate each control
    foreach ($this->getControls() as $controlId => $control) {
      $controlResult = $this->validateControl($controlId, $control, $context);
      $results['controls'][$controlId] = $controlResult;
      
      // Update summary
      if ($controlResult['status'] === 'pass') {
        $results['summary']['passCount']++;
      } elseif ($controlResult['status'] === 'fail') {
        $results['summary']['failCount']++;
      } else {
        $results['summary']['naCount']++;
      }
    }
    
    return $results;
  }

  /**
   * Get compliance controls.
   */
  protected function getControls() {
    return [
      'MY-001' => [
        'name' => 'My First Control',
        'description' => 'Description of first control',
        'category' => 'security',
        'severity' => 'high',
        'requirements' => [
          'Requirement 1',
          'Requirement 2',
        ],
        'validationRules' => [
          [
            'type' => 'condition',
            'rule' => 'someCondition === true',
          ],
        ],
      ],
      'MY-002' => [
        'name' => 'My Second Control',
        'description' => 'Description of second control',
        'category' => 'privacy',
        'severity' => 'medium',
        'requirements' => [
          'Requirement 1',
          'Requirement 2',
        ],
        'validationRules' => [
          [
            'type' => 'threshold',
            'metric' => 'data_encryption_percentage',
            'threshold' => 100,
            'comparison' => 'equals',
          ],
        ],
      ],
    ];
  }

  /**
   * Validate a specific control.
   */
  protected function validateControl($controlId, $control, $context) {
    $this->logger->debug(sprintf('Validating control %s', $controlId));
    
    $result = [
      'controlId' => $controlId,
      'status' => 'pass',  // 'pass', 'fail', or 'na'
      'details' => [],
      'evidence' => [],
      'timestamp' => \Drupal::time()->getRequestTime(),
    ];
    
    // Implement validation logic based on control type
    if (isset($control['validationRules'])) {
      foreach ($control['validationRules'] as $rule) {
        $ruleResult = $this->validateRule($rule, $context);
        if (!$ruleResult['pass']) {
          $result['status'] = 'fail';
          $result['details'] = array_merge($result['details'], $ruleResult['details']);
          $result['evidence'][] = $ruleResult['evidence'];
        }
      }
    }
    
    return $result;
  }

  /**
   * Validate a specific rule.
   */
  protected function validateRule($rule, $context) {
    // Implement rule validation logic
    $pass = FALSE;
    $details = [];
    $evidence = '';
    
    switch ($rule['type']) {
      case 'condition':
        // Evaluate condition
        $pass = /* evaluation logic */;
        $evidence = sprintf('Condition evaluated to %s', $pass ? 'true' : 'false');
        break;
        
      case 'threshold':
        // Get metric value
        $metricValue = /* get metric value */;
        // Compare against threshold
        $pass = $this->compareThreshold($metricValue, $rule['threshold'], $rule['comparison']);
        $evidence = sprintf('Metric %s value %s %s %s', 
          $rule['metric'], 
          $metricValue, 
          $rule['comparison'], 
          $rule['threshold']
        );
        break;
        
      default:
        $pass = FALSE;
        $evidence = sprintf('Unknown rule type: %s', $rule['type']);
    }
    
    return [
      'pass' => $pass,
      'details' => $details,
      'evidence' => $evidence,
    ];
  }

  /**
   * Compare a value against a threshold.
   */
  protected function compareThreshold($value, $threshold, $comparison) {
    switch ($comparison) {
      case 'equals': return $value === $threshold;
      case 'not_equals': return $value !== $threshold;
      case 'greater_than': return $value > $threshold;
      case 'less_than': return $value < $threshold;
      case 'greater_than_or_equals': return $value >= $threshold;
      case 'less_than_or_equals': return $value <= $threshold;
      default: return FALSE;
    }
  }
}
```

## Creating a Policy Plugin

Policy plugins implement security policies and enforcement mechanisms.

### TypeScript Implementation

```typescript
import { PolicyPlugin, PolicyEngine } from 'secure_project';

export class MyPolicyPlugin implements PolicyPlugin {
  id = 'my-policy-plugin';
  name = 'My Policy Plugin';
  version = '1.0.0';
  description = 'Custom policy plugin';
  
  private engine: PolicyEngine;
  
  async initialize(engine: PolicyEngine): Promise<void> {
    this.engine = engine;
    
    // Register policies
    await this.registerPolicies();
  }
  
  private async registerPolicies(): Promise<void> {
    // Example policy
    await this.engine.setPolicy({
      id: 'data-access-policy',
      name: 'Data Access Control Policy',
      description: 'Enforces data access based on tenant isolation',
      priority: 100,
      enabled: true,
      rules: [
        {
          id: 'tenant-isolation',
          name: 'Tenant Isolation Rule',
          description: 'Prevents cross-tenant data access',
          conditions: [
            { field: 'action', operator: 'equals', value: 'access' },
            { field: 'resourceType', operator: 'equals', value: 'data' }
          ],
          effect: 'check',
          checkFunction: 'validateTenantIsolation',
          metadata: {
            auditLevel: 'detailed',
            severity: 'critical'
          }
        }
      ]
    });
  }
  
  getAPI(): Record<string, Function> {
    return {
      validateTenantIsolation: this.validateTenantIsolation.bind(this),
      getPolicies: this.getPolicies.bind(this),
      evaluatePolicy: this.evaluatePolicy.bind(this)
    };
  }
  
  async validateTenantIsolation(context: any): Promise<{allowed: boolean, reason?: string}> {
    const { user, resource } = context;
    
    // Check tenant isolation
    if (user.tenantId !== resource.tenantId) {
      return {
        allowed: false,
        reason: `Cross-tenant access denied: User tenant ${user.tenantId} attempting to access resource in tenant ${resource.tenantId}`
      };
    }
    
    return { allowed: true };
  }
  
  async getPolicies(): Promise<any[]> {
    return this.engine.getPolicies();
  }
  
  async evaluatePolicy(policyId: string, context: any): Promise<any> {
    return this.engine.evaluatePolicy(policyId, context);
  }
}
```

### Drupal Implementation

```php
<?php

namespace Drupal\my_module\Plugin\UnifiedCompliance;

use Drupal\unified_compliance_security\Plugin\PolicyPluginBase;
use Drupal\unified_compliance_security\Plugin\PolicyPluginInterface;

/**
 * @PolicyPlugin(
 *   id = "my_custom_policy",
 *   label = @Translation("My Custom Policy"),
 *   description = @Translation("Custom security policy"),
 * )
 */
class MyCustomPolicyPlugin extends PolicyPluginBase implements PolicyPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function getPolicies() {
    return [
      'data-access-policy' => [
        'name' => 'Data Access Control Policy',
        'description' => 'Enforces data access based on tenant isolation',
        'priority' => 100,
        'enabled' => TRUE,
        'rules' => $this->getRules(),
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function evaluatePolicy($policyId, $context) {
    $policy = $this->getPolicies()[$policyId] ?? NULL;
    
    if (!$policy) {
      return [
        'allowed' => FALSE,
        'reason' => sprintf('Policy %s not found', $policyId),
      ];
    }
    
    if (!$policy['enabled']) {
      return [
        'allowed' => TRUE,
        'reason' => sprintf('Policy %s is disabled', $policyId),
      ];
    }
    
    $results = [];
    
    foreach ($policy['rules'] as $ruleId => $rule) {
      $ruleResult = $this->evaluateRule($ruleId, $rule, $context);
      $results[$ruleId] = $ruleResult;
      
      if (!$ruleResult['allowed']) {
        return $ruleResult;
      }
    }
    
    return [
      'allowed' => TRUE,
      'details' => $results,
    ];
  }

  /**
   * Get policy rules.
   */
  protected function getRules() {
    return [
      'tenant-isolation' => [
        'name' => 'Tenant Isolation Rule',
        'description' => 'Prevents cross-tenant data access',
        'conditions' => [
          ['field' => 'action', 'operator' => 'equals', 'value' => 'access'],
          ['field' => 'resourceType', 'operator' => 'equals', 'value' => 'data'],
        ],
        'effect' => 'check',
        'checkFunction' => 'validateTenantIsolation',
        'metadata' => [
          'auditLevel' => 'detailed',
          'severity' => 'critical',
        ],
      ],
    ];
  }

  /**
   * Evaluate a specific rule.
   */
  protected function evaluateRule($ruleId, $rule, $context) {
    // Check conditions
    if (!$this->evaluateConditions($rule['conditions'], $context)) {
      return [
        'allowed' => TRUE,
        'reason' => 'Conditions not met',
      ];
    }
    
    // Evaluate effect
    switch ($rule['effect']) {
      case 'allow':
        return [
          'allowed' => TRUE,
          'reason' => sprintf('Rule %s explicitly allows', $ruleId),
        ];
        
      case 'deny':
        return [
          'allowed' => FALSE,
          'reason' => sprintf('Rule %s explicitly denies', $ruleId),
        ];
        
      case 'check':
        $checkFunction = $rule['checkFunction'];
        if (method_exists($this, $checkFunction)) {
          return $this->$checkFunction($context);
        }
        return [
          'allowed' => FALSE,
          'reason' => sprintf('Check function %s not found', $checkFunction),
        ];
        
      default:
        return [
          'allowed' => FALSE,
          'reason' => sprintf('Unknown effect %s', $rule['effect']),
        ];
    }
  }

  /**
   * Evaluate conditions.
   */
  protected function evaluateConditions($conditions, $context) {
    if (empty($conditions)) {
      return TRUE;
    }
    
    foreach ($conditions as $condition) {
      if (!$this->evaluateCondition($condition, $context)) {
        return FALSE;
      }
    }
    
    return TRUE;
  }

  /**
   * Evaluate a single condition.
   */
  protected function evaluateCondition($condition, $context) {
    $field = $condition['field'];
    $operator = $condition['operator'];
    $value = $condition['value'];
    
    if (!isset($context[$field])) {
      return FALSE;
    }
    
    $contextValue = $context[$field];
    
    switch ($operator) {
      case 'equals':
        return $contextValue === $value;
      case 'not_equals':
        return $contextValue !== $value;
      case 'contains':
        return strpos($contextValue, $value) !== FALSE;
      case 'in':
        return in_array($contextValue, (array) $value);
      default:
        return FALSE;
    }
  }

  /**
   * Validate tenant isolation.
   */
  public function validateTenantIsolation($context) {
    $user = $context['user'];
    $resource = $context['resource'];
    
    // Check tenant isolation
    if ($user['tenantId'] !== $resource['tenantId']) {
      return [
        'allowed' => FALSE,
        'reason' => sprintf(
          'Cross-tenant access denied: User tenant %s attempting to access resource in tenant %s',
          $user['tenantId'],
          $resource['tenantId']
        ),
      ];
    }
    
    return ['allowed' => TRUE];
  }
}
```

## Creating an Audit Plugin

Audit plugins extend the audit logging capabilities.

### TypeScript Implementation

```typescript
import { AuditPlugin, AuditLogger } from 'secure_project';

export class MyAuditPlugin implements AuditPlugin {
  id = 'my-audit-plugin';
  name = 'My Audit Plugin';
  version = '1.0.0';
  description = 'Custom audit plugin';
  
  private logger: AuditLogger;
  
  async initialize(logger: AuditLogger): Promise<void> {
    this.logger = logger;
  }
  
  getAPI(): Record<string, Function> {
    return {
      logCustomEvent: this.logCustomEvent.bind(this),
      getAuditSummary: this.getAuditSummary.bind(this),
      getHighRiskEvents: this.getHighRiskEvents.bind(this)
    };
  }
  
  async logCustomEvent(event: any): Promise<void> {
    await this.logger.log({
      ...event,
      source: 'my-audit-plugin',
      timestamp: new Date().toISOString()
    });
  }
  
  async getAuditSummary(timeframe: string): Promise<any> {
    // Calculate audit summary
    const startTime = this.calculateStartTime(timeframe);
    
    const logs = await this.logger.query({
      from: startTime.toISOString(),
      to: new Date().toISOString()
    });
    
    return {
      total: logs.length,
      byType: this.groupBy(logs, 'type'),
      byUser: this.groupBy(logs, 'userId'),
      byResource: this.groupBy(logs, 'resourceType'),
      highRisk: logs.filter(log => log.severity === 'high' || log.severity === 'critical').length
    };
  }
  
  async getHighRiskEvents(timeframe: string): Promise<any[]> {
    const startTime = this.calculateStartTime(timeframe);
    
    const logs = await this.logger.query({
      from: startTime.toISOString(),
      to: new Date().toISOString(),
      filter: {
        severity: ['high', 'critical']
      }
    });
    
    return logs;
  }
  
  private calculateStartTime(timeframe: string): Date {
    const now = new Date();
    
    switch (timeframe) {
      case 'day':
        return new Date(now.setDate(now.getDate() - 1));
      case 'week':
        return new Date(now.setDate(now.getDate() - 7));
      case 'month':
        return new Date(now.setMonth(now.getMonth() - 1));
      case 'year':
        return new Date(now.setFullYear(now.getFullYear() - 1));
      default:
        return new Date(now.setDate(now.getDate() - 1)); // Default to day
    }
  }
  
  private groupBy(array: any[], key: string): Record<string, number> {
    return array.reduce((result, item) => {
      const keyValue = item[key] || 'unknown';
      result[keyValue] = (result[keyValue] || 0) + 1;
      return result;
    }, {});
  }
}
```

## Testing Plugins

### Testing TypeScript Plugins

```typescript
import { ComplianceFrameworkEngine } from 'secure_project';
import { MyCompliancePlugin } from './MyCompliancePlugin';

describe('MyCompliancePlugin', () => {
  let plugin: MyCompliancePlugin;
  let mockEngine: ComplianceFrameworkEngine;
  
  beforeEach(() => {
    mockEngine = {
      registerPlugin: jest.fn(),
      getLogger: jest.fn().mockReturnValue({
        info: jest.fn(),
        debug: jest.fn(),
        error: jest.fn()
      }),
      getConfig: jest.fn().mockReturnValue({})
    } as any;
    
    plugin = new MyCompliancePlugin();
  });
  
  test('should initialize correctly', async () => {
    await plugin.initialize(mockEngine);
    expect(mockEngine.registerPlugin).toHaveBeenCalledWith(plugin);
  });
  
  test('should provide API methods', () => {
    const api = plugin.getAPI();
    expect(api.validateMyFramework).toBeDefined();
    expect(api.getComplianceStatus).toBeDefined();
    expect(api.generateReport).toBeDefined();
  });
  
  test('should validate framework correctly', async () => {
    await plugin.initialize(mockEngine);
    
    const context = {
      // Test context
    };
    
    const result = await plugin.validateMyFramework(context);
    
    expect(result.frameworkId).toBe('MY_FRAMEWORK');
    expect(result.summary.totalCount).toBe(plugin.frameworks[0].controls.length);
  });
  
  // Additional tests for specific validation logic
});
```

### Testing Drupal Plugins

```php
<?php

namespace Drupal\Tests\my_module\Unit\Plugin\UnifiedCompliance;

use Drupal\Tests\UnitTestCase;
use Drupal\my_module\Plugin\UnifiedCompliance\MyCustomCompliancePlugin;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;

/**
 * @coversDefaultClass \Drupal\my_module\Plugin\UnifiedCompliance\MyCustomCompliancePlugin
 * @group my_module
 */
class MyCustomCompliancePluginTest extends UnitTestCase {

  /**
   * The plugin instance.
   *
   * @var \Drupal\my_module\Plugin\UnifiedCompliance\MyCustomCompliancePlugin
   */
  protected $plugin;

  /**
   * The mocked logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $loggerChannel;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    
    // Create mock logger
    $this->loggerChannel = $this->createMock(LoggerChannelInterface::class);
    
    $loggerFactory = $this->createMock(LoggerChannelFactoryInterface::class);
    $loggerFactory->method('get')
      ->willReturn($this->loggerChannel);
    
    // Create plugin instance
    $this->plugin = new MyCustomCompliancePlugin([], 'my_custom_compliance', [], $loggerFactory);
  }

  /**
   * @covers ::getFrameworks
   */
  public function testGetFrameworks() {
    $frameworks = $this->plugin->getFrameworks();
    
    $this->assertArrayHasKey('MY_FRAMEWORK', $frameworks);
    $this->assertEquals('My Custom Framework', $frameworks['MY_FRAMEWORK']['name']);
    $this->assertEquals('1.0.0', $frameworks['MY_FRAMEWORK']['version']);
  }

  /**
   * @covers ::validateCompliance
   */
  public function testValidateCompliance() {
    $context = [
      // Test context
    ];
    
    $results = $this->plugin->validateCompliance($context);
    
    $this->assertEquals('MY_FRAMEWORK', $results['frameworkId']);
    $this->assertEquals(count($this->plugin->getControls()), $results['summary']['totalCount']);
  }

  /**
   * @covers ::validateControl
   */
  public function testValidateControl() {
    $method = new \ReflectionMethod($this->plugin, 'validateControl');
    $method->setAccessible(TRUE);
    
    $controlId = 'MY-001';
    $control = [
      'name' => 'My First Control',
      'description' => 'Description of first control',
      'category' => 'security',
      'severity' => 'high',
      'validationRules' => [],
    ];
    $context = [];
    
    $result = $method->invoke($this->plugin, $controlId, $control, $context);
    
    $this->assertEquals($controlId, $result['controlId']);
    $this->assertEquals('pass', $result['status']);
  }

  /**
   * @covers ::compareThreshold
   */
  public function testCompareThreshold() {
    $method = new \ReflectionMethod($this->plugin, 'compareThreshold');
    $method->setAccessible(TRUE);
    
    $this->assertTrue($method->invoke($this->plugin, 10, 10, 'equals'));
    $this->assertTrue($method->invoke($this->plugin, 11, 10, 'greater_than'));
    $this->assertTrue($method->invoke($this->plugin, 9, 10, 'less_than'));
    $this->assertTrue($method->invoke($this->plugin, 10, 10, 'greater_than_or_equals'));
    $this->assertTrue($method->invoke($this->plugin, 10, 10, 'less_than_or_equals'));
    $this->assertTrue($method->invoke($this->plugin, 9, 10, 'not_equals'));
    
    $this->assertFalse($method->invoke($this->plugin, 11, 10, 'equals'));
    $this->assertFalse($method->invoke($this->plugin, 10, 10, 'greater_than'));
    $this->assertFalse($method->invoke($this->plugin, 10, 10, 'less_than'));
    $this->assertFalse($method->invoke($this->plugin, 9, 10, 'greater_than_or_equals'));
    $this->assertFalse($method->invoke($this->plugin, 11, 10, 'less_than_or_equals'));
    $this->assertFalse($method->invoke($this->plugin, 10, 10, 'not_equals'));
    $this->assertFalse($method->invoke($this->plugin, 10, 10, 'invalid_operator'));
  }
}
```

## Registering Plugins

### Registering TypeScript Plugins

Register your TypeScript plugins with the framework's engine:

```typescript
import { ComplianceFrameworkEngine, PolicyEngine, AuditLogger } from 'secure_project';
import { MyCompliancePlugin } from './MyCompliancePlugin';
import { MyPolicyPlugin } from './MyPolicyPlugin';
import { MyAuditPlugin } from './MyAuditPlugin';

// Setup engines
const complianceEngine = new ComplianceFrameworkEngine();
const policyEngine = new PolicyEngine();
const auditLogger = new AuditLogger();

// Create plugin instances
const myCompliancePlugin = new MyCompliancePlugin();
const myPolicyPlugin = new MyPolicyPlugin();
const myAuditPlugin = new MyAuditPlugin();

// Initialize plugins
async function initializePlugins() {
  await myCompliancePlugin.initialize(complianceEngine);
  await myPolicyPlugin.initialize(policyEngine);
  await myAuditPlugin.initialize(auditLogger);
  
  console.log('Plugins initialized successfully');
}

initializePlugins().catch(console.error);
```

### Registering Drupal Plugins

Register your Drupal plugins in your module's services.yml file:

```yaml
# In my_module.services.yml
services:
  my_module.compliance_plugin.my_custom_compliance:
    class: Drupal\my_module\Plugin\UnifiedCompliance\MyCustomCompliancePlugin
    arguments: ['@logger.factory']
    tags:
      - { name: unified_compliance_security.compliance_plugin }
  
  my_module.policy_plugin.my_custom_policy:
    class: Drupal\my_module\Plugin\UnifiedCompliance\MyCustomPolicyPlugin
    arguments: ['@logger.factory']
    tags:
      - { name: unified_compliance_security.policy_plugin }
```

## Best Practices

When developing plugins for the Unified Compliance and Security Framework, follow these best practices:

1. **Follow TDD Methodology**: Write tests first, then implement the plugin
2. **Implement Clear Interfaces**: Ensure your plugin implements the correct interface
3. **Document Thoroughly**: Include detailed documentation for all methods and properties
4. **Handle Errors Gracefully**: Implement comprehensive error handling
5. **Follow Naming Conventions**: Use consistent naming patterns
6. **Optimize Performance**: Consider performance implications of your plugin
7. **Maintain Compatibility**: Ensure your plugin works with different versions of the framework
8. **Use Dependency Injection**: Avoid hard-coded dependencies
9. **Implement Logging**: Use the provided logger for all important events
10. **Follow Security Best Practices**: Be especially careful with security-related code

## Conclusion

The plugin architecture of the Unified Compliance and Security Framework makes it highly extensible and customizable. By creating custom plugins, you can adapt the framework to your specific compliance, security, and code ownership needs.

For more information, see:

- [API Reference](./api-reference)
- [Security Dashboard Guide](./security-dashboard)
- [Compliance Framework Configuration](./compliance-frameworks)
- [Integration with bfcicomponents](./cicd-integration)
