---
title: Usage Guide - Unified Compliance Framework
description: How to use the Unified Compliance and Security Framework
id: unified-compliance-usage
---

# Usage Guide

This guide explains how to use the Unified Compliance and Security Framework for common security, compliance, and code ownership tasks.

## Security and Audit Logging

### Basic Security Logging

To log security events from your Drupal code:

```php
// In your Drupal code
$complianceManager = \Drupal::service('unified_compliance.manager');

// Log security events
$complianceManager->logSecurityEvent(
  $user->id(),
  $user->getRoles(),
  'user_login',
  'user',
  $user->id(),
  [
    'ip_address' => \Drupal::request()->getClientIp(),
    'user_agent' => \Drupal::request()->headers->get('User-Agent'),
  ]
);
```

### Validation of Data Access

Implement data access validation to enforce data segregation:

```php
// Validate data access
$accessResult = $complianceManager->validateDataAccess(
  $user->id(),
  $user->get('field_tenant')->value,
  $user->getRoles(),
  'view',
  'node',
  $node->get('field_tenant')->value,
  $node->id()
);

if (!$accessResult['allowed']) {
  throw new AccessDeniedHttpException($accessResult['reason']);
}
```

### Implementing Access Hooks

Integrate with Drupal's access system:

```php
/**
 * Implements hook_node_access().
 */
function my_module_node_access(NodeInterface $node, $op, AccountInterface $account) {
  $complianceManager = \Drupal::service('unified_compliance.manager');
  
  $accessResult = $complianceManager->validateDataAccess(
    $account->id(),
    $account->get('field_tenant')->value,
    $account->getRoles(),
    $op,
    'node',
    $node->get('field_tenant')->value,
    $node->id()
  );
  
  if (!$accessResult['allowed']) {
    return AccessResult::forbidden($accessResult['reason']);
  }
  
  return AccessResult::neutral();
}
```

## Code Ownership Analysis

### Analyzing Code Ownership

Perform code ownership analysis on a repository:

```php
// In your Drupal code
$complianceManager = \Drupal::service('unified_compliance.manager');

// Analyze code ownership
$ownershipResults = $complianceManager->analyzeCodeOwnership('/path/to/repository');

// Process results
foreach ($ownershipResults['files'] as $file => $owner) {
  echo "File: $file, Primary Owner: $owner\n";
}
```

### Detecting Knowledge Silos

Identify knowledge silos in your codebase:

```php
// Detect knowledge silos
$silos = $complianceManager->detectKnowledgeSilos('/path/to/repository');

// Display silos
foreach ($silos as $silo) {
  echo "Knowledge Silo: " . $silo['developer'] . "\n";
  echo "Exclusive knowledge of " . count($silo['files']) . " files\n";
  echo "Risk Level: " . $silo['riskLevel'] . "\n";
}
```

### Predicting Code Reviewers

Use AI to predict the best reviewers for a merge request:

```php
// Predict reviewers for a merge request
$reviewers = $complianceManager->predictReviewers([
  'id' => 'mr-123',
  'title' => 'Add new feature',
  'sourceBranch' => 'feature/new-feature',
  'targetBranch' => 'main',
  'author' => 'user1',
  'changedFiles' => ['src/feature.ts', 'tests/feature.test.ts']
]);

// Display reviewer recommendations
foreach ($reviewers as $reviewer) {
  echo "Reviewer: " . $reviewer['username'] . "\n";
  echo "Confidence: " . $reviewer['confidence'] . "\n";
  echo "Rationale: " . $reviewer['rationale'] . "\n";
}
```

## Compliance Validation

### Validating Code Ownership Compliance

Check compliance with code ownership requirements:

```php
// Validate code ownership compliance
$codeComplianceResult = $complianceManager->validateCodeOwnershipCompliance('/path/to/repository');

// Display compliance status
echo "Compliance Status: " . ($codeComplianceResult['compliant'] ? 'Compliant' : 'Non-Compliant') . "\n";
echo "Issues Found: " . count($codeComplianceResult['violations']) . "\n";

// Display violations
foreach ($codeComplianceResult['violations'] as $violation) {
  echo "Violation: " . $violation['description'] . "\n";
  echo "Severity: " . $violation['severity'] . "\n";
  echo "Remediation: " . $violation['remediation'] . "\n";
}
```

### Generating Compliance Reports

Generate comprehensive compliance reports:

```php
// Generate comprehensive compliance report
$complianceReport = $complianceManager->generateComplianceReport(
  '/path/to/repository',
  ['CODE_OWNERSHIP', 'GDPR', 'HIPAA']
);

// Process report
echo "Overall Compliance Score: " . $complianceReport['overallScore'] . "%\n";

foreach ($complianceReport['frameworks'] as $framework => $results) {
  echo "Framework: $framework\n";
  echo "Compliance: " . $results['score'] . "%\n";
  echo "Controls Passing: " . $results['passCount'] . "/" . $results['totalCount'] . "\n";
}
```

## Dashboard and UI

### Accessing the Dashboard

The module provides a comprehensive dashboard at `/admin/reports/compliance-security` with:

1. **Security Overview**: Audit logs, active alerts, recent violations
2. **Compliance Status**: Status across all frameworks
3. **Code Ownership**: Knowledge silo visualization, reviewer metrics
4. **Policy Enforcement**: Active policies, approval workflows, decisions

### Reviewing Audit Logs

Access detailed audit logs at `/admin/reports/compliance-security/audit-log` to review:

- Security events
- Access violations
- Compliance changes
- Critical operations
- Administrative actions

The logs can be filtered by:

- Date range
- User
- Event type
- Severity
- Resource type

### Managing Compliance Frameworks

Navigate to `/admin/config/security/compliance/frameworks` to:

- Enable/disable compliance frameworks
- Configure framework-specific settings
- Set validation schedules
- View compliance status
- Generate compliance reports

### Code Ownership Dashboard

The code ownership dashboard at `/admin/reports/compliance-security/code-ownership` provides:

- Repository knowledge maps
- Developer expertise analysis
- Knowledge silo visualization
- Reviewer recommendations
- Historical review patterns

## Plugin Development

### Creating a Custom Compliance Plugin

Extend the framework with custom compliance plugins:

```php
<?php

namespace Drupal\my_module\Plugin\UnifiedCompliance;

use Drupal\unified_compliance_security\Plugin\CompliancePluginBase;
use Drupal\unified_compliance_security\Plugin\CompliancePluginInterface;

/**
 * @CompliancePlugin(
 *   id = "my_custom_compliance",
 *   label = @Translation("My Custom Compliance"),
 *   description = @Translation("Custom compliance framework"),
 * )
 */
class MyCustomCompliancePlugin extends CompliancePluginBase implements CompliancePluginInterface {

  /**
   * {@inheritdoc}
   */
  public function getFrameworks() {
    return [
      'MY_FRAMEWORK' => [
        'name' => 'My Custom Framework',
        'version' => '1.0.0',
        'controls' => $this->getControls(),
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function validateCompliance($context) {
    // Implement compliance validation logic
    $results = [];
    
    // Check compliance for each control
    foreach ($this->getControls() as $controlId => $control) {
      $results[$controlId] = [
        'compliant' => $this->validateControl($controlId, $control, $context),
        'details' => $this->getControlDetails($controlId, $context),
      ];
    }
    
    return $results;
  }

  /**
   * Get compliance controls.
   */
  protected function getControls() {
    return [
      'MY-001' => [
        'name' => 'My First Control',
        'description' => 'Description of first control',
        'category' => 'security',
        'severity' => 'high',
      ],
      'MY-002' => [
        'name' => 'My Second Control',
        'description' => 'Description of second control',
        'category' => 'privacy',
        'severity' => 'medium',
      ],
    ];
  }

  /**
   * Validate a specific control.
   */
  protected function validateControl($controlId, $control, $context) {
    // Implement control-specific validation logic
    return TRUE;
  }

  /**
   * Get details about a control's validation.
   */
  protected function getControlDetails($controlId, $context) {
    return [
      'evidence' => 'Evidence details here',
      'timestamp' => time(),
      'validator' => 'automatic',
    ];
  }
}
```

### Implementing TypeScript Plugins

For more advanced functionality, implement a TypeScript plugin:

```typescript
// In your TypeScript code
import { CompliancePlugin, ComplianceFrameworkEngine } from 'secure_project';

export class MyCompliancePlugin implements CompliancePlugin {
  id = 'my-compliance-plugin';
  name = 'My Compliance Plugin';
  version = '1.0.0';
  description = 'Custom compliance plugin';
  
  frameworks = [
    {
      id: 'MY_FRAMEWORK',
      name: 'My Custom Framework',
      version: '1.0.0',
      jurisdiction: 'Internal',
      provider: 'Organization',
      description: 'Custom compliance framework',
      controls: [
        {
          id: 'MY-001',
          name: 'My First Control',
          description: 'Description of first control',
          category: 'security',
          severity: 'high',
          automatable: true,
          requirements: [
            'Requirement 1',
            'Requirement 2',
          ],
          validationRules: [
            {
              type: 'condition',
              rule: 'someCondition === true',
            }
          ]
        }
      ]
    }
  ];
  
  async initialize(engine: ComplianceFrameworkEngine): Promise<void> {
    // Initialize plugin
  }
  
  getAPI(): Record<string, Function> {
    return {
      // Your API methods
      validateMyFramework: this.validateMyFramework.bind(this),
    };
  }
  
  async validateMyFramework(context: any): Promise<any> {
    // Implementation
    return {
      compliant: true,
      score: 100,
      details: {
        // Detailed results
      }
    };
  }
}
```

## Advanced Usage

### Custom Audit Storage

Configure custom audit storage backends:

```php
// In a module file
function my_module_unified_compliance_security_storage_backends_alter(array &$backends) {
  $backends['my_custom_storage'] = [
    'class' => 'Drupal\my_module\Storage\MyCustomAuditStorage',
    'label' => t('My Custom Storage'),
  ];
}
```

### Integration with GitLab CI/CD

Integrate compliance checks into your CI/CD pipeline:

```yaml
# In .gitlab-ci.yml
compliance_check:
  stage: test
  script:
    - curl -X POST "https://your-drupal-site.com/api/compliance/validate" 
      -H "Authorization: Bearer $CI_TOKEN" 
      -H "Content-Type: application/json" 
      -d '{"repository":"$CI_PROJECT_PATH","commitSha":"$CI_COMMIT_SHA"}'
  only:
    - merge_requests
```

### Real-time Monitoring

Set up real-time monitoring of compliance status:

```php
// In a custom module
function my_module_cron() {
  $complianceManager = \Drupal::service('unified_compliance.manager');
  
  // Check compliance status
  $status = $complianceManager->getComplianceStatus();
  
  // If compliance score drops below threshold, send alert
  if ($status['overallScore'] < 80) {
    \Drupal::service('unified_compliance.alert_manager')->sendAlert(
      'compliance_degradation',
      'Compliance score has dropped below threshold',
      [
        'score' => $status['overallScore'],
        'threshold' => 80,
        'details' => $status,
      ]
    );
  }
}
```

## Next Steps

For more advanced usage scenarios:

- [Security Dashboard Guide](./security-dashboard) - How to use the security dashboard
- [Compliance Framework Configuration](./compliance-frameworks) - Configure compliance frameworks
- [API Reference](./api-reference) - Complete API documentation
- [Plugin Development](./plugin-development) - Create custom plugins
- [Integration with bfcicomponents](./cicd-integration) - Integrate with CI/CD
- [Troubleshooting](./troubleshooting) - Common issues and solutions
